/**
 * @modification $Date: 2004/06/14 19:33:33 $
 * @statut       unstable
 * @file         com/memoire/slaf/SlafAbstractTitlePane.java
 * @version      0.20
 * @author       Guillaume Desnoix
 * @email        guillaume-desnoix@memoire.com
 * @license      GNU General Public License 2 (GPL2)
 * @copyright    1999-2004 Guillaume Desnoix
 */

package com.memoire.slaf;

import java.awt.*;
import java.awt.event.*;
import java.awt.image.*;
import javax.swing.*;
import javax.swing.border.*;
import javax.swing.event.InternalFrameEvent;
import java.util.EventListener;
import java.beans.*;

public abstract class SlafAbstractTitlePane
  extends JComponent 
  implements LayoutManager, ActionListener, PropertyChangeListener
{
  //protected JRootPane  rp_;
  //protected Window     frame;
  protected JMenuBar   menuBar;
  //protected boolean    isPalette=false;

  // the three control buttons
  protected SlafInternalFrameTitleButton lineButton;
  protected SlafInternalFrameTitleButton iconButton;
  protected SlafInternalFrameTitleButton maxButton;
  protected SlafInternalFrameTitleButton closeButton;

  private Icon lineIcon; 
  private Icon maxIcon; 
  private Icon altMaxIcon; 
  private Icon iconIcon; 
  private Icon closeIcon;

  private int riseWidth         =0;	
  private int interButtonRight;  // 3
  private int interButtonHeight; // -1
  private int interButtonWidth;  // 3
  private int interIconLeft;     // 3
  private int interIconHeight;   // -1

  protected RolloverListener rolloverListener=null;
  protected boolean isRollover=false;
  protected boolean isPressed =false;

  private int previousHeight;

  private int   pos,incrPos;
  private Timer timer;

  protected SlafAbstractTitlePane()//JRootPane _rp,SlafRootPaneUI _ui,Window f)
  {
    //rp_=_rp;
    //frame=f;

    setFont(SlafManager.getFont("InternalFrameTitlePane.font"));
    setBorder(SlafManager.getBorder("InternalFrameTitlePane.border"));

    SlafUtils.forceOpaque
      (this,SlafManager.getBoolean("InternalFrameTitlePane.opaque",true));

    interIconLeft=SlafManager.getInteger
      ("InternalFrameTitlePane.interIconLeft",3);
    interIconHeight=SlafManager.getInteger
      ("InternalFrameTitlePane.interIconHeight",-1);
    interButtonRight=SlafManager.getInteger
      ("InternalFrameTitlePane.interButtonRight",3);
    interButtonWidth=SlafManager.getInteger
      ("InternalFrameTitlePane.interButtonWidth",3);
    interButtonHeight=SlafManager.getInteger
      ("InternalFrameTitlePane.interButtonHeight",-1);

    /*
    lineIcon  =SlafManager.getIcon("InternalFrameTitlePane.linifyIcon");
    maxIcon   =SlafManager.getIcon("InternalFrameTitlePane.maximizeIcon");
    altMaxIcon=SlafManager.getIcon("InternalFrameTitlePane.altMaximizeIcon");
    iconIcon  =SlafManager.getIcon("InternalFrameTitlePane.iconizeIcon");
    closeIcon =SlafManager.getIcon("InternalFrameTitlePane.closeIcon");
    */

    /*
    lineIcon  =createIcon("linifyIcon");
    maxIcon   =createIcon("maximizeIcon");
    altMaxIcon=createIcon("altMaximizeIcon");
    iconIcon  =createIcon("iconizeIcon");
    closeIcon =createIcon("closeIcon");
    */

    /*
      menuBar = new JMenuBar(){
      public boolean isFocusTraversable() { return false; }
      public void requestFocus() {}
      public void paint(Graphics g) {}
      public boolean isOpaque() { return false; }
      };
      menuBar.setBorderPainted(false);
    */

    /*
      boolean no_border=SlafManager.getBoolean
      ("InternalFrameTitlePane.borderHidden",true);
    */

    lineButton = new SlafInternalFrameTitleButton();
    setIcons(lineButton,"linifyIcon");
    // lineButton.putClientProperty("paintActive", Boolean.TRUE);
    // if(no_border) lineButton.setBorder(handyEmptyBorder);
    // lineButton.forceOpaque(!no_border);
    lineButton.setActionCommand("Linify");
    lineButton.addActionListener(this);

    iconButton = new SlafInternalFrameTitleButton();
    setIcons(iconButton,"iconizeIcon");
    // iconButton.putClientProperty("paintActive",Boolean.TRUE);
    // if(no_border) iconButton.setBorder(handyEmptyBorder);
    // iconButton.forceOpaque(!no_border);
    iconButton.addActionListener(this);
    iconButton.setActionCommand("Iconify");
    
    maxButton = new SlafInternalFrameTitleButton();
    setIcons(maxButton,"maximizeIcon");
    // maxButton.putClientProperty("paintActive", Boolean.TRUE);
    // if(no_border) maxButton.setBorder(handyEmptyBorder);
    // maxButton.forceOpaque(!no_border);
    maxButton.addActionListener(this);
    maxButton.setActionCommand("Maximize");

    closeButton = new SlafInternalFrameTitleButton();
    setIcons(closeButton,"closeIcon");
    // closeButton.putClientProperty("paintActive", Boolean.TRUE);
    // if(no_border) closeButton.setBorder(handyEmptyBorder);
    // closeButton.forceOpaque(!no_border);
    closeButton.addActionListener(this);
    closeButton.setActionCommand("Close");
    
    setLayout(this);
    
    // add(menuBar);
    add(lineButton);
    add(iconButton);
    add(maxButton);
    add(closeButton);             
    
    //rp_.addPropertyChangeListener(this);
    //frame.addPropertyChangeListener(this);

    if(SlafManager.getBoolean("InternalFrameTitlePane.rollover",true))
    {
      rolloverListener=new RolloverListener();
      // TMP addMouseListener(rolloverListener);
      lineButton .addMouseListener(rolloverListener);
      iconButton .addMouseListener(rolloverListener);
      maxButton  .addMouseListener(rolloverListener);
      closeButton.addMouseListener(rolloverListener);
    }

    addMouseListener(new MouseAdapter()
      {
        public void mouseClicked(MouseEvent _evt)
        {
          if((_evt.getSource()==SlafAbstractTitlePane.this)&&
             (_evt.getClickCount()==2))
            setMaximum(!isMaximum());
        }
      });
  }

  protected abstract Color     getBackground0();
  protected abstract Color     getForeground0();
  protected abstract String    getFrameTitle();
  protected abstract Icon      getFrameIcon();
  protected abstract Component getFrameComponent();

  public abstract boolean isEnabled();
  public abstract boolean isSelected();
  public abstract boolean isClosable();
  public abstract boolean isIconifiable();
  public abstract boolean isLinifiable();
  public abstract boolean isMaximizable();
  public abstract boolean isResizable();

  public abstract boolean isClosed();
  public abstract void    setClosed(boolean b);
  public abstract boolean isIcon();
  public abstract void    setIcon(boolean b);
  public abstract boolean isMaximum();
  public abstract void    setMaximum(boolean b);
  public abstract boolean isPalette();
  public abstract void    setPalette(boolean b);

  public final boolean shouldBeVisible()
  {
    return !isPalette()
      //||isLinifiable()
      ||isMaximizable()
      ||isClosable   ()
      ||isIconifiable();
  }

  private final void setIcons(SlafInternalFrameTitleButton _c,String _s)
  {
    String uk=_s.substring(0,1).toUpperCase()+_s.substring(1);

    String nk="InternalFrameTitlePane."+_s;
    String ik="InternalFrameTitlePane.inactive"+uk;
    String pk="InternalFrameTitlePane.pressed" +uk;
    String rk="InternalFrameTitlePane.rollover"+uk;

    Icon n=SlafManager.getIcon(nk);
    Icon i=SlafManager.getIcon(ik);
    Icon p=SlafManager.getIcon(pk);
    Icon r=SlafManager.getIcon(rk);

    _c.setIcon((n==null) ? null : ((i==null) ? n : new DynamicIcon(n,i)));
    if(p!=null) _c.setPressedIcon (p);
    if(r!=null) _c.setRolloverIcon(r);
  }

  private final class DynamicIcon implements Icon
  {
    private Icon n_,i_;

    public DynamicIcon(Icon _n, Icon _i)
    {
      n_=_n;
      i_=_i;
    }

    public int getIconWidth()
    { return n_.getIconWidth(); }

    public int getIconHeight()
    { return n_.getIconHeight(); }

    public void paintIcon(Component _c, Graphics _g, int _x, int _y)
    {
      if(isSelected()) n_.paintIcon(_c,_g,_x,_y);
      else             i_.paintIcon(_c,_g,_x,_y);
    }
  }

  protected final void paintBackground(Graphics g)
  {
    //System.err.println("WINDOW TITLE PANE OPAQUE="+isOpaque());

    if(isOpaque()||
       (SlafManager.getInteger("InternalFrameTitlePane.alpha",-1)!=-1))
    {
      Rectangle r=SlafUtils.getFullRect(this);
      int       t=SlafUtils.ENABLED;
      Color     c=null;

      if(isPalette())
	c=SlafManager.getColor("InternalFrameTitlePane.paletteBackground");

      if(c==null)
      {
        if(isSelected())
	  c=SlafManager.getColor("InternalFrameTitlePane.selectionBackground");
	else
	  c=SlafManager.getColor("InternalFrameTitlePane.background");
      }

      if(c==null)
      {
	c=SlafUtils.getBackground(this);//frame.getBackground();
             if(isPressed)    t=SlafUtils.PRESSED;
        else if(isSelected()) t=SlafUtils.SELECTED;
        else if(isRollover)   t=SlafUtils.ROLLOVER;
      }

      SlafUtils.paintBackground(g,r,this,c,t,"InternalFrameTitlePane");
    }
  }

  public final Color getBackground()
  {
    Color r=null;

    if(isPalette())
	r=SlafManager.getColor("InternalFrameTitlePane.paletteBackground");

    if(r==null)
    {
      r=getBackground0();

      Color s=SlafManager.getColor("InternalFrameTitlePane.selectionBackground");
      Color u=SlafManager.getColor("InternalFrameTitlePane.background");

      if(isSelected())
      {
        if(s!=null) r=s;
        else        r=SlafUtils.darken(r,2);
      }
      else
      {
        if(u!=null) r=u;
      }

           if(isPressed)  r=SlafUtils.darken(r,2);
      else if(isRollover) r=SlafUtils.brighten(r,1);
    }

    return r;
  }

  public final Color getForeground()
  {
    Color r=null;

    if(isPalette())
	r=SlafManager.getColor("InternalFrameTitlePane.paletteForeground");

    if(r==null)
    {
      r=getForeground0();

      Color s=SlafManager.getColor("InternalFrameTitlePane.selectionForeground");
      Color u=SlafManager.getColor("InternalFrameTitlePane.foreground");

      if(isSelected())
      {
        if(s!=null) r=s;
        else        r=SlafUtils.darken(r,2);
      }
      else
      {
        if(u!=null) r=u;
      }

           if(isPressed)  r=SlafUtils.darken(r,2);
      else if(isRollover) r=SlafUtils.brighten(r,1);
    }

    return r;
  }

  public final void paintComponent(Graphics g)
  {
    //System.err.println("PAINT SIZE="+getBounds());

    paintBackground(g);

    int width =getWidth();
    int height=getHeight();
    
    Color bg=getBackground();
    Color fg=getForeground();

    // if(isSelected) bg=bg.darker();
    // g.setColor(bg);
    // g.fill3DRect(0,0,width,height,!isSelected);
    // g.fillRect(0,0,width,height);

    /*
    int dx=width*2/3;
    int xmin=width-dx-1;
    Color b2=bg.darker(); // .darker();

    int rb1=bg.getRed  ();
    int gb1=bg.getGreen();
    int bb1=bg.getBlue ();
    int rb2=b2.getRed  ();
    int gb2=b2.getGreen();
    int bb2=b2.getBlue ();

    g.setColor(bg);
    g.fillRect(1,1,xmin,height-1);

    for(int x=0;x<dx;x++)
    {
      g.setColor(new Color(rb1+(rb2-rb1)*x/dx,
                           gb1+(gb2-gb1)*x/dx,
                           bb1+(bb2-bb1)*x/dx));
      g.drawLine(xmin+x,1,xmin+x,height-1);
    }
    */

    int     titleLength=0;
    Insets  insets     =getInsets();
    int     xOffset    =insets.left;
    int     paneH      =height-insets.top-insets.bottom;
    String  title      =getFrameTitle();
    Icon    icon       =getFrameIcon();
    boolean reduc      =false;
    int     iconH,iconW;

    if(!SlafManager.getBoolean("InternalFrame.iconVisible",true))
      icon=null;

    if(icon!=null)
    {
      iconH=icon.getIconHeight();
      iconW=icon.getIconWidth();

      reduc=(SlafManager.getBoolean("InternalFrame.iconReduced",false)
	&&(icon instanceof ImageIcon));

      if(reduc)
      {
	if(iconH>0) iconW=iconW*paneH/iconH;
	iconH=paneH;
      }
    }
    else
    {
      iconW=0;
      iconH=0;
    }

    if(title!=null)
    {
      Font ft=getFont();

      g.setFont(ft);

      FontMetrics fm=g.getFontMetrics();

      if(icon!=null) xOffset+=iconW+2;

      int yOffset=insets.top+(height-insets.top-insets.bottom
			      -fm.getHeight())/2+fm.getAscent();

      titleLength=fm.stringWidth(title);
      g.setColor(fg);

      int maxw=width-insets.right-xOffset-interButtonRight;

      boolean padding=SlafManager.getBoolean
        ("InternalFrameTitlePane.padding",false);

      //SlafUtils.trace("~~~~~ PADDING="+padding+" bw="+buttonWidth+" ibw="+interButtonWidth);
      if(         isLinifiable ()) maxw-=getButtonWidth(lineButton )+interButtonWidth;
      if(padding||isClosable   ()) maxw-=getButtonWidth(closeButton)+interButtonWidth;
      if(padding||isMaximizable()) maxw-=getButtonWidth(maxButton  )+interButtonWidth;
      if(padding||isIconifiable()) maxw-=getButtonWidth(iconButton )+interButtonWidth;

      if((titleLength<=maxw)||
         !isSelected()||
         !SlafManager.getBoolean("InternalFrameTitlePane.scrolling",true))
      {
	// SlafUtils.trace("~~~~~~ Short Title");
	Shape s=null;
	if(maxw>=0)
	{
	  s=g.getClip();
	  g.setClip(xOffset,0,maxw,height);
	}

        //g.setColor(Color.red);
        //g.drawRect(xOffset+1,1,maxw-2,height-3);

        int dx;
        switch(SlafManager.getInteger("InternalFrameTitlePane.textHorizontalAlignment",0))
        {
        case  1: dx=(maxw-titleLength)/2; break;
        case  2: dx=maxw-titleLength-3; break;
        default: dx=3;
        }

        //g.setColor(Color.green);
        //g.drawRect(dx+xOffset+1,2,titleLength-1,height-5);
        //g.setColor(fg);

	//g.drawString(title,xOffset,yOffset);
	paintText(g,title,dx+xOffset,yOffset);
	if(maxw>=0)
	{
	  g.setClip(s);
	}

	xOffset+=titleLength;
	if(timer!=null)
	{
	  // SlafUtils.trace("~~~~~~ Stop Timer");
	  timer.stop();
	  timer=null;
	}
      }
      else
      {
	// SlafUtils.trace("~~~~~~ Long Title");
	if(timer==null)
	{
	  // SlafUtils.trace("~~~~~~ Start Timer");
	  pos=0;
	  incrPos=3;
	  timer=new Timer(200,new ActionListener()
            {
	      public void actionPerformed(ActionEvent _evt)
	      { repaint(); }
            });
	  timer.setInitialDelay(1000);
	  timer.setCoalesce(true);
	  timer.setRepeats(true);
	  timer.start();
	}
	else
	{
	  if(pos<0) { pos=0; incrPos=Math.abs(incrPos); }
	  if(pos>titleLength-maxw) { pos=titleLength-maxw; incrPos=-Math.abs(incrPos); }
	  pos+=incrPos;
	  //SlafUtils.trace("~~~~~~ POS="+pos);
	}
	Shape s=g.getClip();
	g.setClip(xOffset,0,maxw,height);
	//SlafUtils.trace("~~~~~~ WCLIP="+(xOffset+titleLength-maxPos));
	//g.drawString(title,xOffset-pos,yOffset);
	paintText(g,title,xOffset-pos,yOffset);
	g.setClip(s);
      }

      setToolTipText((maxw>=5) ? title : null);

      if(icon!=null)
      {
	int iconY=insets.top;

	if(!reduc)
	{
	  if(interIconHeight>=0) iconY=interIconHeight;
	  else iconY+=(paneH-iconH)/2;
	  //if(iconY+iconH>=paneH) iconY=Math.max(0,paneH-iconH);
	  icon.paintIcon(getFrameComponent(),g,interIconLeft,iconY);
	}
	else
	if(iconH>0)
	{
	  Image image=((ImageIcon)icon).getImage().getScaledInstance
	    (iconW,iconH,Image.SCALE_SMOOTH);
	  icon=new ImageIcon
	    (SlafResource.processImage(image));
	  icon.paintIcon(getFrameComponent(),g,interIconLeft,iconY);
	}
      }
    }

    xOffset+=interButtonWidth;

    /*
    if(xOffset+height<width)
    {
      Color c=frame.getBackground();
      g.setColor(c.brighter());
      g.drawLine(xOffset,height,xOffset+height,0);
      g.setColor(c);
      g.drawLine(xOffset+1,height,xOffset+height+1,0);
      g.setColor(c.darker());
      g.drawLine(xOffset+2,height,xOffset+height+2,0);
    }
    */
  }

  private final int getButtonWidth(JButton _c)
  {
    Icon   i=_c.getIcon();
    int    w=(i==null) ? 0 : i.getIconWidth();
    int    h=(i==null) ? 0 : i.getIconWidth();
    Insets b=_c.getInsets();
    if(b!=null)
    {
      h+=b.top+b.bottom;
      w+=b.left+b.right;
      if(w<h) w=h;
    }

    if(interButtonHeight>=0)
    {
      int tph=getHeight();
      if(h<tph) h=tph-2*interButtonHeight;
      if(w<h) w=h;
    }
    //System.err.println("~~~~~ BW="+w);
    return w;
  }

  public final Point getToolTipLocation(MouseEvent _evt)
  {
    return new Point(0,getHeight());
  }

  protected final void paintText(Graphics _g, String _s, int _x, int _y)
  {
    if("shadow".equals(SlafManager.getString
		       ("InternalFrameTitlePane.textEffect")))
    {
      Color old=_g.getColor();
      _g.setColor(new Color(64,64,64));
      _g.drawString(_s,_x+1,_y+1);
      _g.setColor(old);
    }

    _g.drawString(_s,_x,_y);
  }

  public final void paintChildren(Graphics g)
  {
    /*
    int alpha=255;
    if(isSelected())
      alpha=SlafManager.getInteger("InternalFrameTitlePane.selectedAlpha",alpha);
    else
      alpha=SlafManager.getInteger("InternalFrameTitlePane.alpha",alpha);

    Object old=null;
    if(alpha<255)
    {
      old=((Graphics2D)g).getComposite();
      ((Graphics2D)g).setComposite
	(AlphaComposite.getInstance
	 (AlphaComposite.SRC_OVER,((float)alpha)/255.f));
    }
    */

    super.paintChildren(g);

    /*
    if(old!=null)
    {
      ((Graphics2D)g).setComposite((Composite)old);
    }
    */
  }
					     
  /*
  void postClosingEvent(Window frame)
  {
    InternalFrameEvent e = new InternalFrameEvent
      (frame, InternalFrameEvent.INTERNAL_FRAME_CLOSING);

    // Try posting event, unless there's a SecurityManager.
    if(Window.class.getClassLoader()==null)
    {
      try {
	Toolkit.getDefaultToolkit().getSystemEventQueue().postEvent(e);
	return;
      } catch (SecurityException se) {
	// Use dispatchEvent instead.
      }
    }
    frame.dispatchEvent(e);
  }
  */

  protected final void linify()
  {
    Component c=getFrameComponent();
    if(c==null) return;

    Dimension as=c.getSize();
    Dimension ps=c.getPreferredSize();
    Dimension ms=getMinimumFrameSize();

    /*
    if(as.height!=ms.height) c.setSize(as.width,ms.height);
    else                     c.setSize(as.width,ps.height);
    */

    if(as.height!=ms.height)
    {
      previousHeight=as.height;
      c.setSize(as.width,ms.height);
    }
    else
    {
      if(previousHeight<ms.height)
	previousHeight=ps.height;
      c.setSize(as.width,previousHeight);
    }
  }

  public final void actionPerformed(ActionEvent e)
  {
    if("Minimize".equals(e.getActionCommand()))
      SlafUtils.error
        ("InternalFrameTitlePane: Minimize command not handle");

    if("Linify".equals(e.getActionCommand())&&isResizable())
    {
      linify();
    }
    else
    if("Close".equals(e.getActionCommand())&&isClosable())
    {
      setClosed(true);
    }
    else
    if("Iconify".equals(e.getActionCommand())&&isIconifiable())
    {
      if(!isIcon()) setIcon(true);
      else          setIcon(false);

      ButtonModel model=iconButton.getModel();
      if(model!=null) model.setRollover(false);
    }
    /*
    else
    if("Minimize".equals(e.getActionCommand()) && frame.isMaximizable())
    {
      try { frame.setIcon(true); }
      catch(PropertyVetoException ex) { }
    }
    */
    else
    if("Maximize".equals(e.getActionCommand())&&isMaximizable())
    {
      if(!isMaximum()) setMaximum(true);
      else             setMaximum(false);
    }
    else
    if("Restore".equals(e.getActionCommand())&&isMaximizable()&&isMaximum())
    {
      setMaximum(false);
    }
    else
    if("Restore".equals(e.getActionCommand())&&isIconifiable()&&isIcon())
    {
      setIcon(false);
    }
  }
					     
  public final void propertyChange(PropertyChangeEvent evt)
  {
    String  name=(String)evt.getPropertyName();

    if(shouldBeVisible()&&(getHeight()!=0))
    {
      revalidate();
    }
    else
    if(!shouldBeVisible()&&(getHeight()==0))
    {
      revalidate();
    }
  }

  public final void updateMaxIcon()
  {
    if(isMaximum())
      setIcons(maxButton,"altMaximizeIcon");
    else
      setIcons(maxButton,"maximizeIcon");
  }
    
  public final void addLayoutComponent(String name, Component c)
  {
  }

  public final void removeLayoutComponent(Component c)
  {
  }   
 
  public final Dimension preferredLayoutSize(Container c)
  {
    return getPreferredSize(c);
  }

  public final Dimension getPreferredSize(Container c)
  {
    return new Dimension(c.getSize().width,computeHeight());
  }

  public final Dimension minimumLayoutSize(Container c)
  {
    return preferredLayoutSize(c);
  }
    
  protected abstract Dimension getMinimumFrameSize();

  protected abstract boolean isHidden();

  protected final int computeHeight()
  {
    if(isHidden())         return 0;
    if(!shouldBeVisible()) return 0;

    int fh=SlafManager.getInteger("InternalFrameTitlePane.height",-1);
    if(fh>=0) return fh;

    Insets insets=getInsets();
    String title =getFrameTitle();
    Icon   icon  =getFrameIcon();
    
    int iconHeight;
    boolean reduc=(SlafManager.getBoolean("InternalFrame.iconReduced",false)
		   &&(icon instanceof ImageIcon));
    iconHeight=(reduc||(icon==null) ? 0 : icon.getIconHeight());
    if(interIconHeight>=0) iconHeight+=interIconHeight;
    else                   iconHeight+=+insets.top+insets.bottom;

    int fontHeight  =0;
    int buttonHeight=0;

    if(isClosable()||isMaximizable()||
       isIconifiable()||isLinifiable())
      buttonHeight=closeButton.getIcon().getIconHeight();

    if((title!=null)&&!title.equals(""))
      fontHeight=Toolkit.getDefaultToolkit()
	.getFontMetrics(getFont()).getHeight();

    int finalHeight=Math.max(fontHeight,buttonHeight);
    if(finalHeight>0)
      finalHeight=Math.max(finalHeight+insets.top+insets.bottom,iconHeight);
    else
      finalHeight=insets.top+insets.bottom;

    //System.err.println("FINAL HEIGHT="+finalHeight);
    return finalHeight;
  }	

  public final void reshape(int _x,int _y,int _w,int _h)
  {
    if(SlafManager.getBoolean("InternalFrameTitlePane.short",false))
    {
      int pw=getShortWidth();
      if(_w>pw)	_w=pw;
    }

    super.reshape(_x,_y,_w,_h);
  }

  protected final int getShortWidth()
  {
    int h=getHeight();

    Insets insets=getInsets();

    int buttonHeight=closeButton.getIcon().getIconHeight();
    int buttonWidth =closeButton.getIcon().getIconWidth();

    Insets b=closeButton.getInsets();
    if(b!=null)
    {
      buttonHeight+=b.top+b.bottom;
      buttonWidth +=b.left+b.right;
      if(buttonWidth<buttonHeight) buttonWidth=buttonHeight;
    }

    if(interButtonHeight>=0)
    {
      if(buttonHeight<h)           buttonHeight=h-2*interButtonHeight;
      if(buttonWidth<buttonHeight) buttonWidth=buttonHeight;
    }

    int w=insets.left+insets.right+interButtonWidth;//-interButtonRight;
      
    if(isClosable())    w+=(buttonWidth+interButtonWidth);
    if(isMaximizable()) w+=(buttonWidth+interButtonWidth);
    if(isIconifiable()) w+=(buttonWidth+interButtonWidth);
    if(isLinifiable())  w+=(buttonWidth+interButtonWidth);

    String title=getFrameTitle();
    Icon   icon =getFrameIcon();

    if(icon!=null)
    {
      int iconH=icon.getIconHeight();
      int iconW=icon.getIconWidth();
      int paneH=h-insets.top-insets.bottom;

      boolean reduc=(SlafManager.getBoolean("InternalFrame.iconReduced",false)
		     &&(icon instanceof ImageIcon));
      if(reduc&&(iconH>0)) iconW=iconW*paneH/iconH;
      w+=iconW+2;
    }

    if(title!=null)
    {
      FontMetrics fm=getToolkit().getFontMetrics(getFont());
      w+=fm.stringWidth(title);
    }

    return w+2;
  }

  public final void layoutContainer(Container c)
  {
    int w=getWidth();
    int h=getHeight();

    Insets insets=getInsets();

    int buttonHeight=closeButton.getIcon().getIconHeight();
    int buttonWidth =closeButton.getIcon().getIconWidth();
    // int buttonHeight=closeButton.getPreferredSize().height;
    // int buttonWidth =closeButton.getPreferredSize().width;

    /*
      boolean no_border=SlafManager.getBoolean
      ("InternalFrameTitlePane.borderHidden",true);
    */
    Insets b=closeButton.getInsets();
    if(b!=null)
    {
      buttonHeight+=b.top+b.bottom;
      buttonWidth +=b.left+b.right;
      if(buttonWidth<buttonHeight) buttonWidth=buttonHeight;
    }

    if(interButtonHeight>=0)
    {
      if(buttonHeight<h)           buttonHeight=h-2*interButtonHeight;
      if(buttonWidth<buttonHeight) buttonWidth=buttonHeight;
    }

    int x=w+interButtonWidth-interButtonRight;
    int y=insets.top+(h-insets.top-insets.bottom-buttonHeight)/2;
      
    if(isClosable()) {
      x -= (buttonWidth+interButtonWidth);
      closeButton.setBounds(x,y,buttonWidth,buttonHeight);
    }
    /* else if(closeButton.getParent() != null)
      closeButton.getParent().remove(closeButton);
    */
    closeButton.setVisible(isClosable());
      
    if(isMaximizable()) {
      x -= (buttonWidth+interButtonWidth);
      maxButton.setBounds(x,y,buttonWidth,buttonHeight);
    }
    /* else if(maxButton.getParent() != null)
      maxButton.getParent().remove(maxButton);
    */
    maxButton.setVisible(isMaximizable());

    if(isIconifiable()) {
      x -= (buttonWidth+interButtonWidth);
      iconButton.setBounds(x,y,buttonWidth,buttonHeight);
    }
    /*
      else if(iconButton.getParent() != null)
      iconButton.getParent().remove(iconButton);
    */
    iconButton.setVisible(isIconifiable());

    if(isLinifiable())
    {
      x-=(buttonWidth+interButtonWidth);
      lineButton.setBounds(x,y,buttonWidth,buttonHeight);
    }
    lineButton.setVisible(isLinifiable());
      
    riseWidth=w-x;
  }

  private class RolloverListener extends MouseAdapter
  {
    public void mousePressed(MouseEvent e)
    {
      if(e.getSource()==SlafAbstractTitlePane.this)
      {
	isPressed=true;
	repaint();
      }
    }

    public void mouseReleased(MouseEvent e)
    {
      if(e.getSource()==SlafAbstractTitlePane.this)
      {
	isPressed=false;
	repaint();
      }
    }

    public void mouseEntered(MouseEvent e)
    {
      if(!isRollover)
      {
	isRollover=true;
	repaint();
      }
    }

    public void mouseExited(MouseEvent e)
    {
      if(isRollover)
      {
	isRollover=false;
	repaint();
      }
    }
  }
}
