/**
 * @modification $Date: 2004/01/26 18:17:33 $
 * @statut       unstable
 * @file         com/memoire/slaf/SlafButtonBorder.java
 * @version      0.20
 * @author       Guillaume Desnoix
 * @email        guillaume-desnoix@memoire.com
 * @license      GNU General Public License 2 (GPL2)
 * @copyright    1999-2004 Guillaume Desnoix
 */

package com.memoire.slaf;
import com.memoire.slaf.*;

import javax.swing.*;
import javax.swing.border.*;
import javax.swing.plaf.basic.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.plaf.*;

public class SlafButtonBorder extends AbstractBorder
{
  private String ui_;
  private Insets insets_;
  private Border enabled;
  private Border pressed;
  private Border disabled;
  private Border selected;
  private Border rollover;
  private Border rollseld;

  public SlafButtonBorder(String _ui)
    { ui_=_ui; }

  private void setBorders(Component _c)
  {
    if(insets_!=null) return;

    if(ui_==null)
    {
      ui_=((JComponent)_c).getUIClassID();
      if(ui_.endsWith("UI")) ui_=ui_.substring(0,ui_.length()-2);
    }

    if(enabled  ==null) enabled  =SlafManager.getBorder(ui_+".enabledBorder" );
    if(pressed  ==null) pressed  =SlafManager.getBorder(ui_+".pressedBorder" );
    if(disabled ==null) disabled =SlafManager.getBorder(ui_+".disabledBorder");
    if(selected ==null) selected =SlafManager.getBorder(ui_+".selectedBorder");
    if(rollover ==null) rollover =SlafManager.getBorder(ui_+".rolloverBorder");
    if(rollseld ==null) rollseld =SlafManager.getBorder
			  (ui_+".rolloverSelectedBorder");

    if(enabled  ==null) enabled  =new LineBorder(Color.green,3);
    if(pressed  ==null) pressed  =enabled;
    if(disabled ==null) disabled =enabled;
    if(selected ==null) selected =enabled;
    if(rollover ==null) rollover =enabled;
    if(rollseld ==null) rollseld =pressed;

    Insets ebi=enabled .getBorderInsets(_c);
    Insets pbi=pressed .getBorderInsets(_c);
    Insets dbi=disabled.getBorderInsets(_c);
    Insets sbi=selected.getBorderInsets(_c);
    Insets rbi=rollover.getBorderInsets(_c);
    Insets xbi=rollseld.getBorderInsets(_c);

    int ti=SlafUtils.max(ebi.top   ,pbi.top   ,dbi.top   ,sbi.top   ,rbi.top   ,xbi.top   );
    int bi=SlafUtils.max(ebi.bottom,pbi.bottom,dbi.bottom,sbi.bottom,rbi.bottom,xbi.bottom);
    int li=SlafUtils.max(ebi.left  ,pbi.left  ,dbi.left  ,sbi.left  ,rbi.left  ,xbi.left  );
    int ri=SlafUtils.max(ebi.right ,pbi.right ,dbi.right ,sbi.right ,rbi.right ,xbi.right );

    insets_=new Insets(ti,li,bi,ri);
  }

  public void paintBorder(Component c, Graphics g, int x, int y, int w, int h)
  {
    setBorders(c);

    if(!(c instanceof AbstractButton))
    {
      SlafUtils.error("c is not an AbstractButton [SlafButtonBorder.java]");
      SlafUtils.error("c is a "+c.getClass());
      return;
    }

    AbstractButton button=(AbstractButton)c;
    ButtonModel    model =button.getModel();

    if(  (button.getParent() instanceof JComponent)
       &&Boolean.TRUE.equals(((JComponent)button.getParent())
			     .getClientProperty("JToolBar.isRollover")))
    {
      if(model.isEnabled())
      {
	if(model.isPressed()&&model.isArmed())
	  pressed.paintBorder(c,g,x,y,w,h);
	else
	if(model.isRollover())
	{
	  if(model.isSelected())
	    rollseld.paintBorder(c,g,x,y,w,h);
	  else
	    rollover.paintBorder(c,g,x,y,w,h);
	}
	else
	if(model.isSelected())
	  selected.paintBorder(c,g,x,y,w,h);
	// else
	// enabled.paintBorder(c,g,x,y,w,h);
      }
      // else
      // disabled.paintBorder(c,g,x,y,w,h);
    }
    else
    if(model.isEnabled())
    {
      if(model.isPressed()&&model.isArmed())
	pressed.paintBorder(c,g,x,y,w,h);
      else
      if(model.isRollover())
      {
	if(model.isSelected())
	  rollseld.paintBorder(c,g,x,y,w,h);
	else
	  rollover.paintBorder(c,g,x,y,w,h);
      }
      else
      if(c instanceof JMenu)
      {
	if(model.isSelected())
	  selected.paintBorder(c,g,x,y,w,h);
	else
	  enabled.paintBorder(c,g,x,y,w,h);
      }
      else
      if(c instanceof JMenuItem)
      {
	if(model.isArmed())
	  selected.paintBorder(c,g,x,y,w,h);
	else
	  enabled.paintBorder(c,g,x,y,w,h);
      }
      else
      if(model.isSelected())
	selected.paintBorder(c,g,x,y,w,h);
      else
	enabled.paintBorder(c,g,x,y,w,h);
    }
    else
      disabled.paintBorder(c,g,x,y,w,h);
  }

  public Insets getBorderInsets(Component _c)
  {
    return getBorderInsets(_c,new Insets(0,0,0,0));
  }

  public Insets getBorderInsets(Component _c, Insets _insets)
  {
    setBorders(_c);

    _insets.top   =insets_.top;
    _insets.left  =insets_.left;
    _insets.bottom=insets_.bottom;
    _insets.right =insets_.right;

    return _insets;
  }
}
