/**
 * @modification $Date: 2004/01/26 18:17:33 $
 * @statut       unstable
 * @file         com/memoire/slaf/SlafColorChooserUI.java
 * @version      0.20
 * @author       Guillaume Desnoix
 * @email        guillaume-desnoix@memoire.com
 * @license      GNU General Public License 2 (GPL2)
 * @copyright    1999-2004 Guillaume Desnoix
 */

package com.memoire.slaf;
import com.memoire.slaf.*;

import java.util.*;
import java.awt.*;
import java.awt.image.*;
import java.awt.event.*;
import java.beans.*;
import java.io.Serializable;

import javax.swing.*;
import javax.swing.colorchooser.*;
import javax.swing.event.*;
import javax.swing.border.*;
import javax.swing.plaf.*;
import javax.swing.plaf.basic.*;

public class SlafColorChooserUI
  //extends ColorChooserUI
  extends BasicColorChooserUI
{
  public static ComponentUI createUI(JComponent c)
  {
    return new SlafColorChooserUI();
  }

  public void installUI(JComponent _c)
  {
    super.installUI(_c);
    LookAndFeel.installBorder(_c,"ColorChooser.border");
  }

  public void uninstallDefaults(JComponent _c)
  {
    //LookAndFeel.uninstallBorder(_c,"ColorChooser.border");
    super.uninstallUI(_c);
  }

  /*
  JColorChooser chooser;

  JTabbedPane tabbedPane;
  JPanel singlePanel;

  JPanel previewPanelHolder;
  JComponent previewPanel;
  boolean isMultiPanel = false;

  protected AbstractColorChooserPanel[] defaultChoosers;

  protected ChangeListener previewListener;
  protected PropertyChangeListener propertyChangeListener;

  protected AbstractColorChooserPanel[] createDefaultChoosers()
  {
    AbstractColorChooserPanel[] panels=
      ColorChooserComponentFactory.getDefaultChooserPanels();
    return panels;
  }

  protected void uninstallDefaultChoosers()
  {
    for(int i=0; i<defaultChoosers.length; i++)
      chooser.removeChooserPanel(defaultChoosers[i]);
  }

  public void installUI( JComponent c )
  {
    chooser = (JColorChooser)c;

    super.installUI( c );

    installDefaults();
    installListeners();

    tabbedPane = new JTabbedPane();
    singlePanel = new JPanel();

    //tabbedPane.setTabPlacement(JTabbedPane.LEFT);
    singlePanel.setLayout(new BorderLayout());

    chooser.setLayout(new BorderLayout());

    defaultChoosers = createDefaultChoosers();
    chooser.setChooserPanels(defaultChoosers);

    installPreviewPanel();
  }

  public void uninstallUI( JComponent c )
  {
    uninstallListeners();
    uninstallDefaultChoosers();
    uninstallDefaults();

    defaultChoosers = null;
    chooser = null;
    tabbedPane = null;
  }

  protected void installPreviewPanel()
  {
    previewPanelHolder = new JPanel(new BorderLayout());
    String previewString = SlafManager.getString("ColorChooser.previewText");
    previewPanelHolder.setBorder(new TitledBorder(previewString));

    previewPanel = chooser.getPreviewPanel();
    if((previewPanel==null)||(previewPanel instanceof UIResource))
      previewPanel=ColorChooserComponentFactory.getPreviewPanel();

    previewPanel.setForeground(chooser.getColor());
    previewPanelHolder.add(previewPanel);
    chooser.add(previewPanelHolder, BorderLayout.SOUTH);
  }

  protected void installDefaults()
  {
    LookAndFeel.installColorsAndFont(chooser,
				     "ColorChooser.background", 
				     "ColorChooser.foreground",
				     "ColorChooser.font");
    LookAndFeel.installBorder(chooser,"ColorChooser.border");
  }

  protected void uninstallDefaults()
  {
  }
    
  protected void installListeners()
  {
    propertyChangeListener = createPropertyChangeListener();
    chooser.addPropertyChangeListener( propertyChangeListener );

    previewListener = new PreviewListener();
    chooser.getSelectionModel().addChangeListener(previewListener);
  }

  protected PropertyChangeListener createPropertyChangeListener()
  {
    return new PropertyHandler();
  }
  
  protected void uninstallListeners()
  {
    chooser.removePropertyChangeListener( propertyChangeListener );
    chooser.getSelectionModel().removeChangeListener(previewListener);
  }

  class PreviewListener implements ChangeListener
  {
    public void stateChanged( ChangeEvent e )
    {
      ColorSelectionModel model = (ColorSelectionModel)e.getSource();
      if (previewPanel != null)
      {
	previewPanel.setForeground(model.getSelectedColor());
	previewPanel.repaint();
      }
    }
  }

  public class PropertyHandler implements PropertyChangeListener
  {
    public void propertyChange(PropertyChangeEvent e)
    {
      if(e.getPropertyName().equals(JColorChooser.CHOOSER_PANELS_PROPERTY))
      {
	AbstractColorChooserPanel[] oldPanels=
	  (AbstractColorChooserPanel[])e.getOldValue();
	AbstractColorChooserPanel[] newPanels=
	  (AbstractColorChooserPanel[]) e.getNewValue();

	for(int i=0; i<oldPanels.length; i++)
	{
	  // remove old panels		  
	  Container wrapper = oldPanels[i].getParent();
	  if (wrapper != null) {
	    //Container parent = wrapper.getParent();
	    //if (parent != null)
	    //parent.remove(wrapper);  // remove from hierarchy
	    wrapper.remove(oldPanels[i]); // @GDX
	    oldPanels[i].uninstallChooserPanel(chooser); // uninstall
	  }
	}
	
	int numNewPanels = newPanels.length;
	if (numNewPanels == 0) {  // removed all panels and added none
	  chooser.remove(tabbedPane);
	  return;
	} 
	else if (numNewPanels == 1) {  // one panel case
	  chooser.remove(tabbedPane);
	  // JPanel centerWrapper = new JPanel(new BorderLayout());
	  // centerWrapper.add(newPanels[0],BorderLayout.CENTER);
	  // singlePanel.add(centerWrapper,BorderLayout.CENTER);
	  singlePanel.add(newPanels[0],BorderLayout.CENTER);
	  chooser.add(singlePanel);
	}
	else {   // multi-panel case
	  if ( oldPanels.length < 2 ) {// moving from single to multiple
	    chooser.remove(singlePanel);
	    tabbedPane = new JTabbedPane();
	    //tabbedPane.setTabPlacement(JTabbedPane.LEFT);
	    chooser.add(tabbedPane, BorderLayout.CENTER);
	  }
	  
	  for (int i = 0; i < newPanels.length; i++) {	    
	    // JPanel centerWrapper = new JPanel(new BorderLayout());
	    // centerWrapper.add(newPanels[i],BorderLayout.CENTER);
	    // tabbedPane.addTab(newPanels[i].getDisplayName(), centerWrapper);
	    tabbedPane.addTab(newPanels[i].getDisplayName(),newPanels[i]);
	  }
	}
	
	for(int i=0; i<newPanels.length; i++)
	  newPanels[i].installChooserPanel(chooser);
      }
      
      if(e.getPropertyName().equals(JColorChooser.PREVIEW_PANEL_PROPERTY))
      {
	JComponent oldPanel = (JComponent) e.getOldValue();
	JComponent newPanel = (JComponent) e.getNewValue();
	if(oldPanel!=null) chooser.remove(oldPanel);
	chooser.add(newPanel, BorderLayout.SOUTH);
      }
    }
  }
  */
}

