/**
 * @modification $Date: 2004/01/26 18:17:33 $
 * @statut       unstable
 * @file         com/memoire/slaf/SlafCompoundBorder.java
 * @version      0.20
 * @author       Guillaume Desnoix
 * @email        guillaume-desnoix@memoire.com
 * @license      GNU General Public License 2 (GPL2)
 * @copyright    1999-2004 Guillaume Desnoix
 */

package com.memoire.slaf;
import com.memoire.slaf.*;

import javax.swing.*;
import javax.swing.border.*;
import java.awt.*;

public class SlafCompoundBorder extends AbstractBorder
{
  protected Border  outside_;
  protected Border  inside_;
  protected boolean reverse_;

  public SlafCompoundBorder()
  {
    this(null,null,false);
  }	    

  public SlafCompoundBorder(Border _outside, Border _inside)
  {
    this(_outside,_inside,false);
  }

  public SlafCompoundBorder(Border _outside, Border _inside, boolean _reverse)
  {
    outside_=_outside;
    inside_ =_inside;
    reverse_=_reverse;
  }

  public boolean isBorderOpaque()
  { 
    return ((outside_!=null)&&outside_.isBorderOpaque())
	&& ((inside_ !=null)&&inside_ .isBorderOpaque()); 
  }

  public void paintBorder(Component _c,Graphics _g,int _x,int _y,int _w,int _h)
  {
    Insets insets;
    int    px,py,pw,ph;

    px=_x;
    py=_y;
    pw=_w;
    ph=_h;

    if(outside_!=null)
    {
      insets=outside_.getBorderInsets(_c);
      px+=insets.left;
      py+=insets.top;
      pw=pw-insets.right -insets.left;
      ph=ph-insets.bottom-insets.top;
    }

    if(reverse_)
    {
      if(inside_ !=null) inside_ .paintBorder(_c,_g,px,py,pw,ph);
      if(outside_!=null) outside_.paintBorder(_c,_g,_x,_y,_w,_h);
    }
    else
    {
      if(outside_!=null) outside_.paintBorder(_c,_g,_x,_y,_w,_h);
      if(inside_ !=null) inside_ .paintBorder(_c,_g,px,py,pw,ph);
    }
  }
     
  public Insets getBorderInsets(Component _c, Insets _insets)
  {
    Insets i;

    _insets.top=_insets.left=_insets.right=_insets.bottom=0;

    if(outside_!=null)
    {
      i=outside_.getBorderInsets(_c);
      _insets.top   +=i.top;
      _insets.left  +=i.left;
      _insets.right +=i.right;
      _insets.bottom+=i.bottom;
    }
    if(inside_!=null)
    {
      i=inside_.getBorderInsets(_c);
      _insets.top   +=i.top;
      _insets.left  +=i.left;
      _insets.right +=i.right;
      _insets.bottom+=i.bottom;
    }

    return _insets;
  }

  public Insets getBorderInsets(Component _c)
  {
    return getBorderInsets(_c,new Insets(0,0,0,0));
  }

  public Border getOutsideBorder()
  {
    return outside_;
  }

  public Border getInsideBorder()
  {
    return inside_;
  }
}

