/**
 * @modification $Date: 2004/06/11 19:23:24 $
 * @statut       unstable
 * @file         com/memoire/slaf/SlafCustomTheme.java
 * @version      0.20
 * @author       Guillaume Desnoix
 * @email        guillaume-desnoix@memoire.com
 * @license      GNU General Public License 2 (GPL2)
 * @copyright    1999-2004 Guillaume Desnoix
 */

package com.memoire.slaf;
import com.memoire.slaf.*;

import java.awt.*;
import java.awt.image.*;
import java.io.*;
import java.net.*;
import java.util.*;
import java.util.zip.*;

import javax.swing.*;
import javax.swing.border.*;
import javax.swing.plaf.*;
import javax.swing.plaf.metal.*;

/*
 * TODO: replace UIMnaager.put par resources_ ?
 */
public final class SlafCustomTheme extends SlafDefaultTheme
{
  private ClassLoader     classloader_;
  private Properties      props_;
  private Hashtable       icons_;
  private String          base_;
  private String          name_;

  public SlafCustomTheme()
  {
    this(null);
  }

  public SlafCustomTheme(String _name)
  {
    super();

    name_=_name;
    if(!"default".equals(_name))
    {
      icons_=new Hashtable();
      base_ ="";
      loadThemeFile(_name);
      icons_.clear();
      icons_=null;
      base_ =null;
    }
  }

  private void initClassLoader()
  {
    classloader_=(ClassLoader)UIManager.getLookAndFeelDefaults()
	.get("ClassLoader");
    //if(classloader_!=null)
    //  SlafUtils.trace("Using SlafManager.class loader: "+classloader_);
    if(classloader_==null)
    {
      classloader_=getClass().getClassLoader();
      //SlafUtils.trace("Using default class loader: "+classloader_);
    }
    if((classloader_==null)&&(SlafUtils.jdk()>=1.2))
    {
      classloader_=ClassLoader.getSystemClassLoader();
    }
  }

  protected synchronized boolean loadProperties(String _file)
  {
    boolean r=false;

    try
    {
      SlafUtils.trace("try .slafrc as "+_file);

      InputStream in;
      if(_file.startsWith("slaf:"))
      {
	initClassLoader();
	in=classloader_/*getClass()*/
	    .getResourceAsStream(_file.substring(5));
      }
      else
      if(_file.startsWith("jar:"))
      {
	int i=_file.indexOf("!/");
	//SlafUtils.trace("JAR FILE ="+_file.substring(4,i));
	//SlafUtils.trace("JAR ENTRY="+_file.substring(i+2));
	ZipFile  zip  =new ZipFile(_file.substring(4,i));
	ZipEntry entry=zip.getEntry(_file.substring(i+2));
	in=zip.getInputStream(entry);
      }
      else
      if(_file.startsWith("http:")||_file.startsWith("ftp:"))
      {
	System.err.println("SLF: connecting remote theme server");
	in=new URL(_file).openStream();
      }
      else
      if(_file.startsWith("file:"))
      {
	in=new URL(_file).openStream();
      }
      else
	in=new FileInputStream(_file);

      if(in!=null)
      {
	props_.load(in);
	r=true;
      }
    }
    catch(FileNotFoundException ex)
    {
    }
    catch(ZipException ex)
    {
    }
    catch(Exception ex)
    {
      SlafUtils.error(_file+" -- "+ex);
    }

    return r;
  }

  protected synchronized void loadThemeFile(String _name)
  {
    if(_name!=null) _name="_"+_name;
    else            _name="";
    SlafUtils.trace("load custom theme: "+_name);

    Properties old=props_;
    props_=new Properties();

    String  f;
    boolean ok   =false;
    String  slash=System.getProperty("file.separator");

    try
    {
      if(!ok)
      {
        base_=System.getProperty("user.home")+slash;
	f=base_+".slafrc"+_name;
	base_=base_+".slaf"+slash+"icons"+slash;
	ok=loadProperties(f);
      }
    }
    catch(SecurityException ex) { }

    try
    {
      if(!ok)
      {
        base_=System.getProperty("user.dir")+slash;
	f=base_+".slafrc"+_name;
	base_=base_+".slaf"+slash+"icons"+slash;
	ok=loadProperties(f);
      }
    }
    catch(SecurityException ex) { }

    if(!"".equals(_name))
    {
      if(!ok)
      {
	base_="slaf:com/memoire/slaf/themes/";
	f=base_+"rc"+_name+".txt";
	base_=base_+"icons/";
	ok=loadProperties(f);
      }

      try
      {
        if(!ok)
        {
	  base_=System.getProperty("slaf.server");
	  f=base_+"slafrc"+_name;
	  base_=base_+"icons/";
	  ok=loadProperties(f);
        }
      }
      catch(SecurityException ex) { }

      try
      {
	if(!ok)
        {
	  base_=slash+"usr"+slash+"share"+slash+"slaf"+slash;
	  f=base_+"slafrc"+_name;
	  base_=base_+"icons"+slash;
	  ok=loadProperties(f);
	}
      }
      catch(SecurityException ex) { }

      try
      {
        if(!ok)
        {
	  base_="jar:"+System.getProperty("user.home")+slash+
	      _name.substring(1)+".slaf!/";
	  f=base_+_name.substring(1)+".rc";
	  ok=loadProperties(f);
	}
      }
      catch(SecurityException ex) { }

      try
      {
        if(!ok)
        {
	  base_="jar:"+System.getProperty("user.dir")+slash+
	      _name.substring(1)+".slaf!/";
	  f=base_+_name.substring(1)+".rc";
	  ok=loadProperties(f);
	}
      }
      catch(SecurityException ex) { }

      try
      {
        if(!ok)
        {
	  base_="jar:"+slash+"usr"+slash+"share"+slash+"slaf"+slash+
	      _name.substring(1)+".slaf!/";
	  f=base_+_name.substring(1)+".rc";
	  ok=loadProperties(f);
	}
      }
      catch(SecurityException ex) { }

      try
      {
        if(!ok)
        {
	  base_="http://www.memoire.com/guillaume-desnoix/slaf/theme/";
	  f=base_+"slafrc"+_name;
	  base_=base_+"icons/";
	  ok=loadProperties(f);
        }
      }
      catch(SecurityException ex) { }
    }

    if(!ok) return;

    // System.getProperties().list(System.out);

    String tps=props_.getProperty("Theme.inherit",null);
    if(tps!=null)
    {
      int i=0;
      do
      {
	i=tps.indexOf(',');
	if(i<0) { loadThemeFile(tps); tps=""; }
	else    { loadThemeFile(tps.substring(0,i)); tps=tps.substring(i+1); }
      } while(!"".equals(tps));
    }

    Boolean ro=(Boolean)readProperty("Theme.rollover",null);
    if(ro!=null) UIManager.put("Theme.rollover",ro);

    Border tb=(Border)readProperty("Theme.border",null);
    if(tb!=null) UIManager.put("Theme.border",tb);

    Color fc=(Color)readProperty("Theme.focus",null);
    if(fc!=null) UIManager.put("Theme.focus",fc);
    Integer fr=(Integer)readProperty("Theme.focusRect",null);
    if(fr!=null) UIManager.put("Theme.focusRect",fr);
    Border fb=(Border)readProperty("Theme.focusBorder",null);
    if(fb!=null) UIManager.put("Theme.focusBorder",fb);

    // Filter

    Boolean aif=(Boolean)readProperty("Theme.applyIconFilter",null);
    if(aif!=null) UIManager.put("Theme.applyIconFilter",aif);

    //SlafIconFactory.reinit();
    ImageFilter tif;
    String      tif_name;

    String[] tif_names=new String[]
    {
      "color","enabledColor","disabledColor",
      "icon","enabledIcon","pressedIcon",
      "rolloverIcon","selectedIcon","disabledIcon"
    };

    for(int i=0;i<tif_names.length;i++)
    {
      tif_name="Theme."+tif_names[i]+"Filter";
      tif=readFilter(tif_name,null);
      if(tif!=null) UIManager.put(tif_name,tif);
      else          UIManager.getDefaults().remove(tif_name);
    }

    // Very very generic (*.prop)

    for(Enumeration e1=props_.keys(); e1.hasMoreElements(); )
    {
      String s=e1.nextElement().toString();

      if(!s.startsWith("*.")) continue;
      String fin=s.substring(1,s.length());

      Object o=readProperty(s,null);

      if(o!=null)
      {
	SlafUtils.trace("very generic: "+s);
	for(Enumeration e2=resources_.keys(); e2.hasMoreElements(); )
	{
	  String t=e2.nextElement().toString();
	  // if(t.indexOf('*')>=0) continue;
	  
	  if(t.endsWith(fin))
	  {
	    // SlafUtils.trace("set: "+t+" with "+s);
	    resources_.put(t,o);
	  }
	}
      }
      else
	SlafUtils.trace("unreconized very generic property: "+s);
    }

    // Very generic (*Type.prop)

    for(Enumeration e1=props_.keys(); e1.hasMoreElements(); )
    {
      String s=e1.nextElement().toString();
      if(s.startsWith("*.")) continue;

      int    i=s.indexOf('*');
      if(i!=0) continue;
      String fin=s.substring(i+1,s.length());

      Object o=readProperty(s,null);

      if(o!=null)
      {
	SlafUtils.trace("very generic: "+s);
	for(Enumeration e2=resources_.keys(); e2.hasMoreElements(); )
	{
	  String t=e2.nextElement().toString();
	  // if(t.indexOf('*')>=0) continue;
	  
	  if(t.endsWith(fin))
	  {
	    // SlafUtils.trace("set: "+t+" with "+s);
	    resources_.put(t,o);
	  }
	}
      }
      else
	SlafUtils.trace("unreconized very generic property: "+s);
    }

    // Generic (Type*.prop)

    for(Enumeration e1=props_.keys(); e1.hasMoreElements(); )
    {
      String s=e1.nextElement().toString();
      int    i=s.indexOf('*');

      if(i<=0) continue;
      String debut=s.substring(0,i);
      String fin  =s.substring(i+1,s.length());

      Object o=readProperty(s,null);

      if(o!=null)
      {
	SlafUtils.trace("generic: "+s);
	for(Enumeration e2=resources_.keys(); e2.hasMoreElements(); )
	{
	  String t=e2.nextElement().toString();
	  // if(t.indexOf('*')>=0) continue;
	  
	  if(t.startsWith(debut)&&t.endsWith(fin))
	  {
	    // SlafUtils.trace("set: "+t+" with "+s);
	    resources_.put(t,o);
	  }
	}
      }
      else
	SlafUtils.trace("unreconized generic property: "+s);
    }


    // Qualified

    for(Enumeration e=props_.keys(); e.hasMoreElements(); )
    {
      String s=e.nextElement().toString();
      if(s.indexOf('*')>=0)     continue;
      if(s.startsWith("Theme")) continue;

      if((s.charAt(0)>='A')&&(s.charAt(0)<='Z'))
      {
	Object o=readProperty(s,null);

	if(o!=null)
	  resources_.put(s,o);
	else
	  SlafUtils.trace("unreconized custom property: "+s);
      }
    }

    props_.clear();
    props_=old;

    SlafUtils.trace("end of custom theme: "+_name);
  }

  // Property

  private Object readProperty(String _s, UIResource _d)
  {
    Object o=null;
    String k=_s;
    String v=props_.getProperty(k);

    String l=_s.toLowerCase();
    //if(l.endsWith("inactive")) l=l.substring(0,l.length()-8);

    if("null".equals(v))
      o=NULL;
    else
    if(  l.endsWith("foreground")
       ||l.endsWith("background")
       ||l.endsWith("color"     )
       ||l.endsWith("shadow"    )
       ||l.endsWith("highlight" )
       ||l.endsWith("focus"     )
       ||l.equals("tree.line")
       ||l.equals("tree.hash"))
      o=readColor(k,null);
    else
    /*
      if(l.endsWith("size"))
      o=readSize(k,null);
      else
    */
    if(  l.endsWith("margin")
       ||l.endsWith("insets"))
      o=readInsets(k,null);
    else
    if(l.endsWith("font"))
      o=readFont(k,null);
    else
    if(l.endsWith("border"))
      o=readBorder(k,null);
    else
    if(l.endsWith("icon"))
      o=readIcon(k,null);
    else
    if(l.endsWith("texture"))
      o=readImage(k,null);
    else
    if(v!=null)
    {
      if(v.equals("true")||v.equals("false"))
	o=new Boolean(v);
      else
      if(((v.length()>0)&&Character.isDigit(v.charAt(0)))||
	 ((v.length()>1)&&Character.isDigit(v.charAt(1))&&
	  ((v.charAt(0)=='-')||(v.charAt(0)=='-'))))
      {
	// o=new Integer(v);
	if(v.indexOf(',')<0) o=new Integer(v);
	else                 o=readSize(k,null);
      }
      else
      if((v.length()>1)&&(v.charAt(0)=='"')&&(v.charAt(v.length()-1)=='"'))
	o=v.substring(1,v.length()-1);
      else
	SlafUtils.error("ignored property "+k+"="+v);
    }

    return o;
  }

  // Color

  private int decodeColor(String s) // throws Exception
  {
    int r=0;

    if(s.startsWith("#"))
    {
      s=s.substring(1);
      r=(int)Long.parseLong(s,16);
    }
    else
      r=Integer.parseInt(s,10);

    return r;
  }

  private ColorUIResource readColor(String _s, ColorUIResource _d)
  {
    ColorUIResource r=_d;

    String s=props_.getProperty(_s);
    if(s!=null)
    {
      try { r=new ColorUIResource(decodeColor(s)); }
      catch(Exception ex) { SlafUtils.error("unreconized color "+_s); }
    }

    return r;
  }

  private DimensionUIResource readSize(String _s, DimensionUIResource _d)
  {
    DimensionUIResource r=_d;

    String s=props_.getProperty(_s);
    if(s!=null)
    {
      try
      {
	int i=s.indexOf(',');
	int w=Integer.parseInt(s.substring(0,i));
	int h=Integer.parseInt(s.substring(i+1));
	r=new DimensionUIResource(w,h);
      }
      catch(Exception ex) { SlafUtils.error("unreconized size "+_s); }
    }

    return r;
  }

  private InsetsUIResource readInsets(String _s, InsetsUIResource _d)
  {
    InsetsUIResource r=_d;

    String s=props_.getProperty(_s);
    if(s!=null)
    {
      try
      {
	int i,top,left,bottom,right;
	i=s.indexOf(',');
	top=Integer.parseInt(s.substring(0,i));
	s=s.substring(i+1);
	i=s.indexOf(',');
	left=Integer.parseInt(s.substring(0,i));
	s=s.substring(i+1);
	i=s.indexOf(',');
	bottom=Integer.parseInt(s.substring(0,i));
	s=s.substring(i+1);
	right=Integer.parseInt(s);

	r=new InsetsUIResource(top,left,bottom,right);
	//SlafUtils.trace("####### INSETS="+r);
      }
      catch(Exception ex) { SlafUtils.error("unreconized insets "+_s); }
    }

    return r;
  }

  // Font

  private FontUIResource readFont(String _s, FontUIResource _d)
  {
    FontUIResource r=_d;

    String s=props_.getProperty(_s);
    if(s!=null)
    {
      try
      {
        /*
	GraphicsEnvironment ge=
	    GraphicsEnvironment.getLocalGraphicsEnvironment();
	Font[] ft=ge.getAllFonts();
	for(int i=0;i<ft.length;i++)
	    System.err.println(ft[i]);
	*/

	s=s.replace('-',',');

	// SlafUtils.trace("S="+s);
	String ff=s.substring(0,s.indexOf(','));
	String fs=s.substring(s.indexOf(',')+1,s.lastIndexOf(','));
	String fp=s.substring(s.lastIndexOf(',')+1,s.length());
	// SlafUtils.trace("X="+ff+","+fs+","+fp);

	int ifs=Font.PLAIN;
	fs=fs.toLowerCase();
	if(fs.equals("bold"  ))     ifs=Font.BOLD;
	if(fs.equals("italic"))     ifs=Font.ITALIC;
	if(fs.equals("bolditalic")) ifs=Font.BOLD|Font.ITALIC;

	int ifp=Integer.parseInt(fp);

	/*
	try
	{
	  GraphicsEnvironment ge=GraphicsEnvironment.
	    getLocalGraphicsEnvironment();
	  String[] fonts=ge.getAvailableFontFamilyNames();
	  //ltFamille_.setListData(fonts);
	  for(int i=0;i<fonts.length;i++)
	  {
	    System.err.print(fonts[i]+" ");
	    if(fonts[i].equals(ff))
	      System.err.print("<FOUND> ");
	  }
	}
	catch(NoClassDefFoundError ex) { }
	*/

	Font f=new Font(ff,ifs,ifp);
	r=new FontUIResource(f);
      }
      catch(Exception ex) { SlafUtils.error("unreconized font "+_s); }
    }

    return r;
  }

  // Border

  private BorderUIResource readBorder(String _s, BorderUIResource _d)
  {
    BorderUIResource r=_d;

    String s=props_.getProperty(_s);
    if(s!=null)
    {
      try { r=new BorderUIResource(decodeBorder(s)); }
      catch(Exception ex) { SlafUtils.error("unreconized border "+_s); }
    }

    return r;
  }

  private Border decodeBorder(String s)
  {
    Border r=null;

        // SlafUtils.trace("S="+s);
        int i=s.lastIndexOf('&');

	if(i>=0)
	{
	  String inside =s.substring(i+1);
	  String outside=s.substring(0,i);
	  r=new CompoundBorder
	    (decodeBorder(outside),decodeBorder(inside));
	}
	else
	if(s.equals("rootpane")) r=new SlafRootPaneBorder(null);
	else
	if(s.equals("titlepane")) r=new SlafInternalFrameTitlePaneBorder();
	else
	if(s.equals("toolbar")) r=new SlafToolBarBorder();
	else
	if(s.startsWith("component"))
	{
	  i=s.indexOf(',');
	  s=s.substring(i+1);
	  r=new SlafComponentBorder(s);
	}
	else
	if(s.startsWith("frame"))
	{
	  i=s.indexOf(',');
	  s=s.substring(i+1);
	  r=new SlafFrameBorder(s);
	}
	else
	if(s.startsWith("button"))
	{
	  i=s.indexOf(',');
	  s=s.substring(i+1);
	  r=new SlafButtonBorder(s);
	}
	else
	if(s.startsWith("matte"))
	{
	  i=s.indexOf(',');
	  // String btype=s.substring(0,i); == matte
	  s=s.substring(i+1);

	  i=s.indexOf(',');
	  String bthickness=s.substring(0,i);
	  s=s.substring(i+1);

	  String bicon=s;

	  int  ithickness=Integer.parseInt(bthickness);
	  Icon iicon=loadImageIcon(bicon);

	  r=new MatteBorder(ithickness,ithickness,
			    ithickness,ithickness,
			    iicon);
	}
	else
	if(s.startsWith("line"))
	{
	  i=s.indexOf(',');
	  // String btype=s.substring(0,i); == line
	  s=s.substring(i+1);

	  i=s.indexOf(',');
	  String bthickness=s.substring(0,i);
	  s=s.substring(i+1);

	  int ithickness=Integer.parseInt(bthickness);

	  Color icolor=null;
	  try { icolor=new Color(decodeColor(s)); }
	  catch(Exception ex) { icolor=Color.black; }

	  r=new SlafLineBorder(icolor,ithickness);
	}
	else
	if(s.startsWith("xline"))
	{
	  i=s.indexOf(',');
	  // String btype=s.substring(0,i); == xline
	  s=s.substring(i+1);

	  int[] v=new int[8];

	  for(int j=0;j<8;j++)
	  {
	    i=s.indexOf(',');
	    v[j]=Integer.parseInt(s.substring(0,i));
	    s=s.substring(i+1);
	  }


	  Color icolor=null;
	  int   iline=SlafLineBorder.SOLID;

	  i=s.indexOf(',');
	  if(i>=0)
	  {
	    if("dot".equals(s.substring(i+1))) iline=SlafLineBorder.DOT;
	    s=s.substring(0,i);
	  }

	  try { icolor=new Color(decodeColor(s)); }
	  catch(Exception ex) { icolor=Color.black; }

	  r=new SlafLineBorder(v[0],v[1],v[2],v[3],
			       v[4],v[5],v[6],v[7],
			       icolor,iline);
	}
	else
	if(s.startsWith("empty"))
	{
	  i=s.indexOf(',');
	  // String btype=s.substring(0,i); == empty
	  s=s.substring(i+1);

	  int[] v=new int[4];

	  for(int j=0;j<3;j++)
	  {
	    i=s.indexOf(',');
	    v[j]=Integer.parseInt(s.substring(0,i));
	    s=s.substring(i+1);
	  }

	  v[3]=Integer.parseInt(s);

	  r=new SlafLineBorder(v[0],v[1],v[2],v[3],
			       0,0,0,0,Color.black,
			       SlafLineBorder.SOLID);
	}
	else
	if(s.startsWith("rounded"))
	{
	  i=s.indexOf(',');
	  // String btype=s.substring(0,i); == rounded
	  s=s.substring(i+1);

	  i=s.indexOf(',');
	  String bmargin=s.substring(0,i);
	  s=s.substring(i+1);

	  i=s.indexOf(',');
	  String bthickness=s.substring(0,i);
	  s=s.substring(i+1);

	  i=s.indexOf(',');
	  String brayon;

	  if(i>=0)
	  {
	    brayon=s.substring(0,i);
	    s=s.substring(i+1);
	  }
	  else
	  {
	    brayon=s;
	    s="";
	  }

	  int imargin   =Integer.parseInt(bmargin);
	  int ithickness=Integer.parseInt(bthickness);
	  int irayon    =Integer.parseInt(brayon);

	  Color icolor=null;
	  try { icolor=new Color(decodeColor(s)); }
	  catch(Exception ex) { icolor=Color.black; }

	  r=new SlafRoundedBorder(imargin,ithickness,irayon,icolor);
	}
	else
	if(s.startsWith("image"))
	{
	  i=s.indexOf(',');
	  // String btype=s.substring(0,i); == image
	  s=s.substring(i+1);

	  int[] v=new int[8];

	  for(int j=0;j<8;j++)
	  {
	    i=s.indexOf(',');
	    v[j]=Integer.parseInt(s.substring(0,i));
	    s=s.substring(i+1);
	  }

	  Image image=null;
	  ImageIcon ii=loadImageIcon(s);
	  if(ii!=null) image=ii.getImage();
	  r=new SlafImageBorder(v[0],v[1],v[2],v[3],
				v[4],v[5],v[6],v[7],
				image);
	}
	else
	{
	  String btype,bmargin,bthickness,bshadow,bbg,bfg;

	  i=s.indexOf(',');
	  btype=s.substring(0,i);
	  s=s.substring(i+1);

	  i=s.indexOf(',');
	  bmargin=s.substring(0,i);
	  s=s.substring(i+1);

	  i=s.indexOf(',');
	  if(i>=0)
	  {
	    bthickness=(i==0 ? "0" : s.substring(0,i));
	    s=s.substring(i+1);
	    i=s.indexOf(',');
	    if(i>=0)
	    {
	      bshadow=(i==0 ? "0" : s.substring(0,i));
	      s=s.substring(i+1);
	      i=s.indexOf(',');
	      if(i>=0)
	      {
		bbg=(i==0 ? null : s.substring(0,i));
		s=s.substring(i+1);
		bfg=s;
	      }
	      else
	      {
		bbg=s;
		bfg=null;
	      }
	    }
	    else
	    {
	      bshadow=s;
	      bbg=null;
	      bfg=null;
	    }
	  }
	  else
	  {
	    bthickness=s;
	    bshadow="0";
	    bbg=null;
	    bfg=null;
	  }

	  // SlafUtils.trace("BORDER="+btype+","+bmargin+","+bthickness+","+bshadow);

	  int itype=SlafGenericBorder.NONE;
	  if(btype.equals("none"       )) itype=SlafGenericBorder.NONE;
	  if(btype.equals("flat"       )) itype=SlafGenericBorder.FLAT;
	  if(btype.equals("foreground" )) itype=SlafGenericBorder.FOREGROUND;
	  if(btype.equals("background" )) itype=SlafGenericBorder.BACKGROUND;
	  if(btype.equals("light"      )) itype=SlafGenericBorder.LIGHT;
	  if(btype.equals("dark"       )) itype=SlafGenericBorder.DARK;
	  if(btype.equals("raised"     )) itype=SlafGenericBorder.RAISED;
	  if(btype.equals("lowered"    )) itype=SlafGenericBorder.LOWERED;
	  if(btype.equals("etched"     )) itype=SlafGenericBorder.ETCHED;
	  if(btype.equals("revetched"  )) itype=SlafGenericBorder.REVETCHED;
	  if(btype.equals("softraised" )) itype=SlafGenericBorder.SOFTRAISED;
	  if(btype.equals("softlowered")) itype=SlafGenericBorder.SOFTLOWERED;
	  if(btype.equals("hardraised" )) itype=SlafGenericBorder.HARDRAISED;
	  if(btype.equals("hardlowered")) itype=SlafGenericBorder.HARDLOWERED;
	  if(btype.equals("wrraised"   )) itype=SlafGenericBorder.WRRAISED;
	  if(btype.equals("wrlowered"  )) itype=SlafGenericBorder.WRLOWERED;
	  if(btype.equals("wsraised"   )) itype=SlafGenericBorder.WSRAISED;
	  if(btype.equals("wslowered"  )) itype=SlafGenericBorder.WSLOWERED;

	  int   imargin   =Integer.parseInt(bmargin);
	  int   ithickness=Integer.parseInt(bthickness);
	  int   ishadow   =Integer.parseInt(bshadow);

	  Color cbg=null;
	  Color cfg=null;

	  if((bbg!=null)&&!bbg.equals("null")) cbg=new Color(decodeColor(bbg));
	  if((bfg!=null)&&!bfg.equals("null")) cfg=new Color(decodeColor(bfg));

	  r=new SlafGenericBorder
	    (itype,imargin,ithickness,ishadow,cbg,cfg);
	}

    return r;
  }

  // Icon

  private ImageIcon loadImageIcon(String s)
  {
    /*
    s= System.getProperty("user.home")
      +System.getProperty("file.separator")+".slaf"
      +System.getProperty("file.separator")+"icons"
      +System.getProperty("file.separator")+s;
    */

    s=base_+s;
    ImageIcon r=(ImageIcon)icons_.get(s);

    if(r==null)
    {
      try
      {
	// SlafUtils.trace("Load icon as "+s);
	if(s.startsWith("slaf:"))
	{
	  if(classloader_==null) initClassLoader();
	  r=SlafResource.SLAF.getIcon
	    (classloader_/*getClass()*/
	     .getResource(s.substring(5)));
	}
	else
	if(  s.startsWith("file:")
	   ||s.startsWith("http:")
	   ||s.startsWith("ftp:"))
	  r=SlafResource.SLAF.getIcon(new URL(s));
	else
	  r=SlafResource.SLAF.getIcon(s);

	//if(r.getIconHeight()<0) r=null;
	//else
	icons_.put(s,r);
      }
      catch(IOException ex)
      {
	SlafUtils.error(s+" -- "+ex.getMessage());
      }
    }
    if(r==null)
      // should create a default icon ?
      SlafUtils.error("can not find icon at "+s);

    return r;
  }

  private IconUIResource readIcon(String _s, IconUIResource _d)
  {
    IconUIResource r=_d;

    String s=props_.getProperty(_s);
    if(s!=null)
    {
      if("dummy".equals(s))
      {
	r=SlafManager.DUMMY_ICON;
      }
      else
      {
	ImageIcon ii=loadImageIcon(s);
	if(ii!=null) r=new IconUIResource(ii);
      }
    }

    return r;
  }

  private Image readImage(String _s, Image _d)
  {
    Image r=_d;

    String s=props_.getProperty(_s);
    if(s!=null)
    {
      ImageIcon ii=loadImageIcon(s);
      if(ii!=null) r=ii.getImage();
    }

    return r;
  }

  private ImageFilter readFilter(String _s, ImageFilter _d)
  {
    ImageFilter r=_d;

    String s=props_.getProperty(_s);
    if(s!=null)
    {
      try { r=decodeFilter(s); }
      catch(Exception ex) { SlafUtils.error("unreconized filter "+_s); }
    }

    return r;
  }

  private ImageFilter decodeFilter(String s)
  {
    ImageFilter r=null;

    if(s!=null)
    {
      int i=s.lastIndexOf('&');

      if(i>=0)
      {
        String inside =s.substring(i+1);
        String outside=s.substring(0,i);
        r=new SlafCompoundFilter
          (decodeFilter(outside),decodeFilter(inside));
      }
      else
      if("null".equals(s))
      {
	r=null;
      }
      else
      if("identity".equals(s))
      {
	r=new SlafIdentityFilter();
      }
      else
      if("blur".equals(s))
      {
	r=new SlafBlurFilter();
      }
      else
      if("invert".equals(s))
      {
	r=new SlafInvertFilter();
      }
      else
      if(s.startsWith("discrete"))
      {
	r=new SlafDiscreteFilter();

	try
	{
          s=s.substring(s.indexOf(',')+1);
          while(!"".equals(s))
          {
            String t;
            i=s.indexOf(',');
            if(i>=0) { t=s.substring(0,i); s=s.substring(i+1); }
            else     { t=s; s=""; }
            int    c=decodeColor(t);
            ((SlafDiscreteFilter)r).addColor(c);
	  }
        }
        catch(Exception ex)
        {
          SlafUtils.error("unreconized discrete filter");
          ex.printStackTrace();
        }
      }
      else
      {
	try
	{
	  //SlafUtils.trace("S="+s);

	  String tf="hue";
	  i=s.indexOf(',');
	  if(i>0) { tf=s.substring(0,i); s=s.substring(i+1); }

	  int rgb1,rgb2;

	  i=s.indexOf(',');
	  if(i<0)
	  {
	    rgb1=0x00000000;
	    rgb2=decodeColor(s);
	  }
	  else
	  {
	    rgb1=decodeColor(s.substring(0,i));
	    rgb2=decodeColor(s.substring(i+1));
	  }

	  if("color".equals(tf))
	    r=new SlafColorFilter(rgb2);
	  else
	  if("hue".equals(tf))
	    r=new SlafHueFilter(rgb1,rgb2);
	  else
	  if("mix".equals(tf))
	    r=new SlafMixFilter(rgb1,rgb2);
	  else
	  if("monochrome".equals(tf))
	    r=new SlafMonochromeFilter(rgb1,rgb2);
	}
	catch(Exception ex) { SlafUtils.error("unreconized filter "+s); }
      }
    }

    return r;
  }
      
  // Fields

  // Methods

  public String getName()
    { return name_; }

  public final void addCustomEntriesToTable(UIDefaults table)
  {
    super.addCustomEntriesToTable(table);

    if(!"default".equals(name_))
    {
      // Theme compatibility

      if(table.get("Menu.checkIcon")==
	 SlafIconFactory.getMenuCheckIcon())
	table.put("Menu.checkIcon",SlafIconFactory.getEmptyIcon
		  (table.get("CheckBoxMenuItem.selectedIcon")));

      if(table.get("MenuItem.checkIcon")==
	 SlafIconFactory.getMenuItemCheckIcon())
	table.put("MenuItem.checkIcon",SlafIconFactory.getEmptyIcon
		  (table.get("CheckBoxMenuItem.selectedIcon")));

      if(table.get("MenuItem.arrowIcon")==
	 SlafIconFactory.getMenuItemArrowIcon())
	table.put("MenuItem.arrowIcon",SlafIconFactory.getEmptyIcon
		  (table.get("Menu.arrowIcon")));

      if(table.get("CheckBoxMenuItem.arrowIcon")==
	 SlafIconFactory.getCheckBoxMenuItemArrowIcon())
	table.put("CheckBoxMenuItem.arrowIcon",SlafIconFactory.getEmptyIcon
		  (table.get("Menu.arrowIcon")));

      if(table.get("RadioButtonMenuItem.arrowIcon")==
	 SlafIconFactory.getRadioButtonMenuItemArrowIcon())
	table.put("RadioButtonMenuItem.arrowIcon",SlafIconFactory.getEmptyIcon
		  (table.get("Menu.arrowIcon")));
    }
  }
}
