/**
 * @modification $Date: 2004/06/11 19:23:24 $
 * @statut       unstable
 * @file         com/memoire/slaf/SlafDefaultTheme.java
 * @version      0.20
 * @author       Guillaume Desnoix
 * @email        guillaume-desnoix@memoire.com
 * @license      GNU General Public License 2 (GPL2)
 * @copyright    1999-2004 Guillaume Desnoix
 */

package com.memoire.slaf;
import com.memoire.slaf.*;

import java.awt.*;
import java.io.*;
import java.util.*;

import javax.swing.*;
import javax.swing.border.*;
import javax.swing.plaf.*;

public class SlafDefaultTheme
{
  public static final Object NULL=new UIDefaults.ActiveValue()
  {
    public Object createValue(UIDefaults _table)
    {
      return null;
    }
  };

  protected Hashtable resources_;

  public SlafDefaultTheme()
  {
    resources_=new Hashtable();

    initBorders (resources_);
    initColors  (resources_);
    initFonts   (resources_);
    initTexts   (resources_);
    initOthers  (resources_);
    initSynonyms(resources_);
  }

  // Color

  private static final int NL=2;

  protected static ColorUIResource brighten(ColorUIResource _c)
    { return brighten(_c,NL); }

  protected static ColorUIResource darken(ColorUIResource _c)
    { return darken(_c,NL); }

  protected static ColorUIResource brighten(ColorUIResource _c, int _n)
    { return new ColorUIResource(SlafUtils.intensity(_c,_n)); }

  protected static ColorUIResource darken(ColorUIResource _c, int _n)
    { return new ColorUIResource(SlafUtils.intensity(_c,-_n)); }

  // Fields

  protected FontUIResource controlFont=new FontUIResource("SansSerif", Font.PLAIN, 12);
  protected FontUIResource systemFont =new FontUIResource("SansSerif", Font.PLAIN, 12);
  protected FontUIResource windowFont =new FontUIResource("SansSerif", Font.PLAIN, 12);
  protected FontUIResource userFont   =new FontUIResource("Monospaced",Font.PLAIN, 12);
  protected FontUIResource smallFont  =new FontUIResource("SansSerif", Font.PLAIN, 10);

  // Methods

  public String getName()
    { return "default"; }

  public UIResource getResource(String _key)
    { return (UIResource)resources_.get(_key); }

  public BorderUIResource getBorderResource(String _key)
  {
    BorderUIResource r=(BorderUIResource)resources_.get(_key);
    if(r==null) r=new BorderUIResource(new EmptyBorder(0,0,0,0));
    return r;
  }

  public ColorUIResource getColorResource(String _key)
  {
    ColorUIResource r=(ColorUIResource)resources_.get(_key);
    if(r==null) r=new ColorUIResource(0,0,0);
    return r;
  }

  public FontUIResource getFontResource(String _key)
  {
    FontUIResource r=(FontUIResource)resources_.get(_key);
    if(r==null) r=new FontUIResource("SansSerif",Font.PLAIN,12);
    return r;
  }


  public void initColors(Hashtable table)
  {
    //SlafUtils.trace("%%% DEFAULT COLORS");

    ColorUIResource CBG=new ColorUIResource(224,224,224);
    ColorUIResource CFG=new ColorUIResource(  0,  0,  0);
    ColorUIResource PBG=new ColorUIResource(224,160,160);
    ColorUIResource PFG=new ColorUIResource(128, 64, 64);
    ColorUIResource WBG=new ColorUIResource(255,255,255);
    ColorUIResource WFG=new ColorUIResource(  0,  0,  0);
    ColorUIResource DBG=new ColorUIResource(128, 96, 96);
    ColorUIResource AFG=new ColorUIResource(128, 64, 64);
    ColorUIResource FOC=new ColorUIResource(255,  0,  0);
    ColorUIResource DIS=new ColorUIResource(160,160,160);//201,201,201

    ColorUIResource D_CBG=darken  (CBG);
    ColorUIResource D_CFG=darken  (CFG);
    ColorUIResource D_PBG=darken  (PBG);
    ColorUIResource B_CBG=brighten(CBG);
    ColorUIResource B_CFG=brighten(CFG);
    ColorUIResource B_PBG=brighten(PBG);

    Object[] colors=
    {
	// "Focus.foreground"                    ,FOC,
	// "Caret.foreground"                    ,FOC,

      "Desktop.background"                     ,DBG,

      "Component.background"                   ,CBG,
      "Component.foreground"                   ,CFG,
      "Component.primaryBackground"            ,PBG,
      "Component.primaryForeground"            ,PFG,
      "Component.disabledForeground"           ,DIS,
      "System.foreground"                      ,AFG,
      "System.disabledForeground"              ,DIS,
      "User.foreground"                        ,CFG,
      "User.highlightBackground"               ,PBG,
      "User.highlightForeground"               ,CFG,

      "Window.background"                      ,WBG,
      "Window.foreground"                      ,CFG,
      "WindowTitle.background"                 ,new ColorUIResource(  0,  0,128),
      "WindowTitle.foreground"                 ,new ColorUIResource(  0,  0,128),
      "WindowTitle.inactiveBackground"         ,new ColorUIResource( 96, 96,128),
      "WindowTitle.inactiveForeground"         ,CFG,
      "Frame.background"                       ,CBG,
      "Frame.foreground"                       ,CFG,
      "Label.background"                       ,CBG,
      "Label.foreground"                       ,DBG,
      // "Label.disabled"                         ,DIS,
      "Label.disabledForeground"               ,NULL,
      // "Label.disabledBackground"               ,CBG,
      "Label.disabledShadow"                   ,DIS,
      "Panel.background"                       ,CBG,
      "Panel.foreground"                       ,CFG,
      "DesktopIcon.foreground"                 ,WFG,
      "DesktopIcon.background"                 ,WBG,
      "DesktopIcon.labelForeground"            ,CFG,
      "DesktopIcon.labelBackground"            ,CBG,
      // "DesktopIcon.selectionLabelForeground"   ,PFG,
      // "DesktopIcon.selectionLabelBackground"   ,PBG,
      // "DesktopIcon.selectionIconForeground"    ,PFG,
      // "DesktopIcon.selectionIconBackground"    ,PBG,
      "InternalFrame.background"               ,CBG,
      "InternalFrame.foreground"               ,CFG,
      "InternalFrame.activeTitleBackground"    ,D_CBG,
      "InternalFrame.activeTitleForeground"    ,B_CFG,
      "InternalFrame.inactiveTitleBackground"  ,CBG,
      "InternalFrame.inactiveTitleForeground"  ,CFG,
      "Button.background"                      ,CBG,
      "Button.foreground"                      ,CFG,
      "Button.focus"                           ,FOC,
      "Button.pressed"                         ,PBG,
      // "Button.disabledText"                    ,DIS,
      // "Button.disabled"                        ,DIS,
      "ToggleButton.background"                ,CBG,
      "ToggleButton.foreground"                ,CFG,
      "ToggleButton.focus"                     ,FOC,
      "ToggleButton.pressed"                   ,PBG,
      "ToggleButton.selected"                  ,D_CBG,
      // "ToggleButton.disabledBackground"        ,DIS,
      // "ToggleButton.disabledSelectedBackground",DIS,
      // "ToggleButton.disabledText"              ,DIS,
      // "ToggleButton.disabledSelectedText"      ,DIS,
      "RadioButton.background"                 ,CBG,
      "RadioButton.foreground"                 ,CFG,
      "RadioButton.focus"                      ,FOC,
      "RadioButtonMenuItem.background"         ,CBG,
      "RadioButtonMenuItem.foreground"         ,CFG,
      //"RadioButtonMenuItem.pressedBackground"  ,PBG,
      //"RadioButtonMenuItem.pressedForeground"  ,PFG,
      "RadioButtonMenuItem.selectionBackground",PBG,
      "RadioButtonMenuItem.selectionForeground",PFG,
      "RadioButtonMenuItem.acceleratorForeground"         ,AFG,
      "RadioButtonMenuItem.acceleratorPressedForeground"  ,PFG,
      "RadioButtonMenuItem.acceleratorSelectionForeground",PFG,
      "CheckBox.background"                    ,CBG,
      "CheckBox.foreground"                    ,CFG,
      "CheckBox.focus"                         ,FOC,
      "CheckBoxMenuItem.background"            ,CBG,
      "CheckBoxMenuItem.foreground"            ,CFG,
      //"CheckBoxMenuItem.pressedBackground"     ,PBG,
      //"CheckBoxMenuItem.pressedForeground"     ,PFG,
      "CheckBoxMenuItem.selectionBackground"   ,PBG,
      "CheckBoxMenuItem.selectionForeground"   ,PFG,
      "CheckBoxMenuItem.acceleratorForeground"         ,AFG,
      "CheckBoxMenuItem.acceleratorPressedForeground"  ,PFG,
      "CheckBoxMenuItem.acceleratorSelectionForeground",PFG,
      "MenuBar.background"                     ,CBG,
      "MenuBar.foreground"                     ,CFG,
      "Menu.background"                        ,CBG,
      "Menu.foreground"                        ,CFG,
      //"Menu.pressedBackground"                 ,PBG,
      //"Menu.pressedForeground"                 ,PFG,
      "Menu.selectionBackground"               ,PBG,
      "Menu.selectionForeground"               ,PFG,
      // "Menu.disabledForeground"                ,DIS,
      "Menu.acceleratorForeground"             ,AFG,
      "Menu.acceleratorPressedForeground"      ,AFG,
      "Menu.acceleratorSelectionForeground"    ,AFG,
      "PopupMenu.background"                   ,CBG,
      "PopupMenu.foreground"                   ,CFG,
      "MenuItem.background"                    ,CBG,
      "MenuItem.foreground"                    ,CFG,
      //"MenuItem.pressedBackground"             ,PBG,
      //"MenuItem.pressedForeground"             ,PFG,
      "MenuItem.selectionBackground"           ,PBG,
      "MenuItem.selectionForeground"           ,PFG,
      // "MenuItem.disabledForeground"            ,DIS,
      "MenuItem.acceleratorForeground"         ,AFG,
      "MenuItem.acceleratorPressedForeground"  ,PFG,
      "MenuItem.acceleratorSelectionForeground",PFG,
      "Separator.background"                   ,CBG,
      "Separator.foreground"                   ,DBG,
      "Separator.highlight"                    ,B_CBG,
      "Separator.shadow"                       ,D_CBG,
      "ToolBarSeparator.background"            ,CBG,
      "ToolBarSeparator.foreground"            ,DBG,
      "ToolBarSeparator.highlight"             ,B_CBG,
      "ToolBarSeparator.shadow"                ,D_CBG,
      "PopupMenuSeparator.background"          ,CBG,
      "PopupMenuSeparator.foreground"          ,DBG,
      "PopupMenuSeparator.highlight"           ,B_CBG,
      "PopupMenuSeparator.shadow"              ,D_CBG,
      "SplitPane.highlight"                    ,B_CBG,
      "SplitPane.shadow"                       ,D_CBG,
      "TextField.background"                   ,CBG,
      "TextField.foreground"                   ,CFG,
      "TextField.inactiveBackground"           ,CBG,
      "TextField.inactiveForeground"           ,DIS,
      "TextField.selectionBackground"          ,PBG,
      "TextField.selectionForeground"          ,PFG,
      "TextField.caretForeground"              ,FOC,
      "PasswordField.background"               ,CBG,
      "PasswordField.foreground"               ,CFG,
      "PasswordField.inactiveBackground"       ,CBG,
      "PasswordField.inactiveForeground"       ,DIS,
      "PasswordField.selectionBackground"      ,PBG,
      "PasswordField.selectionForeground"      ,PFG,
      "PasswordField.caretForeground"          ,FOC,
      "TextArea.background"                    ,WBG,
      "TextArea.foreground"                    ,CFG,
      "TextArea.inactiveBackground"            ,WBG,
      "TextArea.inactiveForeground"            ,DIS,
      "TextArea.selectionBackground"           ,PBG,
      "TextArea.selectionForeground"           ,PFG,
      "TextArea.caretForeground"               ,FOC,
      "TextPane.background"                    ,WBG,
      "TextPane.foreground"                    ,CFG,
      "TextPane.inactiveBackground"            ,WBG,
      "TextPane.inactiveForeground"            ,DIS,
      "TextPane.selectionBackground"           ,PBG,
      "TextPane.selectionForeground"           ,PFG,
      "TextPane.caretForeground"               ,FOC,
      "EditorPane.background"                  ,WBG,
      "EditorPane.foreground"                  ,CFG,
      "EditorPane.inactiveBackground"          ,WBG,
      "EditorPane.inactiveForeground"          ,DIS,
      "EditorPane.selectionBackground"         ,PBG,
      "EditorPane.selectionForeground"         ,PFG,
      "EditorPane.caretForeground"             ,FOC,
      "ProgressBar.background"                 ,CBG,
      "ProgressBar.foreground"                 ,PBG,
      "ProgressBar.selectionBackground"        ,PBG,
      "ProgressBar.selectionForeground"        ,PFG,
      // "ProgressBar.backgroundHighlight"     ,CBG,
      // "ProgressBar.foregroundHighlight"     ,CFG,
      "ScrollBar.background"                   ,CBG,
      "ScrollBar.foreground"                   ,CFG,
      "ScrollBar.shadow"                       ,D_CBG,
      "ScrollBar.highlight"                    ,B_CBG,
      "ScrollBar.thumb"                        ,PBG,
      "ScrollBar.thumbHighlight"               ,B_PBG,
      "ScrollBar.thumbShadow"                  ,D_PBG,
      "ScrollBar.thumbDarkShadow"              ,D_PBG,
      "ScrollBar.thumbLightShadow"             ,B_PBG,
      "ScrollBar.track"                        ,CBG,
      "ScrollBar.trackHighlight"               ,B_CBG,
      "ScrollPane.background"                  ,CBG,
      "ScrollPane.foreground"                  ,CFG,
      "Slider.background"                      ,CBG,
      "Slider.foreground"                      ,CFG,
      "Slider.focus"                           ,FOC,
      "Slider.shadow"                          ,D_CBG,
      "Slider.highlight"                       ,B_CBG,
      // "Slider.trackBackground"                 ,CBG,
      // "Slider.thumbBackground"                 ,D_CBG,
      "Spinner.background"                     ,CBG,
      "Spinner.foreground"                     ,CFG,
      "SpinnerEditor.background"               ,CBG,
      "SpinnerEditor.foreground"               ,CFG,
      "SplitPane.background"                   ,CBG,
      "SplitPane.foreground"                   ,CFG,
      "SplitPane.shadow"                       ,D_CBG,
      "TabbedPane.background"                  ,CBG,
      "TabbedPane.foreground"                  ,CFG,
      "TabbedPane.tabBackground"               ,darken(CBG,NL),
      "TabbedPane.tabForeground"               ,CFG,
      "TabbedPane.selectionTabBackground"      ,CBG,
      "TabbedPane.focus"                       ,FOC,
      "TabbedPane.selected"                    ,PBG,
      "TabbedPane.tabHighlight"                ,B_CBG,
      "TabbedPane.tabShadow"                   ,D_CBG,
      "TabbedPane.tabDarkShadow"               ,darken(CBG,2*NL),
      "TabbedPane.highlight"                   ,B_CBG,
      "TabbedPane.lightHighlight"              ,brighten(CBG,2*NL),
      "TabbedPane.shadow"                      ,D_CBG,
      "TabbedPane.darkShadow"                  ,darken(CBG,2*NL),
      "TabbedPane.selectHighlight"             ,B_PBG,
      "TabbedPane.selectShadow"                ,D_PBG,
      "Table.background"                       ,WBG,//CBG,
      "Table.foreground"                       ,CFG,
      "Table.gridColor"                        ,CBG,//D_CBG,
      "Table.selectionBackground"              ,PBG,
      "Table.selectionForeground"              ,PFG,
      "Table.focusCellBackground"              ,B_CBG,
      "Table.focusCellForeground"              ,CFG,//FOC,
      "Table.focus"                            ,FOC,
      "TableHeader.background"                 ,CBG,
      "TableHeader.foreground"                 ,CFG,
      "ToolBar.background"                     ,CBG,
      "ToolBar.foreground"                     ,CFG,
      "ToolBar.dockingBackground"              ,CBG,
      "ToolBar.dockingForeground"              ,CFG,
      "ToolBar.floatingBackground"             ,CBG,
      "ToolBar.floatingForeground"             ,CFG,
      "ToolTip.background"                     ,new ColorUIResource(255,255,224),
      "ToolTip.foreground"                     ,CFG,
      "TitledBorder.titleColor"                ,CFG,
      "Viewport.background"                    ,CBG,
      "Viewport.foreground"                    ,CFG,

      "ComboBox.background"                    ,CBG,
      "ComboBox.foreground"                    ,CFG,
      "ComboBox.listBackground"                ,WBG,
      "ComboBox.listForeground"                ,WFG,
      "ComboBox.selectedBackground"            ,PBG,
      "ComboBox.selectedForeground"            ,PFG,
      "ComboBox.selectionBackground"           ,PBG,
      "ComboBox.selectionForeground"           ,PFG,
      "ComboBox.disabledBackground"            ,CBG,
      "ComboBox.disabledForeground"            ,DIS,
      "List.background"                        ,WBG,
      "List.foreground"                        ,WFG,
      "List.selectionBackground"               ,PBG,
      "List.selectionForeground"               ,PFG,
      "List.focus"                             ,FOC,
      "Tree.background"                        ,WBG,
      "Tree.foreground"                        ,WFG,
      "Tree.hash"                              ,CFG,
      "Tree.selectionBackground"               ,PBG,
      "Tree.selectionForeground"               ,PFG,
      "Tree.focus"                             ,FOC,
      "Tree.textBackground"                    ,WBG,
      "Tree.textForeground"                    ,WFG,
      "Tree.borderSelectionColor"              ,D_PBG,
      "Tree.selectionBorderColor"              ,D_PBG,

      "ToolBar.dockingColor"                   ,CBG,
      "ToolBar.floatingColor"                  ,CBG,
      "ToolBar.dockingBorderColor"             ,CBG,
      "ToolBar.floatingBorderColor"            ,CBG,

      "ColorChooser.background"                ,CBG,
      "ColorChooser.foreground"                ,CFG,
      //"ColorChooser.selectedColorBorder"       ,PBG,
      "ColorChooser.swatchesDefaultRecentColor",CFG,
      "OptionPane.background"                  ,CBG,
      "OptionPane.foreground"                  ,CFG,
      "OptionPane.messageForeground"           ,CFG,
    };

    for(int i=0; i<colors.length; i+=2)
      table.put((String)colors[i],colors[i+1]);
  }

  public void initFonts(Hashtable table)
  {
    //SlafUtils.trace("%%% DEFAULT FONTS");

     Object[] fonts=
     {
       "Component.font"                ,controlFont,
       "System.font"                   ,systemFont,
       "Window.titleFont"              ,windowFont,
       "User.font"                     ,userFont,
       "Small.font"                    ,userFont,

       "ComboBox.font"                 ,controlFont,
       "ComboBox.editorFont"           ,userFont,
       "DesktopIcon.font"              ,smallFont,
       "Panel.font"                    ,controlFont,
       "Button.font"                   ,controlFont,
       "ToggleButton.font"             ,controlFont,
       "RadioButton.font"              ,controlFont,
       "CheckBox.font"                 ,controlFont,
       "MenuBar.font"                  ,controlFont,
       "Menu.font"                     ,controlFont,
       "MenuItem.font"                 ,controlFont,
       "RadioButtonMenuItem.font"      ,controlFont,
       "CheckBoxMenuItem.font"         ,controlFont,
       "PopupMenu.font"                ,controlFont,
       "Label.font"                    ,controlFont,
       "List.font"                     ,controlFont,
       "ScrollPane.font"               ,controlFont,
       "Spinner.font"                  ,controlFont,
       "SpinnerEditor.font"            ,userFont,
       "SplitPane.font"                ,controlFont,
       "TabbedPane.font"               ,controlFont,
       "Table.font"                    ,controlFont,
       "TableHeader.font"              ,controlFont,
       "ToolBar.font"                  ,controlFont,
       "Tree.font"                     ,controlFont,
       "Viewport.font"                 ,controlFont,
       "OptionPane.font"               ,controlFont,
       "ColorChooser.font"             ,controlFont,

       "InternalFrame.titleFont"            ,windowFont,
       "InternalFrameTitlePane.font"        ,windowFont,

       "ProgressBar.font"                   ,smallFont,
       "TitledBorder.font"                  ,smallFont,
       "ToolTip.font"                       ,smallFont,
       "CheckBoxMenuItem.acceleratorFont"   ,smallFont,
       "RadioButtonMenuItem.acceleratorFont",smallFont,
       "Menu.acceleratorFont"               ,smallFont,
       "MenuItem.acceleratorFont"           ,smallFont,
       "InternalFrameTitlePane.paletteFont" ,smallFont,

       "TextField.font"                ,userFont,
       "PasswordField.font"            ,userFont,
       "TextArea.font"                 ,userFont,
       "TextPane.font"                 ,userFont,
       "EditorPane.font"               ,userFont,
     };

    for(int i=0; i<fonts.length; i+=2)
      table.put((String)fonts[i],new FontUIResource((Font)fonts[i+1]));
  }

  public void initSynonyms(Hashtable table)
  {
    //SlafUtils.trace("%%% DEFAULT SYNONYMS");

    Object[] synonyms=
    {
      //// comment this later VVV
      "white"                         ,new ColorUIResource(255,255,255),
      "black"                         ,new ColorUIResource(  0,  0,  0),

      "desktop"                       ,getColorResource("Desktop.background"),
      "activeCaption"                 ,getResource("WindowTitle.background"),
      "activeCaptionText"             ,getResource("WindowTitle.foreground"),
      "activeCaptionBorder"           ,getResource("Window.background"),
      "inactiveCaption"               ,getResource("Window.background"),
      "inactiveCaptionText"           ,getResource("Window.foreground"),
      "inactiveCaptionBorder"         ,getResource("Window.background"),
      "window"                        ,getResource("Window.background"),
      "windowBorder"                  ,getResource("Window.background"),
      "windowText"                    ,getResource("Window.foreground"),
      "menu"                          ,getResource("Menu.background"),
      "menuText"                      ,getResource("Menu.foreground"),          
      "scrollbar"                     ,getResource("ScrollBar.background"),
      "text"                          ,getResource("Window.background"),        
      "textText"                      ,getResource("Window.foreground"),
      "textHighlight"                 ,new ColorUIResource(0,0,128),
      "textHighlightText"             ,new ColorUIResource(0,0,160),
      "textInactiveText"              ,new ColorUIResource(0,0,192),

      "control"                       ,getColorResource("Component.background"),
      "controlText"                   ,getColorResource("Component.foreground"),
      "controlShadow"                 ,darken(getColorResource("control")),
      "controlDkShadow"               ,darken(getColorResource("controlShadow")),
      "controlHighlight"              ,brighten(getColorResource("control")),
      "controlLtHighlight"            ,brighten(getColorResource("controlHighlight")),

      "info"                          ,new ColorUIResource(0,128,0),
      "infoText"                      ,new ColorUIResource(0,192,0),

      "primaryControl"                ,new ColorUIResource(192,128,128),
      "primaryControlShadow"          ,darken(getColorResource("primaryControl")),
      "primaryControlDkShadow"        ,darken(getColorResource("primaryControlShadow")),
      "primaryControlHighlight"       ,brighten(getColorResource("primaryControl")),
      "primaryControlInfo"            ,new Color(0,0,0),
      "primaryControlDisabled"        ,brighten(getColorResource("primaryControlHighlight")),

    };

    for(int i=0; i<synonyms.length; i+=2)
    {
      // SlafUtils.trace("%%% "+synonyms[i]+"="+synonyms[i+1]);
      table.put((String)synonyms[i],synonyms[i+1]);
    }
  }

  public void initBorders(Hashtable table)
  {
    //SlafUtils.trace("%%% DEFAULT BORDERS");

    Border RAISED11 =new SlafGenericBorder(SlafGenericBorder.RAISED ,1,1,0);
    Border RAISED21 =new SlafGenericBorder(SlafGenericBorder.RAISED ,2,1,0);
    Border LOWERED21=new SlafGenericBorder(SlafGenericBorder.LOWERED,2,1,0);
    Border NONE21   =new SlafGenericBorder(SlafGenericBorder.NONE   ,2,1,0);
    Border NONE00   =new SlafGenericBorder(SlafGenericBorder.NONE   ,0,0,0);
    Border XLINE4   =new SlafLineBorder(0,0,0,0,0,0,4,0,new Color(224,224,224),SlafLineBorder.SOLID);
    Object[] borders=
    {
      "Button.border"                  ,new SlafButtonBorder("Button"),
      "CheckBox.border"                ,new SlafButtonBorder("CheckBox"),
      "CheckBoxMenuItem.border"        ,new SlafButtonBorder("CheckBoxMenuItem"),
      "ColorChooser.border"            ,new EmptyBorder(5,5,0,5),
      //SlafLineBorder(5,5,0,5,0,0,0,0,Color.black,SlafLineBorder.SOLID),
      "ComboBox.border"                ,NONE00,
      "ComboBox.buttonBorder"          ,new SlafButtonBorder("Button"),
      "ComboBox.editorBorder"          ,RAISED11,
      "ComboBox.listBorder"            ,RAISED21,
      "ComboBox.selectedCellBorder"    ,RAISED11,
      "ComboBox.cellBorder"            ,new SlafGenericBorder(SlafGenericBorder.NONE   ,1,1,0),
      "DesktopIcon.border"             ,new SlafGenericBorder(SlafGenericBorder.NONE   ,0,3,0),
      "DesktopIcon.labelBorder"        ,RAISED21,
      "DesktopIcon.selectedLabelBorder",RAISED21,
      "EditorPane.border"              ,new SlafComponentBorder("EditorPane"),
      "InternalFrame.border"           ,new SlafGenericBorder(SlafGenericBorder.RAISED ,4,1,0),
      "InternalFrame.paletteBorder"    ,RAISED21,
      "InternalFrameTitlePane.border"  ,new SlafInternalFrameTitlePaneBorder(),
      "InternalFrameTitleButton.border",new SlafButtonBorder("InternalFrameTitleButton"),
      "Label.border"                   ,new SlafComponentBorder("Label"),
      //"List.border"
      "List.focusCellHighlightBorder"  ,NULL,//new LineBorder(new Color(255,0,0)),

      "List.selectedCellBorder"        ,RAISED11,
      "List.cellBorder"                ,new SlafGenericBorder(SlafGenericBorder.NONE   ,1,1,0),
      "MenuBar.border"                 ,RAISED11,
      "Menu.border"                    ,new SlafButtonBorder("Menu"),
      "MenuItem.border"                ,new SlafButtonBorder("MenuItem"),
      "OptionPane.border"              ,new EmptyBorder(5,5,5,5),
      "OptionPane.buttonAreaBorder"    ,NONE00,
      "OptionPane.messageAreaBorder"   ,new SlafGenericBorder(SlafGenericBorder.LOWERED,4,1,0),

      // "Component.border"            ,new LineBorder(Color.red,10),
      // "Frame.border"                ,new SlafGenericBorder(SlafGenericBorder.RAISED ,10,10,0),
      // "Window.border"               ,new SlafGenericBorder(SlafGenericBorder.LOWERED,10,10,0),
      // "Panel.border"                ,new SlafGenericBorder(SlafGenericBorder.LOWERED,10,10,0),
      // "Viewport.border"             ,new SlafGenericBorder(SlafGenericBorder.LOWERED,10,10,0),

      "PasswordField.border"           ,new SlafComponentBorder("PasswordField"),
      "PopupMenu.border"               ,new SlafLightBorder(SlafLightBorder.RAISED,2), //RAISED21,

      "ProgressBar.border"             ,new SlafGenericBorder(SlafGenericBorder.LOWERED,0,1,0),
      "ProgressBar.progressBorder"     ,NONE00,
      "RadioButton.border"             ,new SlafButtonBorder("RadioButton"),
      "RadioButtonMenuItem.border"     ,new SlafButtonBorder("RadioButtonMenuItem"),
      "ScrollBar.trackBorder"          ,new SlafGenericBorder(SlafGenericBorder.RAISED ,0,1,0),
      "ScrollBar.thumbBorder"          ,new SlafGenericBorder(SlafGenericBorder.RAISED ,0,1,0),
      "ScrollBar.lineBorder"           ,new SlafGenericBorder(SlafGenericBorder.LOWERED,0,1,0),
      "ScrollButton.border"            ,new SlafButtonBorder("ScrollButton"),
      "ScrollPane.border"              ,new SlafGenericBorder(SlafGenericBorder.LOWERED,0,1,0),
      "Slider.trackBorder"             ,new SlafGenericBorder(SlafGenericBorder.RAISED ,0,1,0),
      "Slider.thumbBorder"             ,new SlafGenericBorder(SlafGenericBorder.RAISED ,0,1,0),
      "Slider.lineBorder"              ,new SlafGenericBorder(SlafGenericBorder.LOWERED,0,1,0),
      "Spinner.border"                 ,NONE00,
      "SpinnerEditor.border"           ,new SlafComponentBorder("SpinnerEditor"),
      "SplitPane.border"               ,new SlafGenericBorder(SlafGenericBorder.NONE   ,0,3,0),
      "SplitPaneDivider.border"        ,NONE00,
      "TabbedPane.border"              ,NONE00,
      "TabbedPane.contentBorder"       ,RAISED21,
      "TabbedPane.tabBorder"           ,RAISED21,
      "TabbedPane.selectedTabBorder"   ,RAISED21, //LOWERED21,
      "Table.cellBorder"               ,NONE21,
      "Table.selectedCellBorder"       ,NONE21,//RAISED21,
      "Table.scrollPaneBorder"         ,new SlafGenericBorder(SlafGenericBorder.NONE   ,0,3,0),
      "Table.focusCellHighlightBorder" ,NULL,//new LineBorder(new Color(255,0,0)),
      "TableHeader.cellBorder"         ,RAISED11,
      "TextField.border"               ,new SlafComponentBorder("TextField"),
      "TextArea.border"                ,new SlafComponentBorder("TextArea"),
      "TextPane.border"                ,new SlafComponentBorder("TextPane"),
      "TitledBorder.border"            ,new EtchedBorder(),
      "ToggleButton.border"            ,new SlafButtonBorder("ToggleButton"),
      "ToolBar.border"                 ,new SlafToolBarBorder(),
      "ToolTip.border"                 ,new SlafGenericBorder(SlafGenericBorder.FLAT   ,2,1,0),
      // "Tree.border"
      "Tree.selectedCellBorder"        ,RAISED11,
      "Tree.cellBorder"                ,new SlafGenericBorder(SlafGenericBorder.NONE   ,1,1,0),
      "Tree.focusCellHighlightBorder"  ,NULL,//new LineBorder(new Color(255,0,0)),
      "Tree.editorBorder"              ,new SlafGenericBorder(SlafGenericBorder.FLAT   ,1,1,0),

      "Button.enabledBorder"           ,RAISED21,
      "Button.pressedBorder"           ,LOWERED21,
      "Button.disabledBorder"          ,NONE21,

      "CheckBoxMenuItem.enabledBorder" ,NONE21,
      "CheckBoxMenuItem.pressedBorder" ,RAISED21,
      "CheckBoxMenuItem.disabledBorder",NONE21,
      "CheckBoxMenuItem.selectedBorder",RAISED21,

      "EditorPane.enabledBorder"       ,new SlafGenericBorder(SlafGenericBorder.LOWERED,3,0,0),
      "EditorPane.disabledBorder"      ,new SlafGenericBorder(SlafGenericBorder.NONE   ,3,0,0),

      "InternalFrameTitlePane.enabledBorder"   ,new CompoundBorder(XLINE4,RAISED21 ),
      "InternalFrameTitlePane.selectedBorder"  ,new CompoundBorder(XLINE4,LOWERED21),
      "InternalFrameTitlePane.disabledBorder"  ,new CompoundBorder(XLINE4,NONE21   ),

      // "Label.enabledBorder"         ,new SlafGenericBorder(SlafGenericBorder.FLAT,3,0,0),
      // "Label.disabledBorder"        ,new SlafGenericBorder(SlafGenericBorder.FLAT,3,0,0),

      "Menu.enabledBorder"             ,NONE21,
      "Menu.pressedBorder"             ,RAISED21,
      "Menu.disabledBorder"            ,NONE21,
      "Menu.selectedBorder"            ,RAISED21,

      "MenuItem.enabledBorder"         ,NONE21,
      "MenuItem.pressedBorder"         ,RAISED21,
      "MenuItem.disabledBorder"        ,NONE21,
      "MenuItem.selectedBorder"        ,RAISED21,

      "PasswordField.enabledBorder"    ,LOWERED21,
      "PasswordField.disabledBorder"   ,NONE21,

      "RadioButtonMenuItem.enabledBorder"        ,NONE21,
      "RadioButtonMenuItem.pressedBorder"        ,RAISED21,
      "RadioButtonMenuItem.disabledBorder"       ,NONE21,
      "RadioButtonMenuItem.selectedBorder"       ,RAISED21,

      "ScrollButton.enabledBorder"               ,RAISED21,
      "ScrollButton.pressedBorder"               ,LOWERED21,
      "ScrollButton.disabledBorder"              ,NONE21,

      "SpinnerEditor.enabledBorder"              ,LOWERED21,
      "SpinnerEditor.disabledBorder"             ,NONE21,

      "InternalFrameTitleButton.enabledBorder"   ,NONE00,
      "InternalFrameTitleButton.pressedBorder"   ,NONE00,
      "InternalFrameTitleButton.disabledBorder"  ,NONE00,

      "ToggleButton.enabledBorder"     ,RAISED21,
      "ToggleButton.pressedBorder"     ,LOWERED21,
      "ToggleButton.disabledBorder"    ,NONE21,
      "ToggleButton.selectedBorder"    ,LOWERED21,

      "TextArea.enabledBorder"         ,new SlafGenericBorder(SlafGenericBorder.LOWERED,3,0,0),
      "TextArea.disabledBorder"        ,new SlafGenericBorder(SlafGenericBorder.NONE   ,3,0,0),
      "TextField.enabledBorder"        ,LOWERED21,
      "TextField.disabledBorder"       ,NONE21,
      "TextPane.enabledBorder"         ,new SlafGenericBorder(SlafGenericBorder.LOWERED,3,0,0),
      "TextPane.disabledBorder"        ,new SlafGenericBorder(SlafGenericBorder.NONE   ,3,0,0),

      // RootPane
      "RootPane.frameBorder",              new SlafLineBorder(Color.blue  ,5),
      "RootPane.plainDialogBorder",        new SlafLineBorder(Color.green ,5),
      "RootPane.informationDialogBorder",  new SlafLineBorder(Color.red   ,5),
      "RootPane.errorDialogBorder",        new SlafLineBorder(Color.red   ,5),
      "RootPane.colorChooserDialogBorder", new SlafLineBorder(Color.yellow,5),
      "RootPane.fileChooserDialogBorder",  new SlafLineBorder(Color.yellow,5),
      "RootPane.questionDialogBorder",     new SlafLineBorder(Color.red   ,5),
      "RootPane.warningDialogBorder",      new SlafLineBorder(Color.red   ,5),

      // additionnal
      "Theme.border"                   ,RAISED11,

      // specific BU
      "Column.border"                  ,RAISED11,
      "StatusBar.border"               ,RAISED11,
    };

    for(int i=0; i<borders.length; i+=2)
    {
      Object v=borders[i+1];
      if(v instanceof Border) v=new BorderUIResource((Border)v);
      table.put((String)borders[i],v);
    }
  }

  public void initTexts(Hashtable table)
  {
    //SlafUtils.trace("%%% DEFAULT TEXTS");

    /*
    Object[] texts=
    {
       "ColorChooser.cancelText"              ,"Annuler",
       "ColorChooser.hsbBlueText"             ,"Bleu",
       "ColorChooser.hsbBrightnessText"       ,"Luminosit",
       "ColorChooser.hsbGreenText"            ,"Vert",
       "ColorChooser.hsbHueText"              ,"Teinte",
       "ColorChooser.hsbNameText"             ,"TSL",
       "ColorChooser.hsbRedText"              ,"Rouge",
       "ColorChooser.hsbSaturationText"       ,"Saturation",
       "ColorChooser.okText"                  ,"Valider",
       "ColorChooser.previewText"             ,"Prvisualisation",
       "ColorChooser.resetText"               ,"Rinitialiser",
       "ColorChooser.rgbBlueText"             ,"Bleu",
       "ColorChooser.rgbGreenText"            ,"Vert",
       "ColorChooser.rgbNameText"             ,"RVB",
       "ColorChooser.rgbRedText"              ,"Rouge",
       "ColorChooser.swatchesNameText"        ,"Nuancier",
       "ColorChooser.swatchesRecentText"      ,"Rcents",
       "ColorChooser.rgbBlueMnemonic"         ,"R",
       "ColorChooser.rgbGreenMnemonic"        ,"V",
       "ColorChooser.rgbRedMnemonic"          ,"B",
       "ColorChooser.sampleText"              ,"Echantillon",

       "FileChooser.cancelButtonText"         ,"Annuler",
       "FileChooser.helpButtonText"           ,"Aide",
       "FileChooser.openButtonText"           ,"Ouvrir",
       "FileChooser.saveButtonText"           ,"Enregistrer",
       "FileChooser.updateButtonText"         ,"Mettre  jour",
       "FileChooser.cancelButtonToolTipText"  ,"Annuler la slection des fichiers",
       "FileChooser.detailsViewButtonToolTipText","Affichage dtaill",
       "FileChooser.helpButtonToolTipText"    ,"Aide du slecteur de fichiers",
       "FileChooser.homeFolderToolTipText"    ,"Rpertoire utilisateur",
       "FileChooser.listViewButtonToolTipText","Affichage en liste",
       "FileChooser.newFolderToolTipText"     ,"Nouveau rpertoire",
       "FileChooser.openButtonToolTipText"    ,"Ouvrir le fichier slection",
       "FileChooser.saveButtonToolTipText"    ,"Enregistrer le fichier slection",
       "FileChooser.updateButtonToolTipText"  ,"Mettre  jour la liste",
       "FileChooser.upFolderToolTipText"      ,"Rpertoire parent",

       "FileChooser.acceptAllFileFilterText"  ,"Tous fichiers (*.*)",
       "FileChooser.directoryDescriptionText" ,"Rpertoire",
       "FileChooser.fileDescriptionText"      ,"Fichier",
       "FileChooser.newFolderErrorText"       ,"Erreur  la cration du dossier",
       "FileChooser.lookInLabelText"          ,"Rpertoire:",
       "FileChooser.fileNameLabelText"        ,"Fichier:",
       "FileChooser.filesOfTypeLabelText"     ,"Filtre:",

       "FileChooser.upFolderAccessibleName"   ,"Parent",
       "FileChooser.homeFolderAccessibleName" ,"Maison",
       "FileChooser.newFolderAccessibleName"  ,"Nouveau",
       "FileChooser.listViewButtonAccessibleName"   ,"Liste",
       "FileChooser.detailsViewButtonAccessibleName","Dtails",

       "FileChooser.newFolderErrorSeparator"  ,":",
       "FileChooser.cancelButtonMnemonic"     ,"N",
       "FileChooser.fileNameLabelMnemonic"    ,"F",
       "FileChooser.filesOfTypeLabelMnemonic" ,"T",
       "FileChooser.helpButtonMnemonic"       ,"A",
       "FileChooser.lookInLabelMnemonic"      ,"L",
       "FileChooser.openButtonMnemonic"       ,"O",
       "FileChooser.saveButtonMnemonic"       ,"E",
       "FileChooser.updateButtonMnemonic"     ,"M",

       "MenuItem.acceleratorDelimiter"        ,"-", 

       "OptionPane.cancelButtonText"          ,"Annuler",
       "OptionPane.noButtonText"              ,"Non",
       "OptionPane.okButtonText"              ,"Valider",
       "OptionPane.yesButtonText"             ,"Oui",
    };

    if(Locale.getDefault().getLanguage().startsWith("fr"))
      for(int i=0; i<texts.length; i+=2)
	table.put((String)texts[i],texts[i+1]);
    */

    if(  //(SlafUtils.jdk()<1.2)&&
       Locale.getDefault().getLanguage().startsWith("fr"))
    {
      LineNumberReader in=null;
      try
      {
	in=new LineNumberReader
	  (new InputStreamReader
	      (getClass().getResourceAsStream("slaf_fr.txt"),"iso-8859-1"));
	int    n=0;
	String l=null;

	while(true)
	{
	  l=in.readLine();
	  if(l==null) break;
	  n++;

	  l=l.trim();
	  if("".equals(l)||l.startsWith("#")) continue;

	  int    i=l.indexOf('=');
	  String k=l.substring(0,Math.max(0,i-1)).trim();
	  String v=l.substring(i+1);

	  if((i>=0)&&"".equals(k))
	  {
	    SlafUtils.trace("slaf_fr["+n+"]: bad key");
	    continue;
	  }

	  table.put(k,v);
	}
      }
      catch(IOException ex)
      {
	SlafUtils.trace("SlafDefaultTheme.initTexts: "+ex);
      }
      finally
      {
	try { if(in!=null) in.close(); }
	catch(IOException ex) { }
      }
    }
  }

  public void initOthers(Hashtable table)
  {
    //SlafUtils.trace("%%% DEFAULT OTHERS");

    Integer          INT2   =new Integer(2);
    Integer          INT3   =new Integer(3);
    Integer          INT500 =new Integer(500);
    InsetsUIResource INSETS1=new InsetsUIResource(1,1,1,1);

     Object[] others=
     {
       //"ComboBox.bar"                       ,Boolean.TRUE,
       "Separator.bar"                      ,Boolean.TRUE,
       "PopupMenuSeparator.bar"             ,Boolean.TRUE,
       "ToolBarSeparator.bar"               ,Boolean.TRUE,

       "Menu.borderPainted"                 ,Boolean.TRUE,
       "Menu.consumesTabs"                  ,Boolean.TRUE,
       "MenuItem.borderPainted"             ,Boolean.TRUE,
       "CheckBoxMenuItem.borderPainted"     ,Boolean.TRUE,
       "RadioButtonMenuItem.borderPainted"  ,Boolean.TRUE,
       "Tree.changeSelectionWithFocus"      ,Boolean.TRUE,
       "Tree.scrollsOnExpand"               ,Boolean.TRUE,

       // "DesktopIcon.labelOpaque"            ,Boolean.FALSE,
       "DesktopIcon.selectionLabelOpaque"   ,Boolean.TRUE,
       "ProgressThumb.opaque"               ,Boolean.TRUE,
       //"TabbedPane.opaque"                  ,Boolean.TRUE,

       "ScrollBar.horizontalType"           ,INT2,
       "ScrollBar.verticalType"             ,INT2,

       "Button.textIconGap"                 ,INT3,
       "CheckBox.textIconGap"               ,INT3,
       "RadioButton.textIconGap"            ,INT3,
       "TabbedPane.textIconGap"             ,INT3,
       "ToggleButton.textIconGap"           ,INT3,

       "Button.textShiftOffset"             ,INT3,
       "CheckBox.textShiftOffset"           ,INT3,
       "RadioButton.textShiftOffset"        ,INT3,
       "TabbedPane.textShiftOffset"         ,INT3,
       "ToggleButton.textShiftOffset"       ,INT3,

       "ProgressBar.cellLength"             ,new Integer(5),
       "ProgressBar.cellSpacing"            ,new Integer(1),
       "ProgressBar.width"                  ,new Integer(18),
       "DesktopIcon.width"                  ,new Integer(90),
       "ScrollBar.width"                    ,new Integer(18),
       "Spinner.gap"                        ,new Integer(2),
       "SplitPane.dividerSize"              ,new Integer(19),
       "Slider.majorTickLength"             ,new Integer(10),
       "Slider.width"                       ,new Integer(18),
       "TabbedPane.tabRunOverlay"           ,new Integer(2),
       "TabbedPane.interTabSpace"           ,new Integer(-5),
       "Tree.leftChildIndent"               ,new Integer(7),
       "Tree.rightChildIndent"              ,new Integer(13),
       "Tree.rowHeight"                     ,new Integer(16),

       "TextField.caretBlinkRate"           ,INT500,
       "PasswordField.caretBlinkRate"       ,INT500,
       "TextArea.caretBlinkRate"            ,INT500,
       "TextPane.caretBlinkRate"            ,INT500,
       "EditorPane.caretBlinkRate"          ,INT500,

       "Button.margin"                      ,INSETS1,
       "CheckBox.margin"                    ,INSETS1,
       "CheckBoxMenuItem.margin"            ,INSETS1,
       "EditorPane.margin"                  ,INSETS1,
       "Menu.margin"                        ,INSETS1,
       "MenuItem.margin"                    ,INSETS1,
       "PasswordField.margin"               ,INSETS1,
       "RadioButton.margin"                 ,INSETS1,
       "RadioButtonMenuItem.margin"         ,INSETS1,
       "TextArea.margin"                    ,INSETS1,
       "TextField.margin"                   ,INSETS1,
       "TextPane.margin"                    ,INSETS1,
       "ToggleButton.margin"                ,INSETS1,

       "Slider.focusInsets"                 ,new InsetsUIResource(0,0,0,0),
       "TabbedPane.selectedTabPadInsets"    ,new InsetsUIResource(1,1,1,1),
       "TabbedPane.tabInsets"               ,new InsetsUIResource(0,4,1,4),
       "TabbedPane.tabAreaInsets"           ,new InsetsUIResource(3,2,0,2),
       "TabbedPane.contentBorderInsets"     ,new InsetsUIResource(2,2,3,3),

       "ColorChooser.swatchesRecentSwatchSize",new DimensionUIResource(  10,  10),
       "ColorChooser.swatchesSwatchSize"      ,new DimensionUIResource(  10,  10),
       "OptionPane.minimumSize"               ,new DimensionUIResource( 250,  79),
       "ScrollBar.maximumThumbSize"           ,new DimensionUIResource(4096,4096),
       "ScrollBar.minimumThumbSize"           ,new DimensionUIResource(   8,   8),
       "Slider.maximumThumbSize"              ,new DimensionUIResource(  19,  19),
       "ToolBar.separatorSize"                ,new DimensionUIResource(   4,   4),
     };

    for(int i=0; i<others.length; i+=2)
      table.put((String)others[i],others[i+1]);
  }

  public void addCustomEntriesToTable(UIDefaults table)
  {
    /*
    for(Enumeration e=table.keys(); e.hasMoreElements(); )
    {
      String k=(String)e.nextElement();
      Object o=resources_.get(k);

      if(  (o==null)
	 &&!(k.toLowerCase().endsWith("ui"))
	 &&!(k.toLowerCase().endsWith("icon"))
	 &&!(k.toLowerCase().endsWith("keybindings"))
	 &&!(k.toLowerCase().endsWith("editor"))
	 &&!(k.toLowerCase().endsWith("renderer")))
      {
	try
	{
	  String s=""+table.get(k);
	  if(!"".equals(s))
	  SlafUtils.trace(k+" not overloaded: "+s);
	}
	catch(Throwable th) { }
      }
    }
    SlafUtils.trace("%%% FIN CONTROLE");
    */

    // try to reuse the same objects
    Hashtable values=new Hashtable(resources_.size());
    for(Enumeration e=resources_.keys(); e.hasMoreElements(); )
    {
      String k=(String)e.nextElement();
      Object o=resources_.get(k);
      Object p=values.get(o);
      if(p==null) { values.put(o,o); p=o; }
      table.put(k,p);
    }
    values.clear();
    values=null;
  }
}
