/**
 * @modification $Date: 2004/06/25 17:55:53 $
 * @statut       unstable
 * @file         com/memoire/slaf/SlafTest.java
 * @version      0.20
 * @author       Guillaume Desnoix
 * @email        guillaume-desnoix@memoire.com
 * @license      GNU General Public License 2 (GPL2)
 * @copyright    1999-2004 Guillaume Desnoix
 */

package com.memoire.slaf;
import com.memoire.slaf.*;

//import java.lang.reflect.*; 
import java.awt.*; 
import java.awt.event.*; 
import java.beans.*;
import javax.swing.*; 
import javax.swing.event.*; 
import javax.swing.border.*; 
import javax.swing.plaf.*; 

public class SlafDemo
  extends JFrame
  implements ActionListener
{
  private final String[] THEMES=new String[] {
      "Default",
      //"Alien",
      "Aluminium",
      "Apple2c",
      "Apple2e",
      "Children",
      //"Clown",
      //"Ecologic",
      //"Mondrian",
      //"Sand",
      //"Shadow",
      "Tx",
      "Galaxy",
      "Gtk",
      "Gnome",
      "Kde",
      "Step",
      "X11",
      "Aqua",
      "Beos",
      "Edbgruppen",
      "Lothar",
      "Napkin",
      "Pilot",
      "Redmond31",
      "Redmond00",
      "Redmondxp",
    };

  JPanel      pnContent_;
  JComboBox   cbTheme_;
  JTabbedPane tpMain_;
  JButton[]   btTools_;

  public SlafDemo()
  {
    this(null);
  }

  public SlafDemo(String _theme)
  {
    super("Slaf Demo (on jre "+System.getProperty("java.version")+")");

    //setUndecorated(true);

    updateLnf(_theme);

    buildThemeList();
    buildMain();

    JPanel pnNorth=new JPanel()
    {
      public void paintComponent(Graphics _g)
      {
	super.paintComponent(_g);

	Paint old=((Graphics2D)_g).getPaint();
	Rectangle r=new Rectangle(0,0,this.getWidth(),this.getHeight());

	GradientPaint gp=new GradientPaint
	  (r.x,r.y,           new Color( 64, 96,128, 48),
	   r.x,r.y+r.height-1,new Color(255,255,255,128));
	((Graphics2D)_g).setPaint(gp);
	_g.fillRoundRect(r.x,r.y,r.width,r.height,11,11);
	
	((Graphics2D)_g).setPaint(old);
      }
    };
    pnNorth.setLayout(new SlafGridLayout(3,5,5,true,true));

    JLabel logo=new JLabel(new ImageIcon(getClass().getResource("demo.gif")));
    logo.setBorder(new LineBorder(Color.black));
    logo.addMouseListener(new MouseAdapter()
      {
        public void mouseClicked(MouseEvent _evt)
        {
          SlafLookAndFeel.setCurrentTheme("");
          String t=(String)cbTheme_.getSelectedItem();
          if(t!=null) updateLnf(t.toLowerCase());
        }
      });

    pnNorth.add(logo);
    pnNorth.add(new JLabel("Select a theme:"));
    pnNorth.add(cbTheme_);
    pnNorth.setBorder(new SlafRoundedBorder(5,1,11,Color.black));
    //new SlafLineBorder(0,0,5,0,0,0,1,0,Color.black,1));

    JLabel pnSouth=new JLabel()
    {
      public void paintComponent(Graphics _g)
      {
	super.paintComponent(_g);

	Paint old=((Graphics2D)_g).getPaint();
	Rectangle r=new Rectangle(0,0,this.getWidth(),this.getHeight());

	GradientPaint gp=new GradientPaint
	  (r.x,r.y,           new Color(128, 96, 64, 48),
	   r.x,r.y+r.height-1,new Color(255,255,255,128));
	((Graphics2D)_g).setPaint(gp);
	_g.fillRoundRect(r.x,r.y,r.width,r.height,11,11);
	
	((Graphics2D)_g).setPaint(old);

	super.paintComponent(_g);
      }
    };
    pnSouth.setFont(new Font("SansSerif",Font.PLAIN,10));
    pnSouth.setHorizontalTextPosition(SwingConstants.CENTER);
    pnSouth.setText(" Version 0.18 (c)1998-2004 Guillaume Desnoix ");
    pnSouth.setBorder(new SlafRoundedBorder(2,1,11,Color.black));

    pnContent_=new JPanel(new BorderLayout(10,10));
    //new SlafGridLayout(1,10,10,true,true));
    pnContent_.setBorder(new EmptyBorder(5,5,5,5));
    pnContent_.add(pnNorth,BorderLayout.NORTH);
    pnContent_.add(tpMain_,BorderLayout.CENTER);
    pnContent_.add(pnSouth,BorderLayout.SOUTH);

    setContentPane(pnContent_);
    setSize(500,300);

    if((_theme==null)||"".equals(_theme)) _theme="redmondxp";
    _theme=_theme.substring(0,1).toUpperCase()+_theme.substring(1);
    cbTheme_.setSelectedItem(_theme);//updateLnf("lothar");

    //setDefaultOperationOnClose(JFrame.EXIT_ON_CLOSE);
    addWindowListener(new WindowAdapter() {
      public void windowClosing(WindowEvent _evt)
      {
	System.exit(0);
      }
    });

    show();
  }

  public void updateLnf(String _theme)
  {
    if(SlafLookAndFeel.getCurrentTheme().equals(_theme)) return;

    try
    {
      UIManager.setLookAndFeel(new SlafLookAndFeel(_theme));
      if(btTools_!=null)
      {
	if("edbgruppen".equals(_theme))
	  pnContent_.setBackground(new Color(192,204,224));
	else
	  pnContent_.setBackground(new ColorUIResource(255,0,0));
	SwingUtilities.updateComponentTreeUI(this);
        btTools_[0].setIcon(UIManager.getIcon("FileView.directoryIcon"));
	btTools_[1].setIcon(UIManager.getIcon("FileView.fileIcon"));
	btTools_[2].setIcon(UIManager.getIcon("FileView.hardDriveIcon"));
	btTools_[3].setIcon(UIManager.getIcon("FileView.floppyDriveIcon"));
      }
    }
    catch(Exception ex)
    {
      System.err.println("SLF: Error with theme "+_theme);
      ex.printStackTrace();
    }

    /*
    try
    {
      UIManager.setLookAndFeel(new com.sun.java.swing.plaf.motif.MotifLookAndFeel());
    }
    catch(Exception ex) { }
    */

    pack();

    /*
    try
    {
      LookAndFeel old=UIManager.getLookAndFeel();
      UIManager.setLookAndFeel(new javax.swing.plaf.metal.MetalLookAndFeel());

    javax.jnlp.FileOpenService fos=null;
    try {
      fos = (javax.jnlp.FileOpenService)javax.jnlp.ServiceManager.lookup
	("javax.jnlp.FileOpenService");
    } catch (javax.jnlp.UnavailableServiceException e) { } 
    try {
      if(fos!=null)
      {
	javax.jnlp.FileContents fc=fos.openFileDialog(null,null);
	System.err.println("FILE CONTENTS: "+fc);
      }
    }
    catch(Exception ex) { ex.printStackTrace(); }

    UIManager.setLookAndFeel(old);
    }
    catch(UnsupportedLookAndFeelException ulnfex) { }
    //new JFileChooser().showOpenDialog(null);
    */
  }

  private void buildThemeList()
  {
    cbTheme_=new JComboBox(THEMES);
    cbTheme_.addActionListener(this);
  }

  private void buildMain()
  {
    JPanel tab1=new JPanel();
    tab1.setLayout(new SlafGridLayout(4,5,5,true,true,false,false));
    tab1.setBorder(new EmptyBorder(5,5,5,5));
    JButton b11=new JButton("Button");
    JButton b12=new JButton("Button");
    b12.setEnabled(false);
    JButton b111=new JButton("Button");
    JButton b121=new JButton("Button");
    b111.setBackground(Color.red);
    b121.setBackground(Color.red);
    b121.setEnabled(false);

    // TMP
    //b11.getModel().setPressed(true);

    JToggleButton b13=new JToggleButton("Toggle Button");
    JToggleButton b14=new JToggleButton("Toggle Button");
    JToggleButton b131=new JToggleButton("Toggle Button");
    JToggleButton b141=new JToggleButton("Toggle Button");
    b13.setSelected(true);
    b14.setSelected(false);
    b131.setSelected(true);
    b141.setSelected(false);
    b131.setEnabled(false);
    b141.setEnabled(false);

    JLabel b15=new JLabel("Label");
    JLabel b16=new JLabel("Label");
    b16.setEnabled(false);

    JCheckBox b17=new JCheckBox("CheckBox");
    JCheckBox b18=new JCheckBox("CheckBox");
    b17.setSelected(true);
    JCheckBox b171=new JCheckBox("CheckBox");
    JCheckBox b181=new JCheckBox("CheckBox");
    b17.setSelected(true);
    b171.setSelected(true);
    b171.setEnabled(false);
    b181.setEnabled(false);

    JRadioButton b19=new JRadioButton("RadioButton");
    JRadioButton b1A=new JRadioButton("RadioButton");
    ButtonGroup bg=new ButtonGroup();
    bg.add(b19);
    bg.add(b1A);
    b19.setSelected(true);
    JRadioButton b191=new JRadioButton("RadioButton");
    JRadioButton b1A1=new JRadioButton("RadioButton");
    b191.setSelected(true);
    b191.setEnabled(false);
    b1A1.setEnabled(false);

    tab1.add(b11);
    tab1.add(b12);
    tab1.add(b111);//new JLabel());
    tab1.add(b121);//new JLabel());

    tab1.add(b13);
    tab1.add(b14);
    tab1.add(b131);
    tab1.add(b141);

    tab1.add(b15);
    tab1.add(b16);
    tab1.add(new JLabel());
    tab1.add(new JLabel());

    tab1.add(b17);
    tab1.add(b18);
    tab1.add(b171);
    tab1.add(b181);

    tab1.add(b19);
    tab1.add(b1A);
    tab1.add(b191);
    tab1.add(b1A1);

    JPanel tab2=new JPanel();
    tab2.setLayout(new SlafGridLayout(3,5,5,true,true,false,false));
    tab2.setBorder(new EmptyBorder(5,5,5,5));
    tab2.add(new JTextField("TextField"));
    tab2.add(new JPasswordField("Password"));
    try
    {
      tab2.add((JComponent)Class.forName
	       ("javax.swing.JSpinner").newInstance());
    }
    catch(Exception ex)
    {
      tab2.add(new JLabel(""));
    }

    JScrollPane b23=new JScrollPane(new JTextArea("TextArea   \n\n\n\n\n\n\n\n\n\n\n\n\n;-)"));
    b23.setPreferredSize(new Dimension(90,130));
    tab2.add(b23);
    JScrollPane b24=new JScrollPane(new JList(THEMES));
    b24.setPreferredSize(new Dimension(90,130));
    tab2.add(b24);
    JScrollPane b25=new JScrollPane(new JTree());
    b25.setPreferredSize(new Dimension(90,130));
    tab2.add(b25);

    JPanel tab3=new JPanel(new BorderLayout());
    tab3.setBorder(new EmptyBorder(5,5,5,5));
    JDesktopPane b31=new JDesktopPane();
    JScrollPane  b30=new JScrollPane(b31);
    b31.setPreferredSize(new Dimension(300,300));
    JInternalFrame b32=new JInternalFrame("Internal Frame",true,true,true,true);
    JInternalFrame b33=new JInternalFrame("Internal Frame",false,false,true,true);
    JInternalFrame b34=new JInternalFrame("Palette")
      {
	public void updateUI()
	{
	  SwingUtilities.updateComponentTreeUI(this.getContentPane());
	  super.updateUI();
	  this.pack();
	}
      };
    b32.setVisible(true);
    b32.setSize(new Dimension(230,100));
    b32.setLocation(20,20);
    b33.setVisible(true);
    b33.setSize(new Dimension(230,100));
    b33.setLocation(70,50);
    b34.setVisible(true);
    b34.setLocation(50,90);
    b34.putClientProperty("JInternalFrame.isPalette",Boolean.TRUE);
    b34.getContentPane().setLayout(new SlafGridLayout(1,0,0));
    b34.getContentPane().add(new JButton("A"));
    b34.getContentPane().add(new JButton("B"));
    b34.getContentPane().add(new JButton("C"));
    b34.getContentPane().add(new JButton("D"));
    b34.pack();
    b31.add(b34);
    b31.add(b33);
    b31.add(b32);
    tab3.add(b30);

    /*    
    try
    {
      b32.setSelected(false);
      b32.setSelected(true);
      b31.setSelectedFrame(b32);
    }
    catch(PropertyVetoException ex) { }
    */

    JPanel tab4=new JPanel(new SlafGridLayout(1,5,5,true,true));
    tab4.setBorder(new EmptyBorder(5,5,5,5));
    JMenu b4A=new JMenu("Menu");
    JMenu b4C=new JMenu("Sub-Menu");
    b4A.add(new JMenuItem("MenuItem"));
    b4C.add(new JMenuItem("MenuItem"));
    b4A.add(b4C);
    b4A.addSeparator();
    JCheckBoxMenuItem b4A11=new JCheckBoxMenuItem("CheckBoxMenuItem");
    JCheckBoxMenuItem b4A12=new JCheckBoxMenuItem("CheckBoxMenuItem");
    JRadioButtonMenuItem b4A13=new JRadioButtonMenuItem("RadioButtonMenuItem");
    JRadioButtonMenuItem b4A14=new JRadioButtonMenuItem("RadioButtonMenuItem");
    b4A11.setSelected(true);
    b4A13.setSelected(true);
    b4A12.setEnabled(false);
    b4A14.setEnabled(false);
    b4A.add(b4A11);
    b4A.add(b4A12);
    b4A.add(b4A13);
    b4A.add(b4A14);
    JMenu b4B=new JMenu("Help");
    b4B.add(new JMenuItem("Contents"));
    JMenuBar b41=new JMenuBar();
    b41.add(b4A);
    b41.add(b4B);
    JToolBar b42=new JToolBar();
    //b42.setLayout(new SlafGridLayout(5,1,1));
    b42.setFloatable(true);
    b42.putClientProperty("JToolBar.isRollover",Boolean.TRUE);
    b42.setToolTipText("ToolBar");

    btTools_=new JButton[4];
    btTools_[0]=new JButton(UIManager.getIcon("FileView.directoryIcon"));
    btTools_[1]=new JButton(UIManager.getIcon("FileView.fileIcon"));
    btTools_[2]=new JButton(UIManager.getIcon("FileView.hardDriveIcon"));
    btTools_[3]=new JButton(UIManager.getIcon("FileView.floppyDriveIcon"));
    b42.add(btTools_[0]);
    b42.add(btTools_[1]);
    b42.addSeparator();
    b42.add(btTools_[2]);
    b42.add(btTools_[3]);
    btTools_[2].setEnabled(false);

    final JProgressBar b43=new JProgressBar();
    b43.setToolTipText("ProgressBar");
    b43.setValue(50);
    b43.setStringPainted(true);
    b43.setString("50%");
    final JSlider b44=new JSlider();
    b44.setToolTipText("Slider");
    b44.addChangeListener(new ChangeListener()
      {
	public void stateChanged(ChangeEvent _evt)
	{
	  int v=b44.getValue();
	  b43.setValue(v);
	  b43.setString(v+"%");
	}
      });

    tab4.add(b41);
    tab4.add(b42);
    tab4.add(new JSeparator());
    tab4.add(b43);
    tab4.add(b44);
     
    //JPanel tab5=new JPanel();
    //tab5.setBorder(new EmptyBorder(5,5,5,5));
    //tab5.setLayout(new BorderLayout());

    int l=THEMES.length;
    String[][] data=new String[l][2];
    for(int i=0;i<l;i++)
    {
      data[i][0]=THEMES[i];
      data[i][1]=THEMES[i].toLowerCase();
    }
    JScrollPane tab5=new JScrollPane
      (new JTable(data,
		  new String[] { "Theme","Name" }));

    tpMain_=new JTabbedPane();
    tpMain_.addTab("Widgets",null,tab1);
    tpMain_.addTab("Fields" ,null,tab2);
    tpMain_.addTab("Frames" ,null,tab3);
    tpMain_.addTab("Bars"   ,null,tab4);
    tpMain_.addTab("Table"  ,null,tab5);

    tpMain_.setToolTipText
      ("This is a very long tool tip.");
  }

  public void actionPerformed(ActionEvent _evt)
  {
    JComponent source=(JComponent)_evt.getSource();
    if(source==cbTheme_)
    {
      String t=(String)cbTheme_.getSelectedItem();
      if(t!=null) updateLnf(t.toLowerCase());
    }
  }

  public static void main(String[] args)
  {
    // EXPERIMENTAL
    //JDialog.setDefaultLookAndFeelDecorated(true);
    //JFrame. setDefaultLookAndFeelDecorated(true);

    String theme=(args.length>0 ? args[0] : null);

    try
    {
      UIManager.setLookAndFeel(new SlafLookAndFeel(theme));
    }
    catch(Exception ex)
    {
      System.err.println("SLF: Error when starting SLAF");
      ex.printStackTrace();
    }

    new SlafDemo(theme);

    /*
    JFrame f=new JFrame("Slaf Demo");
    JPanel p=new JPanel(new SlafGridLayout(2,5,5));

    p.setBorder(new EmptyBorder(5,5,5,5));
    p.add(new JButton("SLAF"));
    p.add(new JToggleButton(UIManager.getIcon("FileView.directoryIcon")));
    p.add(new JTextField("0.14          "));
    p.add(new JPasswordField("ABC"));
    JScrollPane sp=new JScrollPane
	(new JList(new String[] { " 1 "," 2 "," 3 "," 4 "," 3 "," 3 "," 3 "}));
    sp.setPreferredSize(new Dimension(50,65));
    p.add(sp);
    p.add(new JLabel("L&F"));
    p.add(new JSpinner());
    p.add(new JProgressBar());
    f.setContentPane(p);
    f.setSize(190,190);
    f.show();
    */
  }
}
