/**
 * @modification $Date: 2004/01/26 18:17:33 $
 * @statut       unstable
 * @file         com/memoire/slaf/SlafDesktopIconUI.java
 * @version      0.20
 * @author       Guillaume Desnoix
 * @email        guillaume-desnoix@memoire.com
 * @license      GNU General Public License 2 (GPL2)
 * @copyright    1999-2004 Guillaume Desnoix
 */

package com.memoire.slaf;
import com.memoire.slaf.*;


import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.border.*;
import javax.swing.plaf.*;
import java.beans.*;
import java.util.EventListener;
import java.io.Serializable;
import javax.swing.plaf.basic.BasicDesktopIconUI;

public class SlafDesktopIconUI
       extends BasicDesktopIconUI implements Serializable
{
  JLabel        name;
  JLabel        pict;
  TitleListener titleListener;

  private static Font  font_;
  private static Color bg_;
  private static Color fg_;

  public static ComponentUI createUI(JComponent c)
  {
    return new SlafDesktopIconUI();
  }

  public SlafDesktopIconUI()
  {
  }

  public void installUI(JComponent c)
  {
    super.installUI(c);
    LookAndFeel.installColorsAndFont
      (c,"DesktopIcon.background",
         "DesktopIcon.foreground",
         "DesktopIcon.font");
    LookAndFeel.installBorder(c,"DesktopIcon.border");
    SlafUtils.setOpaque(c,false);
  }

  protected void installDefaults()
  {
    super.installDefaults();

    try
    {
      font_=SlafManager.getFont("DesktopIcon.font");
      bg_  =new Color(SlafManager.getColor("DesktopIcon.background").getRGB());
      fg_  =new Color(SlafManager.getColor("DesktopIcon.foreground").getRGB());
    }
    catch(Exception ex) { }

    // Dimension d=SlafManager.getSize("DesktopIcon.preferredSize");
    // if(d!=null) desktopIcon.setPreferredSize(d);
  }

  private final boolean isFrameSelected()
    { return desktopIcon.getInternalFrame().isSelected(); }
   
  protected void installComponents()
  {
    // SlafUtils.trace(" installComponents");

    JInternalFrame frame  =desktopIcon.getInternalFrame();
    Icon           icon   =frame.getFrameIcon();
    String         title  =frame.getTitle();

    name=new JLabel(title)
    {
      public void setFont(Font ft)
      {
	try
	{
	  ft=SlafManager.getFont("DesktopIcon.font",font_);
	  if(isFrameSelected())
	    ft=SlafManager.getFont("DesktopIcon.selectionFont",ft);
	}
	catch(Exception ex) { }
	super.setFont(ft);
	//super.setFont(font_);
      }
      public void setBackground(Color c)
      {
	try
	{
	  c=SlafManager.getColor("DesktopIcon.labelBackground",bg_);
	  if(isFrameSelected())
	    c=SlafManager.getColor("DesktopIcon.selectionLabelBackground",
				   SlafUtils.darken(c,1));
	}
	catch(Exception ex) { c=Color.blue; }
	super.setBackground(c);
      }
      public void setForeground(Color c)
      {
	try
	{
	  c=SlafManager.getColor("DesktopIcon.labelForeground",fg_);
	  if(isFrameSelected())
	    c=SlafManager.getColor("DesktopIcon.selectionLabelForeground",
				   SlafUtils.darken(c,1));
	}
	catch(Exception ex) { c=Color.green; }
	super.setForeground(c);
      }
      public void setBorder(Border b)
      {
	try
	{
	  b=SlafManager.getBorder("DesktopIcon.labelBorder",null);
	  if(isFrameSelected())
	    b=SlafManager.getBorder("DesktopIcon.selectedLabelBorder",b);
	}
	catch(Exception ex) { b=null; }
	super.setBorder(b);
      }
      public void setOpaque(boolean b)
      { 
	try
	{
	  b=SlafManager.getBoolean("DesktopIcon.labelOpaque",true);
	  if(isFrameSelected())
	    b=SlafManager.getBoolean("DesktopIcon.selectionLabelOpaque",b);
	}
	catch(Exception ex) { b=true; }
	super.setOpaque(b);
      }
    };

    pict=new JLabel(icon)
    {
      public void setFont(Font ft)
      {
	try
	{
	  ft=SlafManager.getFont("DesktopIcon.font",font_);
	  if(isFrameSelected())
	    ft=SlafManager.getFont("DesktopIcon.selectionFont",ft);
	}
	catch(Exception ex) { }
	super.setFont(ft);
	//super.setFont(font_);
      }
      public void setBackground(Color c)
      {
	try
	{
	  c=SlafManager.getColor("DesktopIcon.iconBackground",bg_);
	  if(isFrameSelected())
	    c=SlafManager.getColor("DesktopIcon.selectionIconBackground",
				   SlafUtils.darken(c,1));
	}
	catch(Exception ex) { c=Color.blue; }
	super.setBackground(c);
      }
      public void setForeground(Color c)
      {
	try
	{
	  c=SlafManager.getColor("DesktopIcon.iconForeground",fg_);
	  if(isFrameSelected())
	    c=SlafManager.getColor("DesktopIcon.selectionIconForeground",
				   SlafUtils.darken(c,1));
	}
	catch(Exception ex) { c=Color.green; }
	super.setForeground(c);
      }
      public void setBorder(Border b)
      {
	try
	{
	  b=SlafManager.getBorder("DesktopIcon.iconBorder",null);
	  if(isFrameSelected())
	    b=SlafManager.getBorder("DesktopIcon.selectedIconBorder",b);
	}
	catch(Exception ex) { b=null; }
	super.setBorder(b);
      }
      public void setOpaque(boolean b)
      { 
	try
	{
	  b=SlafManager.getBoolean("DesktopIcon.iconOpaque",false);
	  if(isFrameSelected())
	    b=SlafManager.getBoolean("DesktopIcon.selectionIconOpaque",b);
	}
	catch(Exception ex) { b=false; }
	super.setOpaque(b);
      }
    };

    updateComponents();

    BorderLayout lo=new BorderLayout();
    lo.setVgap(0);
    lo.setHgap(0);
    desktopIcon.setLayout(lo);

    String p;

    p=SlafManager.getPosition("DesktopIcon.labelPosition","South");
    if(!p.equals("None")) desktopIcon.add(p,name);

    p=SlafManager.getString("DesktopIcon.iconPosition","North");
    if(!p.equals("None")) desktopIcon.add(p,pict);
    if(p.equals("North")||p.equals("South"))
      name.setHorizontalAlignment(JLabel.CENTER);

    /*
    Dimension d=SlafManager.getSize("DesktopIcon.preferredSize");
    if(d!=null) desktopIcon.setPreferredSize(d);
    desktopIcon.doLayout();
    desktopIcon.validate();
    */

    desktopIcon.setToolTipText("".equals(title) ? null : title);
    titleListener=new TitleListener();
    desktopIcon.getInternalFrame().addPropertyChangeListener(titleListener);
  }

  protected void uninstallComponents()
  {
    desktopIcon.remove(name);
    desktopIcon.remove(pict);
    desktopIcon.setLayout(null);
    desktopIcon.getInternalFrame().removePropertyChangeListener(titleListener);
  }

  protected void updateComponents()
  {
    name.setFont      (null);
    name.setForeground(null);
    name.setBackground(null);
    name.setBorder    (null);
    name.setOpaque    (false);
    pict.setFont      (null);
    pict.setForeground(null);
    pict.setBackground(null);
    pict.setBorder    (null);
    pict.setOpaque    (false);

    //SlafUtils.trace("UPDATE COMPONENTS "+name.isOpaque());
    if(desktopIcon!=null) desktopIcon.repaint();
  }

  public void update(Graphics g, JComponent b)
  {
    SlafUtils.paintBackground(g,b);
    paint(g,b);
  }

  public Dimension getPreferredSize(JComponent c)
  {
    // SlafUtils.trace(" preferredSize");
    Dimension r=c.getLayout().preferredLayoutSize(c);
    int       w=SlafManager.getInteger("DesktopIcon.width");
    int       h=SlafManager.getInteger("DesktopIcon.height");
    if(w>0) r.width=w;
    if(h>0) r.height=h;
    return r;
    // return c.getLayout().preferredLayoutSize(c);
  }

  /*
  public Dimension getPreferredSize(JComponent c)
  {
    Dimension d1=pict.getPreferredSize();
    Dimension d2=name.getPreferredSize();

    int w=Math.max(d1.width,d2.width);
    int h=d1.height+d2.height;

    // w=(w+79)/80*80;
    w+=6;
    return new Dimension(w,h);
  }
  */

  class TitleListener implements PropertyChangeListener, Serializable
  {
    public void propertyChange (PropertyChangeEvent e)
    {
      // SlafUtils.trace("%%% PS="+e.getSource());
      // SlafUtils.trace("%%% PN="+e.getPropertyName());
      // SlafUtils.trace("%%% OV="+e.getOldValue());
      // SlafUtils.trace("%%% NV="+e.getNewValue());

      if(e.getPropertyName().equals("title"))
      {
	name.setText((String)e.getNewValue());
      }
      else
      if(e.getPropertyName().equals("icon"))
      {
	JInternalFrame f=(JInternalFrame)e.getSource();
	pict.setIcon(f.getFrameIcon());
      }
      else
      if(e.getPropertyName().equals("selected"))
      {
	updateComponents();
      }
    }
  }
}
