/**
 * @modification $Date: 2004/01/26 18:17:33 $
 * @statut       unstable
 * @file         com/memoire/slaf/SlafButtonBorder.java
 * @version      0.20
 * @author       Guillaume Desnoix
 * @email        guillaume-desnoix@memoire.com
 * @license      GNU General Public License 2 (GPL2)
 * @copyright    1999-2004 Guillaume Desnoix
 */

package com.memoire.slaf;
import com.memoire.slaf.*;

import java.awt.*;
import java.awt.event.*;

import javax.swing.*;
import javax.swing.border.*;
import javax.swing.plaf.*;
import javax.swing.text.*;

public class SlafExtraBorder extends AbstractBorder
{
  private String ui_;
  private Border normal;
  private Border normalFocus;
  private Border main;
  private Border mainFocus;

  public SlafExtraBorder(String _ui)
    { ui_=_ui; }

  private void setBorders(Component c)
  {
    if(ui_==null)
    {
      ui_=((JComponent)c).getUIClassID();
      if(ui_.endsWith("UI")) ui_=ui_.substring(0,ui_.length()-2);
    }

    if(normal     ==null) normal     =SlafManager.getBorder(ui_+".normalBorder"      );
    if(normalFocus==null) normalFocus=SlafManager.getBorder(ui_+".normalFocusBorder" );
    if(main       ==null) main       =SlafManager.getBorder(ui_+".defaultBorder"     );
    if(mainFocus  ==null) mainFocus  =SlafManager.getBorder(ui_+".defaultFocusBorder");

    if(normal     ==null) normal     =new LineBorder(Color.lightGray,3);
    if(normalFocus==null) normalFocus=new LineBorder(Color.red      ,3);
    if(main       ==null) main       =normal;
    if(mainFocus  ==null) mainFocus  =normalFocus;
  }

  static boolean applied(Component _c)
  {
    if(_c==null) return false;
    //if(_c.hasFocus())             return true;
    if(!_c.isFocusTraversable())    return false;
    if(!(_c instanceof JComponent)) return true;

    if(  (_c instanceof JTextComponent)
       &&!((JTextComponent)_c).isEditable()) return false;

    Container p=_c.getParent();
    if(p instanceof JComboBox) return true;
    if(p instanceof JToolBar)  return false;

    JComponent c=(JComponent)_c;
    return c.isRequestFocusEnabled();
  }

  public void paintBorder(Component c, Graphics g, int x, int y, int w, int h)
  {
    if(!applied(c)) return;

    setBorders(c);

    boolean isFocus =false;
    boolean isNormal=true;

    if(c instanceof JButton   ) isNormal=!((JButton)c).isDefaultButton();
    if(c instanceof JComponent) isFocus =SlafUtils.hasFocus(c);
    //((JComponent)c).hasFocus();

    Border b=main;
         if(isFocus&&isNormal) b=normalFocus;
    else if(isFocus)           b=mainFocus;
    else if(isNormal)          b=normal;
    b.paintBorder(c,g,x,y,w,h);
  }

  public Insets getBorderInsets(Component _c)
  {
    return getBorderInsets(_c,new Insets(0,0,0,0));
  }

  public Insets getBorderInsets(Component _c, Insets _insets)
  {
    if(!applied(_c))
    {
      _insets.top   =0;
      _insets.left  =0;
      _insets.bottom=0;
      _insets.right =0;
    }
    else
    {
      setBorders(_c);

      Insets ebi=normal     .getBorderInsets(_c);
      Insets pbi=main       .getBorderInsets(_c);
      Insets sbi=normalFocus.getBorderInsets(_c);
      Insets rbi=mainFocus  .getBorderInsets(_c);

      int ti=SlafUtils.max(ebi.top   ,pbi.top   ,sbi.top   ,rbi.top   );
      int bi=SlafUtils.max(ebi.bottom,pbi.bottom,sbi.bottom,rbi.bottom);
      int li=SlafUtils.max(ebi.left  ,pbi.left  ,sbi.left  ,rbi.left  );
      int ri=SlafUtils.max(ebi.right ,pbi.right ,sbi.right ,rbi.right );

      _insets.top   =ti;
      _insets.left  =li;
      _insets.bottom=bi;
      _insets.right =ri;
    }

    return _insets;
  }
}
