/**
 * @modification $Date: 2004/01/26 18:17:33 $
 * @statut       unstable
 * @file         com/memoire/slaf/SlafFileChooserUI.java
 * @version      0.20
 * @author       Guillaume Desnoix
 * @email        guillaume-desnoix@memoire.com
 * @license      GNU General Public License 2 (GPL2)
 * @copyright    1999-2004 Guillaume Desnoix
 */

package com.memoire.slaf;
import com.memoire.slaf.*;

import javax.swing.*;
import javax.swing.border.*;
import javax.swing.filechooser.*;
import javax.swing.event.*;
import javax.swing.plaf.*;
import javax.swing.plaf.basic.*;

import java.awt.*;
import java.awt.event.*;
import java.beans.*;
import java.io.*;
import java.util.*;

public class SlafFileChooserUI extends BasicFileChooserUI
{
  private JPanel centerPanel;
  private JPanel bottomPanel;
  private JPanel buttonPanel;

  private JComboBox directoryComboBox;
  private DirectoryComboBoxModel directoryComboBoxModel;
  private Action directoryComboBoxAction=new DirectoryComboBoxAction();

  private FilterComboBoxModel filterComboBoxModel;
  
  private JTextField filenameTextField;
  
  private JList list;

  private JButton approveButton;
  private JButton cancelButton;

  private JComboBox filterComboBox;

  private JPanel bodyPanel=null;
  
  private static final Insets shrinkwrap=new Insets(0,0,0,0);

  private static int PREF_WIDTH =400;
  private static int PREF_HEIGHT=400;
  private static Dimension PREF_SIZE
    =new Dimension(PREF_WIDTH, PREF_HEIGHT);

  private static int MIN_WIDTH =300;
  private static int MIN_HEIGHT=250;
  private static Dimension MIN_SIZE
    =new Dimension(MIN_WIDTH, MIN_HEIGHT);

  private static int LIST_MIN_WIDTH =200;
  private static int LIST_MIN_HEIGHT=150;
  private static Dimension LIST_MIN_SIZE
    =new Dimension(LIST_MIN_WIDTH, LIST_MIN_HEIGHT);

  private int    lookInLabelMnemonic=0;
  private String lookInLabelText=null;

  private int    fileNameLabelMnemonic=0;
  private String fileNameLabelText=null;

  private int    filesOfTypeLabelMnemonic=0;
  private String filesOfTypeLabelText=null;

  private String upFolderToolTipText=null;
  private String upFolderAccessibleName=null;

  private String homeFolderToolTipText=null;
  private String homeFolderAccessibleName=null;

  private String newFolderToolTipText=null;
  private String newFolderAccessibleName=null;

  private String listViewButtonToolTipText=null;
  private String listViewButtonAccessibleName=null;

  private String detailsViewButtonToolTipText=null;
  private String detailsViewButtonAccessibleName=null;

  public static ComponentUI createUI(JComponent c)
  {
    return new SlafFileChooserUI((JFileChooser)c);
  }

  public SlafFileChooserUI(JFileChooser filechooser)
  {
    super(filechooser);
    //filechooser.setFileView(fileview_);
  }

  /*
  protected BasicDirectoryModel model_=null;

  protected void createModel()
  {
    model_=new BasicDirectoryModel(null)
      {
	public void validateFileCache()
	{
	  boolean b=false;
	  try
	  {
	    if(getFileChooser().getCurrentDirectory().canRead()) b=true;
	  }
	  catch(Throwable ex) { System.err.println(ex); }
	  if(b) super.validateFileCache();
	  try
	  {
	    super.validateFileCache();
	  }
	  catch(Throwable ex) { System.err.println(ex); }
	}
      };
  }

  public BasicDirectoryModel getModel()
  {
    return model_;
  }
  */

  //JButton UP_BUTTON,HOME_BUTTON;

  public void installComponents(JFileChooser fc)
  {
    BorderLayout fcLayout=new BorderLayout();
    fcLayout.setVgap(5);
    fc.setLayout(fcLayout);
    fc.setBorder(new EmptyBorder(5,5,5,5));

    JPanel topPanel=new JPanel();
    BorderLayout topLayout=new BorderLayout();
    topLayout.setHgap(5);
    topPanel.setLayout(topLayout);

    fc.add("North",topPanel);

    JLabel l=new JLabel(lookInLabelText);
    l.setDisplayedMnemonic(lookInLabelMnemonic);
    l.setAlignmentX(JComponent.LEFT_ALIGNMENT);
    l.setAlignmentY(JComponent.CENTER_ALIGNMENT);
    topPanel.add("West",l);
    
    directoryComboBox=new JComboBox();
    directoryComboBox.putClientProperty
	("JComboBox.lightweightKeyboardNavigation","Lightweight");
    l.setLabelFor(directoryComboBox);
    directoryComboBoxModel=createDirectoryComboBoxModel(fc);
    directoryComboBox.setModel(directoryComboBoxModel);
    directoryComboBox.addActionListener(directoryComboBoxAction);
    directoryComboBox.setRenderer(createDirectoryComboBoxRenderer(fc));
    directoryComboBox.setAlignmentX(JComponent.LEFT_ALIGNMENT);
    directoryComboBox.setAlignmentY(JComponent.CENTER_ALIGNMENT);

    topPanel.add("Center",directoryComboBox);

    JPanel movePanel=new JPanel();
    SlafGridLayout moveLayout=new SlafGridLayout();
    moveLayout.setColumns(5);
    moveLayout.setHgap(2);
    movePanel.setLayout(moveLayout);
    topPanel.add("East",movePanel);

    JButton b=new JButton(upFolderIcon);
    b.setToolTipText(upFolderToolTipText);
    b.getAccessibleContext().setAccessibleName(upFolderAccessibleName);
    b.setAlignmentX(JComponent.LEFT_ALIGNMENT);
    b.setAlignmentY(JComponent.CENTER_ALIGNMENT);
    b.setMargin(shrinkwrap);
    b.addActionListener(getChangeToParentDirectoryAction());
    movePanel.add(b);

    //UP_BUTTON=b;

    b=new JButton(homeFolderIcon);
    b.setToolTipText(homeFolderToolTipText);
    b.getAccessibleContext().setAccessibleName(homeFolderAccessibleName);
    b.setAlignmentX(JComponent.LEFT_ALIGNMENT);
    b.setAlignmentY(JComponent.CENTER_ALIGNMENT);
    b.setMargin(shrinkwrap);
    b.addActionListener(getGoHomeAction());
    movePanel.add(b);

    //HOME_BUTTON=b;

    b=new JButton(newFolderIcon);
    b.setToolTipText(newFolderToolTipText);
    b.getAccessibleContext().setAccessibleName(newFolderAccessibleName);
    b.setAlignmentX(JComponent.LEFT_ALIGNMENT);
    b.setAlignmentY(JComponent.CENTER_ALIGNMENT);
    b.setMargin(shrinkwrap);
    b.addActionListener(getNewFolderAction());
    movePanel.add(b);

//     JToggleButton tb=new JToggleButton(listViewIcon);
//     tb.setToolTipText(listViewButtonToolTipText);
//     tb.getAccessibleContext().setAccessibleName
//     (listViewButtonAccessibleName);
//     tb.setSelected(true);
//     //tb.setEnabled(false);
//     tb.setAlignmentX(JComponent.LEFT_ALIGNMENT);
//     tb.setAlignmentY(JComponent.CENTER_ALIGNMENT);
//     tb.setMargin(shrinkwrap);
//     movePanel.add(tb);

//     tb=new JToggleButton(detailsViewIcon);
//     tb.setToolTipText(detailsViewButtonToolTipText);
//     tb.getAccessibleContext().setAccessibleName
//     (detailsViewButtonAccessibleName);
//     //tb.setEnabled(false);
//     tb.setAlignmentX(JComponent.LEFT_ALIGNMENT);
//     tb.setAlignmentY(JComponent.CENTER_ALIGNMENT);
//     tb.setMargin(shrinkwrap);
//     movePanel.add(tb);

    centerPanel=new JPanel(new BorderLayout());
    JPanel p=createList(fc);
    p.setMinimumSize(LIST_MIN_SIZE);
    centerPanel.add(p, BorderLayout.CENTER);
    centerPanel.add(getAccessoryPanel(), BorderLayout.EAST);
    JComponent accessory=fc.getAccessory();
    if(accessory!=null) {
      getAccessoryPanel().add(accessory);
    }
    fc.add("Center",centerPanel);

    bottomPanel=new JPanel();
    bottomPanel.setLayout(new BorderLayout());
    fc.add("South",bottomPanel);

    JPanel labelPanel=new JPanel();
    SlafGridLayout labelLayout=new SlafGridLayout();
    labelLayout.setColumns(2);
    labelLayout.setHfilled(true);
    labelLayout.setCfilled(true);
    labelLayout.setVgap(5);
    labelLayout.setHgap(5);
    labelPanel.setLayout(labelLayout);
    bottomPanel.add("North",labelPanel);

    JLabel fnl=new JLabel(fileNameLabelText);
    fnl.setDisplayedMnemonic(fileNameLabelMnemonic);
    labelPanel.add(fnl,0);

    filenameTextField=new JTextField();
    fnl.setLabelFor(filenameTextField);
    filenameTextField.addActionListener(getApproveSelectionAction());
    File f=fc.getSelectedFile();
    if(f!=null) setFileName(fc.getName(f));
    labelPanel.add(filenameTextField,1);

    JLabel ftl=new JLabel(filesOfTypeLabelText);
    ftl.setDisplayedMnemonic(filesOfTypeLabelMnemonic);
    labelPanel.add(ftl,2);

    filterComboBoxModel=createFilterComboBoxModel();
    fc.addPropertyChangeListener(filterComboBoxModel);
    filterComboBox=new JComboBox(filterComboBoxModel);
    ftl.setLabelFor(filterComboBox);
    filterComboBox.setRenderer(createFilterComboBoxRenderer());
    labelPanel.add(filterComboBox,3);

    buttonPanel=new JPanel();
    buttonPanel.setLayout(new FlowLayout(FlowLayout.RIGHT));
    bottomPanel.add("South",buttonPanel);
    
    approveButton=new JButton(getApproveButtonText(fc));
    approveButton.setIcon(SlafManager.getIcon
			  ("FileChooser.approveButtonIcon"));
    approveButton.setMnemonic(getApproveButtonMnemonic(fc));
    approveButton.addActionListener(getApproveSelectionAction());
    approveButton.setToolTipText(getApproveButtonToolTipText(fc));
    buttonPanel.add(approveButton);

    cancelButton=new JButton(cancelButtonText);
    approveButton.setIcon(SlafManager.getIcon
			  ("FileChooser.cancelButtonIcon"));
    cancelButton.setMnemonic(cancelButtonMnemonic);
    cancelButton.setToolTipText(cancelButtonToolTipText);
    cancelButton.addActionListener(getCancelSelectionAction());
    buttonPanel.add(cancelButton);

    /*
    try
    {
      new File(System.getProperty("user.dir")).canRead();
      UP_BUTTON=HOME_BUTTON=null;
    }
    catch(SecurityException ex)
    {
      directoryComboBox.removeActionListener(directoryComboBoxAction);
      directoryComboBox.setEnabled(false);
    }
    */
  }

  protected void installStrings(JFileChooser fc)
  {
    super.installStrings(fc);

    lookInLabelMnemonic            =SlafManager.getInteger("FileChooser.lookInLabelMnemonic"); 
    lookInLabelText                =SlafManager.getString("FileChooser.lookInLabelText");
	
    fileNameLabelMnemonic          =SlafManager.getInteger("FileChooser.fileNameLabelMnemonic");  
    fileNameLabelText              =SlafManager.getString("FileChooser.fileNameLabelText"); 
	
    filesOfTypeLabelMnemonic       =SlafManager.getInteger("FileChooser.filesOfTypeLabelMnemonic");  
    filesOfTypeLabelText           =SlafManager.getString("FileChooser.filesOfTypeLabelText"); 
	
    upFolderToolTipText            =SlafManager.getString("FileChooser.upFolderToolTipText");
    upFolderAccessibleName         =SlafManager.getString("FileChooser.upFolderAccessibleName"); 
	
    homeFolderToolTipText          =SlafManager.getString("FileChooser.homeFolderToolTipText");
    homeFolderAccessibleName       =SlafManager.getString("FileChooser.homeFolderAccessibleName"); 
	
    newFolderToolTipText           =SlafManager.getString("FileChooser.newFolderToolTipText");
    newFolderAccessibleName        =SlafManager.getString("FileChooser.newFolderAccessibleName"); 
	
    listViewButtonToolTipText      =SlafManager.getString("FileChooser.listViewButtonToolTipText"); 
    listViewButtonAccessibleName   =SlafManager.getString("FileChooser.listViewButtonAccessibleName"); 
	
    detailsViewButtonToolTipText   =SlafManager.getString("FileChooser.detailsViewButtonToolTipText"); 
    detailsViewButtonAccessibleName=SlafManager.getString("FileChooser.detailsViewButtonAccessibleName"); 
  }

    protected JPanel createList(JFileChooser fc) {
	JPanel p=new JPanel(new BorderLayout());
	list=new JList();
	list.setCellRenderer(new FileRenderer());
	list.setModel(getModel());
	list.addListSelectionListener(createListSelectionListener(fc));
	list.addMouseListener(createDoubleClickListener(fc, list));
	list.addMouseListener(createSingleClickListener(fc, list));
	JScrollPane scrollpane=new JScrollPane(list);

	p.add(scrollpane, BorderLayout.CENTER);
	return p;
    }

    private MouseListener createSingleClickListener(JFileChooser fc, JList list) {
	return new SingleClickListener(list);
    }

    int lastIndex=-1;
    boolean editing=false;
    int editX=20;
    int editWidth=200;

    private void setEditIndex(int i) {
	lastIndex=i;
    }

    private void resetEditIndex() {
	lastIndex=-1;
    }

    private void cancelEdit() {
	editing=false;
	if(editCell!=null) {
	    list.remove(editCell);
	}
    }

    JTextField editCell=null;
    protected class SingleClickListener extends MouseAdapter {
	JList list;

	public  SingleClickListener(JList list) {
	    this.list=list;
	    editCell=new JTextField();
	    editCell.addActionListener(new EditActionListener());
	}

	public void mouseClicked(MouseEvent e) {
	    if (e.getClickCount() == 1) {
		int index=list.locationToIndex(e.getPoint());
                if(index >= 0 && lastIndex == index && editing == false) {
		    editing=true;
		    Rectangle r=list.getCellBounds(index, index);
		    list.add(editCell);
		    File f=(File) list.getSelectedValue();
		    editCell.setText(getFileChooser().getName(f));
		    editCell.setBounds(editX + r.x, r.y, editWidth, r.height);
		    editCell.selectAll();
		} else {
		    if(index >= 0) {
			setEditIndex(index);
		    } else {
			resetEditIndex();
		    }
		    cancelEdit();
		}
	    } else {
		resetEditIndex();
		cancelEdit();
	    }
	    list.repaint();
	    
	}

    }

    class EditActionListener implements ActionListener
    {
	public void actionPerformed(ActionEvent e)
	{
	    JTextField tf=(JTextField) e.getSource();
	    File        f=(File) list.getSelectedValue();
	    String newFileName=tf.getText();
	    newFileName=newFileName.trim();
	    if(!newFileName.equals(getFileChooser().getName(f)))
	    {
              File f2=getFileChooser().getFileSystemView().createFileObject
		  (getFileChooser().getCurrentDirectory(), newFileName);

		if(f.renameTo(f2))
		    rescanCurrentDirectory(getFileChooser());
		else
		    ; // FAILURE
	    }
	    cancelEdit();
	    list.repaint();
	} 
    }

    protected class FileRenderer extends SlafListCellRenderer
      //DefaultListCellRenderer
    {
      public Component getListCellRendererComponent
      (JList list, Object value, int index, boolean isSelected, boolean cellHasFocus)
      {
	super.getListCellRendererComponent(list, value, index, isSelected, cellHasFocus);
	File file=(File) value;
	String fileName=
	  //file.getName();
	  getFileChooser().getName(file);
	setText(fileName);

	Icon icon=getFileChooser().getIcon(file);
	setIcon(icon);

	if(isSelected) editX=(icon!=null ? icon.getIconWidth() : 0)+4;
	return this;
      }
    }

    public void uninstallUI(JComponent c) {
	// Remove listeners
	c.removePropertyChangeListener(filterComboBoxModel);
	cancelButton.removeActionListener(getCancelSelectionAction());
	approveButton.removeActionListener(getApproveSelectionAction());
	filenameTextField.removeActionListener(getApproveSelectionAction());

	super.uninstallUI(c);
    }

    public Dimension getPreferredSize(JComponent c) {
	return PREF_SIZE;
    }

    public Dimension getMinimumSize(JComponent c) {
	return MIN_SIZE;
    }

    public Dimension getMaximumSize(JComponent c) {
	return new Dimension(Short.MAX_VALUE,Short.MAX_VALUE);
    }

    void setFileSelected() {
	File f=getFileChooser().getSelectedFile();
	if(f!=null && getModel().contains(f)) {
	    list.setSelectedIndex(getModel().indexOf(f));
	    list.ensureIndexIsVisible(list.getSelectedIndex());
	} else {
	    list.clearSelection();
	}
    }

    protected void removeControlButtons()
    {
	bottomPanel.remove(buttonPanel);
    }

    protected void addControlButtons()
    {
	bottomPanel.add(buttonPanel);
    }

    private void doControlButtonsChanged(PropertyChangeEvent e)
    {
	if(getFileChooser().getControlButtonsAreShown())
	    addControlButtons();
	else
	    removeControlButtons();
    }

    public PropertyChangeListener createPropertyChangeListener(JFileChooser fc)
    {
	return new PropertyChangeListener() {
	    public void propertyChange(PropertyChangeEvent e) {
		String prop=e.getPropertyName();
		if(prop.equals(JFileChooser.SELECTED_FILE_CHANGED_PROPERTY)) {
		    cancelEdit();
		    File f=(File) e.getNewValue();
		    if(f!=null) {
			setFileName(getFileChooser().getName(f));
		    } else {
			setFileName(null);
		    }
		    setFileSelected();
		}
		else
		if(prop.equals(JFileChooser.DIRECTORY_CHANGED_PROPERTY))
		{
		    cancelEdit();
		    resetEditIndex();
		    clearIconCache();
		    list.clearSelection();
		    File currentDirectory=getFileChooser().getCurrentDirectory();
		    if(currentDirectory!=null)
		    {
		      directoryComboBoxModel.addItem(currentDirectory);

		      FileSystemView fsv=getFileChooser().getFileSystemView();
		      boolean canwrite=
			//fsv.isFileSystem(currentDirectory)&&
			currentDirectory.canWrite();
		      getNewFolderAction().setEnabled(canwrite);
		      getChangeToParentDirectoryAction().setEnabled
			(!fsv.isRoot(currentDirectory));
		    }
		}
		else
		if(prop.equals(JFileChooser.FILE_FILTER_CHANGED_PROPERTY) || 
		   prop.equals(JFileChooser.FILE_SELECTION_MODE_CHANGED_PROPERTY))
		{
		    cancelEdit();
		    resetEditIndex();
		    clearIconCache();
		    list.clearSelection();
		}
		else
		if(prop == JFileChooser.ACCESSORY_CHANGED_PROPERTY)
		{
		    if(getAccessoryPanel()!=null) {
			if(e.getOldValue()!=null) {
			    getAccessoryPanel().remove((JComponent) e.getOldValue());
			}
			JComponent accessory=(JComponent) e.getNewValue();
			if(accessory!=null) {
			    getAccessoryPanel().add(accessory, BorderLayout.CENTER);
			}
		    }
		}
		else
		if(prop == JFileChooser.APPROVE_BUTTON_TEXT_CHANGED_PROPERTY ||
			  prop == JFileChooser.DIALOG_TYPE_CHANGED_PROPERTY) {
		    JFileChooser chooser=getFileChooser();
		    approveButton.setText(getApproveButtonText(chooser));
		    approveButton.setToolTipText(getApproveButtonToolTipText(chooser));
		    approveButton.setMnemonic(getApproveButtonMnemonic(chooser));
		}
		else
		if(prop.equals(JFileChooser.APPROVE_BUTTON_MNEMONIC_CHANGED_PROPERTY))
		{
		    approveButton.setMnemonic(getApproveButtonMnemonic(getFileChooser()));
		}		
		else
		if(prop.equals(JFileChooser.CONTROL_BUTTONS_ARE_SHOWN_CHANGED_PROPERTY))
		{
		    doControlButtonsChanged(e);
		}
	    }
	};
    }

    public void ensureFileIsVisible(JFileChooser fc, File f) {
	if(getModel().contains(f)) {
	    list.ensureIndexIsVisible(getModel().indexOf(f));
	}
    }

    public void rescanCurrentDirectory(JFileChooser fc) {
	getModel().invalidateFileCache();
	getModel().validateFileCache();
    }

    public String getFileName() {
	if(filenameTextField!=null) {
	    return filenameTextField.getText();
	} else {
	    return null;
	}
    }

    public void setFileName(String filename) {
	if(filenameTextField!=null) {
	    filenameTextField.setText(filename);
	}
    }

    public String getDirectoryName() {
	// get the name from the directory combobox
	return null;
    }

    public void setDirectoryName(String dirname) {
	// set the name in the directory combobox
    }

    protected DirectoryComboBoxRenderer createDirectoryComboBoxRenderer(JFileChooser fc) {
      return new DirectoryComboBoxRenderer();
    }

    // Renderer for DirectoryComboBox
  class DirectoryComboBoxRenderer
    extends SlafComboBoxCellRenderer
	    //DefaultListCellRenderer
  {
    IndentIcon ii=new IndentIcon();

    public Component getListCellRendererComponent(JList list, Object value,
						  int index, boolean isSelected,
						  boolean cellHasFocus)
    {
      super.getListCellRendererComponent(list, value, index,
					   isSelected, cellHasFocus);
      File directory=(File)value;
      if(directory == null)
      {
	setText("");
	return this;
      }

      String fileName=directory.getName();
      //directory.getAbsolutePath();
      //fileview_.getName(directory);
      //getFileChooser().getFileSystemView().getSystemDisplayName(directory);
      //getFileChooser().getFileView().getName(directory);
      //directory.getPath();
      //getFileChooser().getName(directory);
      setText(fileName);
      
      // Find the depth of the directory
      int depth=0;
      /*
      for(int i=fileName.length()-1;i>0;i--)
	if(fileName.charAt(i)==File.separatorChar)
	  depth++;
      if(fileName.length()>1) depth++;
      */
      if(index!=-1) {
	File f=directory;
	while(f.getParent()!=null)
	  {
	    depth++;
	    f=getFileChooser().getFileSystemView().createFileObject
	      (f.getParent());
	  }
      }

      Icon icon=getFileChooser().getIcon(directory);
      //if(roots.contains(directory))
      //if(depth==0)

      /*
	if(fileName.equals(""+File.separator))
	icon=SlafManager.getIcon("FileView.hardDriveIcon",icon);
	else
	icon=SlafManager.getIcon("FileView.directoryIcon",icon);
      */

      ii.icon=icon;
      ii.depth=depth;
      
      setIcon(ii);
      
      return this;
    }
  }

  public static final class IndentIcon implements Icon
  {
    private static final int space=10;

    Icon icon =null;
    int  depth=0;
    
    public void paintIcon(Component c, Graphics g, int x, int y)
    {
      if(icon!=null) icon.paintIcon(c, g, x+depth*space, y);
    }

    public int getIconWidth()
    {
      return (icon!=null ? icon.getIconWidth() : 0)+depth*space;
    }
    
    public int getIconHeight()
    {
      return (icon!=null ? icon.getIconHeight() : 16);
    }
  }

    protected DirectoryComboBoxModel createDirectoryComboBoxModel
      (JFileChooser fc)
    {
      return new DirectoryComboBoxModel();
    }

    protected class DirectoryComboBoxModel
      extends AbstractListModel
      implements ComboBoxModel
    {
      Vector directories=new Vector();
      File selectedDirectory=null;

      public DirectoryComboBoxModel()
      {
	super();

	File dir=getFileChooser().getCurrentDirectory();
	if(dir!=null) addItem(dir);
      }

      private void addItem(File directory)
      {
	if(directory==null) return;

	directories.removeAllElements();

	File[] roots=getFileChooser().getFileSystemView().getRoots();
	for(int i=0; i<roots.length; i++)
	  directories.addElement(roots[i]);

	File canonical=null;
	try
	{
	  canonical=getFileChooser().getFileSystemView().createFileObject
	    (directory.getCanonicalPath());
	}
	catch (IOException ex) { canonical=directory; }

	/*
	  System.err.println("ADD-ITEM: "+directory);
	  System.err.println("        : "+directory.getPath());
	  System.err.println("        : "+directory.getAbsolutePath());
	*/

	  //try
	  //{
	  File f=canonical;
	  Vector path=new Vector(10);
	  while(true)
	  {
	    path.addElement(f);
	    String p=f.getParent();
	    if(p==null) break;
	    f=getFileChooser().getFileSystemView()
	      .createFileObject(p);
          }

	  int index=directories.indexOf(f);
	  if(index>=0)
	  {
	    for(int i=path.size()-2;i>=0;i--)
	      directories.insertElementAt(path.elementAt(i),index+1);
	  }
	  else
	  {
	    for(int i=0;i<path.size();i++)
	      directories.addElement(path.elementAt(i));
	  }
	  //}
	  //catch (FileNotFoundException ex) { }

	  for(int i=0;i<path.size();i++)
	    System.err.println("DEBUG: "+directories.elementAt(i));

	setSelectedItem(canonical);
      }

      public void setSelectedItem(Object selectedDirectory)
      {
	this.selectedDirectory=(File)selectedDirectory;
	fireContentsChanged(this,-1,-1);
      }
      
      public Object getSelectedItem()
      {
	return selectedDirectory;
      }

      public int getSize()
      {
	return directories.size();
      }

      public Object getElementAt(int index)
      {
	return directories.elementAt(index);
      }
    }
  
    protected FilterComboBoxRenderer createFilterComboBoxRenderer() {
	return new FilterComboBoxRenderer();
    }

    public class FilterComboBoxRenderer
      extends SlafComboBoxCellRenderer
    {
      public Component getListCellRendererComponent(JList list,
						    Object value, int index,
						    boolean isSelected,
						    boolean cellHasFocus)
      {
	super.getListCellRendererComponent
	  (list, value, index, isSelected, cellHasFocus);

	javax.swing.filechooser.FileFilter filter=
	  (javax.swing.filechooser.FileFilter)value;
	if(filter!=null) setText(filter.getDescription());
	
	return this;
      }
    }

  protected FilterComboBoxModel createFilterComboBoxModel()
  {
    return new FilterComboBoxModel();
  }

  protected class FilterComboBoxModel
    extends AbstractListModel
    implements ComboBoxModel, PropertyChangeListener
  {
    protected javax.swing.filechooser.FileFilter[] filters;

    protected FilterComboBoxModel()
    {
      super();
      filters=getFileChooser().getChoosableFileFilters();
    }

    public void propertyChange(PropertyChangeEvent e)
    {
      String prop=e.getPropertyName();
      if(prop == JFileChooser.CHOOSABLE_FILE_FILTER_CHANGED_PROPERTY)
      {
	filters=(javax.swing.filechooser.FileFilter[]) e.getNewValue();
	fireContentsChanged(this, -1, -1);
      }
      // hack for setting the default button
      else
      if("ancestor".equals(prop))
      {
	JRootPane rp=getFileChooser().getRootPane();
	if(  (rp!=null)
	   &&(rp.getDefaultButton()==null)
	   &&getFileChooser().getControlButtonsAreShown())
	  rp.setDefaultButton(approveButton);
      }
    }

    public void setSelectedItem(Object filter)
    {
      if(filter!=null) {
	getFileChooser().setFileFilter((javax.swing.filechooser.FileFilter) filter);
	fireContentsChanged(this, -1, -1);
      }
    }

    public Object getSelectedItem()
    {
      javax.swing.filechooser.FileFilter currentFilter=getFileChooser().getFileFilter();
      boolean found=false;
      if(currentFilter!=null)
      {
	for(int i=0; i < filters.length; i++)
	  if(filters[i] == currentFilter)
	    found=true;
	
	if(found == false)
	  getFileChooser().addChoosableFileFilter(currentFilter);
      }
      return getFileChooser().getFileFilter();
    }
    
    public int getSize()
    {
      if(filters!=null)
	return filters.length;
      else
	return 0;
    }

    public Object getElementAt(int index) {
      if(index > getSize() - 1) {
	return getFileChooser().getFileFilter();
      }
      if(filters!=null) {
	return filters[index];
      } else {
	return null;
      }
    }
  }

  public void valueChanged(ListSelectionEvent e) {
    File f=getFileChooser().getSelectedFile();
    if (!e.getValueIsAdjusting() && f!=null && !getFileChooser().isTraversable(f)) {
      setFileName(getFileChooser().getName(f));
    }
  }

  protected class DirectoryComboBoxAction extends AbstractAction
  {
    protected DirectoryComboBoxAction()
    {
      super("DirectoryComboBoxAction");
    }
    
    public void actionPerformed(ActionEvent e)
    { 
      //try {
	File f = (File)directoryComboBox.getSelectedItem();
	//System.err.println("SET-CUR-DIR: "+f);
	getFileChooser().setCurrentDirectory(f);
	//System.err.println("SET-CUR-DIR: ---------------");
      //}
      //catch(SecurityException ex)
      //{
      //System.err.println("DCBA: "+ex);
	//getChangeToParentDirectoryAction().actionPerformed
	//(new ActionEvent(UP_BUTTON,ActionEvent.ACTION_PERFORMED,""));
	//if(HOME_BUTTON!=null) HOME_BUTTON.doClick();
	//directoryComboBoxModel=createDirectoryComboBoxModel(filechooser);
	//directoryComboBox.setModel(directoryComboBoxModel);
      //}
    }
  }
  
  protected JButton getApproveButton(JFileChooser fc)
  {
    return approveButton;
  }

  /*
  protected SlafFileView fileview_=new SlafFileView();

  public FileView getFileView(JFileChooser fc)
  {
    return fileview_;
  }

  public void clearIconCache()
  {
    fileview_.clearIconCache();
  }
  */

//   protected class SlafFileView
//     extends //Basic
//       FileView
//   {
//     public String getName(File f)
//     {
//       String r=f.getName();
//       //try { r="xxx "+super.getName(f); }
//       //catch(SecurityException ex) { }
//       return r;
//     }

//     public String getDescription(File f)
//     {
//       return "xxx "+super.getDescription(f);
//     }

//     public String getTypeDescription(File f)
//     {
//       String r="";
//       try { r="xxx "+super.getTypeDescription(f); }
//       catch(SecurityException ex) { }
//       return r;
//     }

//     public Icon getIcon(File f)
//     {
//       Icon r=new IndentIcon();//null;
//       try { r=super.getIcon(f); }
//       catch(SecurityException ex) { }
//       return r;
//     }

//     public Boolean isHidden(File f)
//     {
//       Boolean r=Boolean.FALSE;
//       try { r=super.isHidden(f); }
//       catch(SecurityException ex) { }
//       return r;
//     }

//     public Boolean isTraversable(File f)
//     {
//       Boolean r=Boolean.TRUE;
//       try { r=Boolean.valueOf(f.isDirectory()); }
//       catch(SecurityException ex) { }
//       return r;
//     }
//   }
}
