/**
 * @modification $Date: 2004/01/26 18:17:33 $
 * @statut       unstable
 * @file         com/memoire/slaf/SlafGenericBorder.java
 * @version      0.20
 * @author       Guillaume Desnoix
 * @email        guillaume-desnoix@memoire.com
 * @license      GNU General Public License 2 (GPL2)
 * @copyright    1999-2004 Guillaume Desnoix
 */

package com.memoire.slaf;
import com.memoire.slaf.*;

import javax.swing.*;
import javax.swing.border.*;
import java.awt.*;

public class SlafGenericBorder extends AbstractBorder
{
  public static final int NONE          =  0;
  public static final int FOREGROUND    =  1;
  public static final int BACKGROUND    =  2;
  public static final int LIGHT         =  3;
  public static final int DARK          =  4;
  public static final int RAISED        =  5;
  public static final int LOWERED       =  6;
  public static final int ETCHED        =  7;
  public static final int REVETCHED     =  8;
  public static final int SOFTRAISED    =  9;
  public static final int SOFTLOWERED   = 10;
  public static final int HARDRAISED    = 11;
  public static final int HARDLOWERED   = 12;
  public static final int FLAT          = 13;
  public static final int WRRAISED      = 14;
  public static final int WRLOWERED     = 15;
  public static final int WSRAISED      = 16;
  public static final int WSLOWERED     = 17;

  private int   type;
  private int   margin;
  private int   thickness;
  private int   shadow;
  private Color bg_;
  private Color fg_;

  public SlafGenericBorder(int _type, int _margin,
			   int _thickness)
  {
    this(_type,_margin,_thickness,0,null,null);
  }

  public SlafGenericBorder(int _type, int _margin,
			   int _thickness, int _shadow)
  {
    this(_type,_margin,_thickness,_shadow,null,null);
  }

  public SlafGenericBorder(int _type, int _margin,
			   int _thickness, int _shadow,
			   Color _bg, Color _fg)
  {
    type     =_type;
    margin   =_margin;
    thickness=_thickness;
    shadow   =_shadow;
    bg_      =_bg;
    fg_      =_fg;
  }

  public String toString()
  {
    String r="border(";
    r+="type="+type+",";
    r+="margin="+margin+",";
    r+="thickness="+thickness+",";
    r+="shadow="+shadow+",";
    return r+")";
  }

  private int getShadow(Component c)
  {
    return (  (c instanceof JComponent)
	    &&Boolean.TRUE.equals
	      (((JComponent)c).getClientProperty("SLAF_NO_SHADOW")))
      ? 0 : shadow;
  }

  public void paintBorder(Component c, Graphics g,
			  int x, int y, int w, int h)
  {
    Color old=g.getColor();
    Color bg =(bg_!=null ? bg_ : c.getBackground());
    Color fg =(fg_!=null ? fg_ : c.getForeground());
    Color pc =bg;
    int   sh =getShadow(c);

    try
    {
      Container p=c.getParent();
      pc=p.getBackground();
    }
    catch(Exception ex) { }

    g.translate(x,y);

    int j;

    if(sh>=0)
    {
      int DISTANCE=2*sh;

      if(SlafUtils.jdk()>=1.2)
      {
	for(j=0;j<sh;j++)
        {
	  g.setColor(new Color(0,0,0,96*(sh-j)/sh));
	  g.drawLine(w-sh+j,DISTANCE+j/2,w-sh+j,h-1-sh);
	  g.drawLine(DISTANCE+j/2,h-sh+j,w-1-sh,h-sh+j);
	  //if(j>0)
	  {
	    g.setColor(new Color(0,0,0,96*(sh-j)/sh));
	    g.drawLine(w-sh,h-sh+j,w-sh+j,h-sh);
	  }
	}
      }
      else
      {
	int rs=pc.getRed();
	int gs=pc.getGreen();
	int bs=pc.getBlue();

	for(j=0;j<sh;j++)
        {
	  g.setColor(new Color(rs*j/sh,gs*j/sh,bs*j/sh));
	  g.drawLine(w-sh,DISTANCE-j,w-sh+j,DISTANCE-j);
	  g.drawLine(DISTANCE-j,h-sh,DISTANCE-j,h-sh+j);
	  g.drawLine(w-sh+j,DISTANCE-j,w-sh+j,h-sh+j);
	  g.drawLine(DISTANCE-j,h-sh+j,w-sh+j,h-sh+j);
	}

	if(!(c instanceof JComponent)||((JComponent)c).isOpaque())
	{
	  g.setColor(pc);
	  g.fillRect(0,h-sh,sh+1,sh+1);
	  g.fillRect(w-sh,0,sh+1,sh+1);
	}
      }

      w-=sh;
      h-=sh;
    }

    /*
    else
    {
      g.setColor(pc);
      g.fillRect(0,h+sh,w,-sh+1);
      g.fillRect(w+sh,0,-sh+1,h);

      // g.translate(-sh,-sh);
      w+=sh;
      h+=sh;
    }
    */

    for(j=0;j<thickness;j++)
    {
      switch(type)
      {
      case NONE:
	break;
      case FOREGROUND:
	g.setColor(fg);
	g.drawRect(j,j,w-1-2*j,h-1-2*j);
	break;
      case BACKGROUND:
	g.setColor(bg);
	g.drawRect(j,j,w-1-2*j,h-1-2*j);
	break;
      case LIGHT:
	g.setColor(bg.brighter());
	g.drawRect(j,j,w-1-2*j,h-1-2*j);
	break;
      case DARK:
	g.setColor(bg.darker());
	g.drawRect(j,j,w-1-2*j,h-1-2*j);
	break;
      case RAISED:
	g.setColor(bg);
	g.draw3DRect(j,j,w-1-2*j,h-1-2*j,true);
	break;
      case LOWERED:
	g.setColor(bg);
	g.draw3DRect(j,j,w-1-2*j,h-1-2*j,false);
	break;
      case ETCHED:
	g.setColor(bg);
	g.draw3DRect(j,j,w-1-2*j,h-1-2*j,(j%2==1));
	break;
      case REVETCHED:
	g.setColor(bg);
	g.draw3DRect(j,j,w-1-2*j,h-1-2*j,(j%2!=1));
	break;
      case SOFTRAISED:
	if(bg.equals(Color.white))
	  g.setColor(SlafUtils.darken(bg,j+1));
	else
	  g.setColor(SlafUtils.brighten(bg,thickness-j));
	g.drawLine(j,j,w-1-j,j);
	g.drawLine(j,j,j,h-1-j);
	g.setColor(SlafUtils.darken(bg,thickness-j));
	g.drawLine(j,h-1-j,w-1-j,h-1-j);
	g.drawLine(w-1-j,j,w-1-j,h-1-j);
	break;
      case SOFTLOWERED:
	g.setColor(SlafUtils.darken(bg,thickness-j));
	g.drawLine(j,j,w-1-j,j);
	g.drawLine(j,j,j,h-1-j);
	if(bg.equals(Color.white))
	  g.setColor(SlafUtils.darken(bg,j+1));
	else
	  g.setColor(SlafUtils.brighten(bg,thickness-j));
	g.drawLine(j,h-1-j,w-1-j,h-1-j);
	g.drawLine(w-1-j,j,w-1-j,h-1-j);
	break;
      case HARDRAISED:
	if(bg.equals(Color.white))
	  g.setColor(SlafUtils.darken(bg,j+1));
	else
	  g.setColor(SlafUtils.brighten(bg,4*(thickness-j-1)));
	g.drawLine(j,j,w-1-j,j);
	g.drawLine(j,j,j,h-1-j);
	g.setColor(SlafUtils.darken(bg,8*(thickness-j)-5));
	g.drawLine(j,h-1-j,w-1-j,h-1-j);
	g.drawLine(w-1-j,j,w-1-j,h-1-j);
	break;
      case HARDLOWERED:
	g.setColor(SlafUtils.darken(bg,8*(thickness-j)-5));
	g.drawLine(j,j,w-1-j,j);
	g.drawLine(j,j,j,h-1-j);
	if(bg.equals(Color.white))
	  g.setColor(SlafUtils.darken(bg,j+1));
	else
	  g.setColor(SlafUtils.brighten(bg,4*(thickness-j-1)));
	g.drawLine(j,h-1-j,w-1-j,h-1-j);
	g.drawLine(w-1-j,j,w-1-j,h-1-j);
	break;
      case FLAT:
	if((j==0)||(j==thickness-1)) g.setColor(fg);
	else                         g.setColor(bg);
	g.drawRect(j,j,w-1-2*j,h-1-2*j);
	break;
      case WRRAISED:
	if(j==0)
	{
	  g.setColor(Color.black);//fg);
	  g.drawLine(1,0  ,w-2,0  );
	  g.drawLine(0,1  ,0,h-2  );
	  g.drawLine(1,h-1,w-2,h-1);
	  g.drawLine(w-1,1,w-1,h-2);
	}
	else
	{
	  if(bg.equals(Color.white)) g.setColor(Color.lightGray);
	  else                       g.setColor(bg.brighter());
	  g.drawLine(j,j,w-1-j,j);
	  g.drawLine(j,j,j,h-1-j);
	  if(bg.equals(Color.white)) g.setColor(Color.gray);
	  else                       g.setColor(bg.darker());
	  g.drawLine(j,h-1-j,w-1-j,h-1-j);
	  g.drawLine(w-1-j,j+1,w-1-j,h-1-j);
	}
	break;
      case WRLOWERED:
	if(j==0)
	{
	  g.setColor(Color.black);//fg);
	  g.drawLine(1,0  ,w-2,0  );
	  g.drawLine(0,1  ,0,h-2  );
	  g.drawLine(1,h-1,w-2,h-1);
	  g.drawLine(w-1,1,w-1,h-2);
	}
	else
	{
	  if(bg.equals(Color.white)) g.setColor(Color.gray);
	  else                       g.setColor(bg.darker());
	  g.drawLine(j,j,w-1-j,j);
	  g.drawLine(j,j,j,h-1-j);
	  if(bg.equals(Color.white)) g.setColor(Color.lightGray);
	  else                       g.setColor(bg.brighter());
	  g.drawLine(j,h-1-j,w-1-j,h-1-j);
	  g.drawLine(w-1-j,j+1,w-1-j,h-1-j);
	}
	break;
      case WSRAISED:
	if(j==0)
	{
	  g.setColor(fg);
	  g.drawRect(0,0,w-1,h-1);
	}
	else
	{
	  if(bg.equals(Color.white)) g.setColor(Color.lightGray);
	  else                       g.setColor(bg.brighter());
	  g.drawLine(j,j,w-1-j,j);
	  g.drawLine(j,j,j,h-1-j);
	  if(bg.equals(Color.white)) g.setColor(Color.gray);
	  else                       g.setColor(bg.darker());
	  g.drawLine(j,h-1-j,w-1-j,h-1-j);
	  g.drawLine(w-1-j,j+1,w-1-j,h-1-j);
	}
	break;
      case WSLOWERED:
	if(j==0)
	{
	  g.setColor(fg);
	  g.drawRect(0,0,w-1,h-1);
	}
	else
	{
	  if(bg.equals(Color.white)) g.setColor(Color.gray);
	  else                       g.setColor(bg.darker());
	  g.drawLine(j,j,w-1-j,j);
	  g.drawLine(j,j,j,h-1-j);
	  if(bg.equals(Color.white)) g.setColor(Color.lightGray);
	  else                       g.setColor(bg.brighter());
	  g.drawLine(j,h-1-j,w-1-j,h-1-j);
	  g.drawLine(w-1-j,j+1,w-1-j,h-1-j);
	}
	break;
      }
    }

    /*
    if(!((JComponent)c).isOpaque())
    {
      g.setColor(pc);
      for(j=thickness;j<thickness+margin;j++)
	g.drawRect(j,j,w-1-2*j,h-1-2*j);
    }
    */

    if(bg_!=null)
    {
      g.setColor(bg_);
      for(j=thickness;j<thickness+margin;j++)
	g.drawRect(j,j,w-1-2*j,h-1-2*j);
    }

    // if(sh<0) g.translate(sh,sh);

    g.translate(-x,-y);
    g.setColor(old);
  }

  public Insets getBorderInsets(Component _c, Insets _insets)
  {
    int  b=thickness+margin;
    int  s=getShadow(_c);

    _insets.top =b;
    _insets.left=b;
    _insets.bottom=b+Math.abs(s);
    _insets.right =b+Math.abs(s);

    return _insets;
  }

  public Insets getBorderInsets(Component _c)
  {
    return getBorderInsets(_c,new Insets(0,0,0,0));

    /*
    int    b=thickness+margin;
    Insets r=null;

    if(sh>=0)
      r=new Insets(b,b,b+sh,b+sh);
    else
      r=new Insets(b,b,b-sh,b-sh);

    return r;
    */
  }

  public boolean isBorderOpaque()
    { return false; }
}
