/**
 * @modification $Date: 2004/05/11 19:06:50 $
 * @statut       unstable
 * @file         com/memoire/slaf/SlafGridLayout.java
 * @version      0.20
 * @author       Guillaume Desnoix
 * @email        guillaume-desnoix@memoire.com
 * @license      GNU General Public License 2 (GPL2)
 * @copyright    1999-2004 Guillaume Desnoix
 */

package com.memoire.slaf;
import com.memoire.slaf.*;

import java.awt.* ;
import java.util.* ;
import javax.swing.plaf.*;

/**
 * layouts in a grid a set of components.
 * Not like the java.awt.GridLayout, rows may have different heigths
 * and columns may have different widths.
 * THIS CLASS IS A COPY OF BuGridLayout.
 */

public class SlafGridLayout
  implements LayoutManager2, UIResource
{
  // Vector components;

  public SlafGridLayout()
  {
    this(3,0,0,true,true);
  }

  public SlafGridLayout(int _columns)
  {
    this(_columns,0,0,true,true);
  }

  public SlafGridLayout(int _columns, int _hgap, int _vgap)
  {
    this(_columns,_hgap,_vgap,true,true);
  }

  public SlafGridLayout(int _columns,
		      int _hgap, int _vgap,
		      boolean _hfilled, boolean _vfilled)
  {
    this(_columns,_hgap,_vgap,_hfilled,_vfilled,true,false);
  }

  public SlafGridLayout(int _columns,
			int _hgap, int _vgap,
			boolean _hfilled, boolean _vfilled,
			boolean _cfilled, boolean _rfilled)
  {
    // components=new Vector(1,1);

    columns_=_columns;
    hgap_=_hgap;
    vgap_=_vgap;
    hfilled_=_hfilled;
    vfilled_=_hfilled;
    cfilled_=_cfilled;
    rfilled_=_rfilled;
    xalign_=0.5f;
    yalign_=0.5f;
  }

  protected int columns_;
  public int getColumns() { return columns_; }
  public void setColumns(int _columns) { columns_=_columns; }

  protected int hgap_;
  public int getHgap() { return hgap_; }
  public void setHgap(int _hgap) { hgap_=_hgap; }

  protected int vgap_;
  public int getVgap() { return vgap_; }
  public void setVgap(int _vgap) { vgap_=_vgap; }

  protected boolean hfilled_;
  public boolean getHfilled() { return hfilled_; }
  public void setHfilled(boolean _hfilled) { hfilled_=_hfilled; }

  protected boolean vfilled_;
  public boolean getVfilled() { return vfilled_; }
  public void setVfilled(boolean _vfilled) { vfilled_=_vfilled; }

  protected boolean cfilled_;
  public boolean getCfilled() { return cfilled_; }
  public void setCfilled(boolean _cfilled) { cfilled_=_cfilled; }

  protected boolean rfilled_;
  public boolean getRfilled() { return rfilled_; }
  public void setRfilled(boolean _rfilled) { rfilled_=_rfilled; }

  protected float xalign_;
  public float getXalign() { return xalign_; }
  public void setXalign(float _xalign) { xalign_=_xalign; }

  protected float yalign_;
  public float getYalign() { return yalign_; }
  public void setYalign(float _yalign) { yalign_=_yalign; }

  public void addLayoutComponent(Component _comp, Object _cstr)
  {
    // components.addElement(_comp);
  }

  public void addLayoutComponent(String _name, Component _comp)
  {
    // components.addElement(_comp);
  }

  public void removeLayoutComponent(Component _comp)
  {
    // components.removeElement(_comp);
  }

  public Dimension minimumLayoutSize(Container _parent)
  {
    synchronized (_parent.getTreeLock())
    {
      // int nbc=components.size();
      int nbc=_parent.getComponentCount();
      int w  =0;
      int h  =0;
    
      int nbcol=columns_;
      if(nbcol==0) nbcol=nbc;
      if(nbcol==0) nbcol=1;
      int nbrow=(nbc+nbcol-1)/nbcol;

      int[] wcol=new int[nbcol];
      int[] hrow=new int[nbrow];

      int i;
      
      for(i=0;i<nbc;i++)
      {
	Component c=_parent.getComponent(i);
	if(c.isVisible())
	{
	  Dimension d=c.getMinimumSize();
	  wcol[i%nbcol]=Math.max(wcol[i%nbcol],d.width);
	  hrow[i/nbcol]=Math.max(hrow[i/nbcol],d.height);
	}
      }

      for(i=0;i<nbcol;i++)
	if(wcol[i]>0)
	{
	  if(i>0) w+=hgap_;
	  w+=wcol[i];
	}

      for(i=0;i<nbrow;i++)
	if(hrow[i]>0)
	{
	  if(i>0) h+=vgap_;
	  h+=hrow[i];
	}

      Insets insets=_parent.getInsets();
      return new Dimension(w+insets.left+insets.right,
                         h+insets.top+insets.bottom);
    }
  }

  public Dimension preferredLayoutSize(Container _parent)
  {
    synchronized (_parent.getTreeLock())
    {
      // int nbc=components.size();
      int nbc=_parent.getComponentCount();
      int w  =0;
      int h  =0;

      int nbcol=columns_;
      if(nbcol==0) nbcol=nbc;
      if(nbcol==0) nbcol=1;
      int nbrow=(nbc+nbcol-1)/nbcol;

      int[] wcol=new int[nbcol];
      int[] hrow=new int[nbrow];

      int i;

      for(i=0;i<nbc;i++)
      {
	Component c=_parent.getComponent(i);
	if(c.isVisible())
	{
	  Dimension d=c.getPreferredSize();
	  wcol[i%nbcol]=Math.max(wcol[i%nbcol],d.width);
	  hrow[i/nbcol]=Math.max(hrow[i/nbcol],d.height);
	}
      }

      for(i=0;i<nbcol;i++)
	if(wcol[i]>0)
	{
	  if(i>0) w+=hgap_;
	  w+=wcol[i];
	  //System.err.println("WCOL["+i+"]="+wcol[i]+" W="+w);
	}

      for(i=0;i<nbrow;i++)
	if(hrow[i]>0)
	{
	  if(i>0) h+=vgap_;
	  h+=hrow[i];
	}

      Insets insets=_parent.getInsets();
      //System.err.println("PREF.SIZE="+w+","+h);
      return new Dimension(w+insets.left+insets.right,
			   h+insets.top+insets.bottom);
    }
  }

  public Dimension maximumLayoutSize(Container _parent)
  {
    return new Dimension(Short.MAX_VALUE,Short.MAX_VALUE);

    /*
    synchronized (_parent.getTreeLock())
    {
      // int nbc=components.size();
      int nbc=_parent.getComponentCount();
      int w  =Integer.MAX_VALUE;
      int h  =0;

      int nbcol=columns_;
      if(nbcol==0) nbcol=nbc;
      if(nbcol==0) nbcol=1;
      int nbrow=(nbc+nbcol-1)/nbcol;

      int[] wcol=new int[nbcol];
      int[] hrow=new int[nbrow];

      int i;

      for(i=0;i<nbc;i++)
      {
	Component c=_parent.getComponent(i);
	if(c.isVisible())
	{
	  Dimension d=c.getMaximumSize();
	  wcol[i%nbcol]=Math.max(wcol[i%nbcol],d.width);
	  hrow[i/nbcol]=Math.max(hrow[i/nbcol],d.height);
	}
      }

      for(i=0;i<nbcol;i++)
      {
	if(i>0) w+=hgap_;
	w+=wcol[i];
      }

      for(i=0;i<nbrow;i++)
      {
	if(i>0) h+=vgap_;
	h+=hrow[i];
      }

      Insets insets=_parent.getInsets();
      return new Dimension(w+insets.left+insets.right,
			   h+insets.top+insets.bottom);
    }
    */
  }

  public void invalidateLayout(Container _parent)
  {
    //layoutContainer(_parent);
  }

  public float getLayoutAlignmentX(Container _parent)
  {
    return (float)0.5;
  }

  public float getLayoutAlignmentY(Container _parent)
  {
    return (float)0.5;
  }

  public void layoutContainer(Container _parent)
  {
    synchronized (_parent.getTreeLock())
    {
      Insets    insets=_parent.getInsets();
      Dimension size  =_parent.getSize();
      Rectangle bounds=new Rectangle
	(insets.left,
	 insets.top,
	 size.width-(insets.left + insets.right),
	 size.height-(insets.top + insets.bottom));

      // int nbc=components.size();
      int nbc=_parent.getComponentCount();
      int x  =0;
      int y  =0;
      int w  =0;
      int h  =0;

      int nbcol=columns_;
      if(nbcol==0) nbcol=nbc;
      if(nbcol==0) nbcol=1;
      int nbrow=(nbc+nbcol-1)/nbcol;

      int[] wcol=new int[nbcol];
      int[] hrow=new int[nbrow];

      int i;

      for(i=0;i<nbc;i++)
      {
	Component c=_parent.getComponent(i);
	if(c.isVisible())
	{
	  Dimension d=c.getPreferredSize();
	  wcol[i%nbcol]=Math.max(wcol[i%nbcol],d.width);
	  hrow[i/nbcol]=Math.max(hrow[i/nbcol],d.height);
	}
      }

      for(i=0;i<nbcol;i++)
	if(wcol[i]>0)
	{
	  if(i>0) w+=hgap_;
	  w+=wcol[i];
	}

      for(i=0;i<nbrow;i++)
	if(hrow[i]>0)
        {
	  if(i>0) h+=vgap_;
	  h+=hrow[i];
	}

      for(i=0;i<nbc;i++)
      {
	Component c=_parent.getComponent(i);
	if(c.isVisible())
	{
	  Dimension d=c.getPreferredSize();
	  int wmax=wcol[i%nbcol];
	  int hmax=hrow[i/nbcol];

	  if((i>0)&&(i%nbcol==0)&&(hmax>0)) y+=vgap_;
	  if((i%nbcol!=0)       &&(wmax>0)) x+=hgap_;

	  w=(hfilled_ ? wmax : Math.min(wmax,d.width ));
	  h=(vfilled_ ? hmax : Math.min(hmax,d.height));

	  // last
	  if(cfilled_&&(i%nbcol==nbcol-1)&&(x+w<bounds.width ))
	      w=wmax=bounds.width -x;
	  if(rfilled_&&(i/nbcol==nbrow-1)&&(y+h<bounds.height))
	      h=hmax=bounds.height-y;
	
	  // center
	  int dx=0;
	  int dy=0;
	  if(!hfilled_) dx=(int)((wmax-w)*xalign_);
	  if(!vfilled_) dy=(int)((hmax-h)*yalign_);

	  c.setBounds(bounds.x+x+dx,bounds.y+y+dy,w,h);
	  //System.err.println("-- "+i%nbcol+","+i/nbcol+": "+c.getBounds());

	  if((i+1)%nbcol!=0) x+=wmax;
	  else { x=0; y+=hmax; }
	}
      }
    }
  }
}
