/**
 * @modification $Date: 2004/02/13 11:22:39 $
 * @statut       unstable
 * @file         com/memoire/slaf/SlafIconFactory.java
 * @version      0.20
 * @author       Guillaume Desnoix
 * @email        guillaume-desnoix@memoire.com
 * @license      GNU General Public License 2 (GPL2)
 * @copyright    1999-2004 Guillaume Desnoix
 */

package com.memoire.slaf;
import com.memoire.slaf.*;

import java.awt.*;
import java.io.Serializable;

import javax.swing.*;
import javax.swing.plaf.*;

public class SlafIconFactory
  implements Serializable
{
  public static final int SIZE=12;

  // List of code-drawn Icons
  private static Icon fileChooserDetailViewIcon;
  private static Icon fileChooserHomeFolderIcon;
  private static Icon fileChooserListViewIcon;
  private static Icon fileChooserNewFolderIcon;
  private static Icon fileChooserUpFolderIcon;

  // private static Icon internalFrameAltMaximizeIcon;
  // private static Icon internalFrameCloseIcon;
  // private static Icon internalFrameDefaultMenuIcon;
  // private static Icon internalFrameMaximizeIcon;
  // private static Icon internalFrameMinimizeIcon;

  private static Icon optionPaneErrorIcon;
  private static Icon optionPaneInformationIcon;
  private static Icon optionPaneWarningIcon;
  private static Icon optionPaneQuestionIcon;

  private static Icon radioButtonIcon; 
  private static Icon radioButtonSelectedIcon; 
  private static Icon radioButtonUnselectedIcon; 

  private static Icon checkBoxIcon;
  private static Icon checkBoxSelectedIcon;
  private static Icon checkBoxUnselectedIcon;

  private static Icon treeComputerIcon;
  private static Icon treeFloppyDriveIcon;
  // private static Icon treeFolderIcon;
  // private static Icon treeHardDriveIcon;
  // private static Icon treeLeafIcon;
  private static Icon treeDarkControlIcon;
  private static Icon treeLightControlIcon;

  private static Icon menuArrowIcon;
  private static Icon menuCheckIcon;
  private static Icon menuItemCheckIcon;
  private static Icon menuItemArrowIcon;
  private static Icon checkBoxMenuItemArrowIcon;
  private static Icon checkBoxMenuItemCheckIcon;
  private static Icon radioButtonMenuItemArrowIcon;
  private static Icon radioButtonMenuItemCheckIcon;

  // Constants
  public static final boolean DARK =false;
  public static final boolean LIGHT=true;

  /*
  public static void reinit()
  {
    fileChooserDetailViewIcon=null;
    fileChooserHomeFolderIcon=null;
    fileChooserListViewIcon=null;
    fileChooserNewFolderIcon=null;
    fileChooserUpFolderIcon=null;

    optionPaneErrorIcon=null;
    optionPaneInformationIcon=null;
    optionPaneWarningIcon=null;
    optionPaneQuestionIcon=null;

    radioButtonIcon=null; 
    radioButtonSelectedIcon=null; 
    radioButtonUnselectedIcon=null; 

    checkBoxIcon=null;
    checkBoxSelectedIcon=null;
    checkBoxUnselectedIcon=null;

    treeComputerIcon=null;
    treeFloppyDriveIcon=null;
    treeDarkControlIcon=null;
    treeLightControlIcon=null;
    
    menuArrowIcon=null;
    menuCheckIcon=null;
    menuItemCheckIcon=null;
    menuItemArrowIcon=null;
    checkBoxMenuItemArrowIcon=null;
    checkBoxMenuItemCheckIcon=null;
    radioButtonMenuItemArrowIcon=null;
    radioButtonMenuItemCheckIcon=null;
  }
  */

  protected static Icon getSlafIcon(String _name, int _size)
  {
    Icon r=SlafResource.SLAF.getIcon(_name,_size);
    if(r==null)
    {
      SlafUtils.trace("ICON NULL : "+_name+", "+_size);
      r=new DefaultIcon(_size);
    }

    return new IconUIResource(r);
  }

  public static Icon getEmptyIcon(Object _icon)
  {
    int w=SIZE;
    int h=SIZE;

    if(_icon instanceof Icon)
    {
      w=((Icon)_icon).getIconWidth ();
      h=((Icon)_icon).getIconHeight();
    }
    //SlafUtils.trace("create an empty icon "+w+"x"+h);
    return new EmptyIcon(w,h);
  }

//*** FileChooser *******************************************

    public static Icon getFileChooserDetailViewIcon() {
      if(fileChooserDetailViewIcon==null)
	fileChooserDetailViewIcon=getSlafIcon("slaf_filechooser_detailview",SIZE);
      return fileChooserDetailViewIcon;
    }

    public static Icon getFileChooserHomeFolderIcon()
    {
      if(fileChooserHomeFolderIcon==null)
	fileChooserHomeFolderIcon=getSlafIcon("slaf_filechooser_homefolder",SIZE);
      return fileChooserHomeFolderIcon;
    }

    public static Icon getFileChooserListViewIcon()
    {
      if(fileChooserListViewIcon==null)
	fileChooserListViewIcon=getSlafIcon("slaf_filechooser_listview",SIZE);
      return fileChooserListViewIcon;
    }

    public static Icon getFileChooserNewFolderIcon()
    {
      if(fileChooserNewFolderIcon==null)
	fileChooserNewFolderIcon=getSlafIcon("slaf_filechooser_newfolder",SIZE);
      return fileChooserNewFolderIcon;
    }

    public static Icon getFileChooserUpFolderIcon()
    {
      if(fileChooserUpFolderIcon==null)
	fileChooserUpFolderIcon=getSlafIcon("slaf_filechooser_upfolder",SIZE);
      return fileChooserUpFolderIcon;
    }

//*** InternalFrame *******************************************

  public static Icon getInternalFrameLinifyIcon()//int size)
  { return getSlafIcon("slaf_line",SIZE); }

  public static Icon getInternalFrameAltMaximizeIcon()//int size)
  { return getSlafIcon("slaf_restore",SIZE); }

  public static Icon getInternalFrameCloseIcon()//int size)
  { return getSlafIcon("slaf_close",SIZE); }

  public static Icon getInternalFrameDefaultMenuIcon()
  { return getSlafIcon("slaf_menu",SIZE); }

  public static Icon getInternalFrameMaximizeIcon()//int size)
  { return getSlafIcon("slaf_maximize",SIZE); }

  public static Icon getInternalFrameMinimizeIcon()//int size)
  { return getSlafIcon("slaf_minimize",SIZE); }

//*** Arrows *******************************************

  public static Icon getNorthIcon()//int size)
  { return getSlafIcon("slaf_north",SIZE); }

  public static Icon getSouthIcon()//int size)
  { return getSlafIcon("slaf_south",SIZE); }

  public static Icon getEastIcon()//int size)
  { return getSlafIcon("slaf_east",SIZE); }

  public static Icon getWestIcon()//int size)
  { return getSlafIcon("slaf_west",SIZE); }

//*** Tree *******************************************

  public static Icon getTreeComputerIcon()
  { return getSlafIcon("slaf_tree_computer",SIZE); }

  public static Icon getTreeFloppyDriveIcon()
  { return getSlafIcon("slaf_tree_floppy",SIZE); }

  public static Icon getTreeFolderOpenIcon()
  { return getSlafIcon("slaf_tree_folder_open",SIZE); }

  public static Icon getTreeFolderClosedIcon()
  { return getSlafIcon("slaf_tree_folder_closed",SIZE); }

  public static Icon getTreeHardDriveIcon()
  { return getSlafIcon("slaf_tree_drive",SIZE); }

  public static Icon getTreeLeafIcon()
  { return getSlafIcon("slaf_tree_leaf",SIZE); }

  public static Icon getTreeControlIcon(boolean isLight)
  {
    if(isLight)
    {
      if(treeLightControlIcon==null)
	treeLightControlIcon=getSlafIcon("slaf_tree_control_plus",SIZE);
      return treeLightControlIcon;
    }
    else
    {
      if(treeDarkControlIcon==null)
	treeDarkControlIcon=getSlafIcon("slaf_tree_control_minus",SIZE);
      return treeDarkControlIcon;
    }
  }

//*** Menu *******************************************

  public static Icon getMenuArrowIcon()
  {
    if(menuArrowIcon==null)
      menuArrowIcon=getSlafIcon("slaf_menu_arrow",SIZE);
    return menuArrowIcon;
  }

  public static Icon getMenuCheckIcon()
  {
    if(menuCheckIcon==null)
      menuCheckIcon=getSlafIcon("slaf_menu_check",SIZE);
    return menuCheckIcon;
  }

  public static Icon getMenuItemArrowIcon()
  {
    if(menuItemArrowIcon==null)
      menuItemArrowIcon=getSlafIcon("slaf_menu_item_arrow",SIZE);
    return menuItemArrowIcon;
  }

  public static Icon getMenuItemCheckIcon()
  {
    if(menuItemCheckIcon==null)
      menuItemCheckIcon=getSlafIcon("slaf_menu_item_check",SIZE);
    return menuItemCheckIcon;
  }

  public static Icon getCheckBoxMenuItemArrowIcon()
  {
    if(checkBoxMenuItemArrowIcon==null)
      checkBoxMenuItemArrowIcon=getSlafIcon("slaf_check_arrow",SIZE);
    return checkBoxMenuItemArrowIcon;
  }

  public static Icon getCheckBoxMenuItemCheckIcon()
  {
    if(checkBoxMenuItemCheckIcon==null)
      checkBoxMenuItemCheckIcon=new CheckBoxMenuItemIcon();
    return checkBoxMenuItemCheckIcon;
  }

  public static Icon getRadioButtonMenuItemArrowIcon()
  {
    if(radioButtonMenuItemArrowIcon==null)
      radioButtonMenuItemArrowIcon=getSlafIcon("slaf_radio_arrow",SIZE);
    return radioButtonMenuItemArrowIcon;
  }

  public static Icon getRadioButtonMenuItemCheckIcon()
  {
    if(radioButtonMenuItemCheckIcon==null)
      radioButtonMenuItemCheckIcon=new RadioButtonMenuItemIcon();
    return radioButtonMenuItemCheckIcon;
  }

//*** Sliders *******************************************

  /*
  public static Icon getHorizontalSliderThumbIcon() {
  // don't cache these, bumps don't get updated otherwise
  return new HorizontalSliderThumbIcon();
  }

  public static Icon getVerticalSliderThumbIcon() {
  // don't cache these, bumps don't get updated otherwise
  return new VerticalSliderThumbIcon();
  }
  */

//*** Divers *******************************************

  public static Icon getComboBoxIcon()
  { return getSlafIcon("slaf_combo",SIZE); }

  public static Icon getCheckBoxIcon()
  {
    if(checkBoxIcon==null)
      checkBoxIcon=new CheckBoxIcon();
    return checkBoxIcon;
  }

  public static Icon getCheckBoxSelectedIcon()
  {
    if(checkBoxSelectedIcon==null)
      checkBoxSelectedIcon=getSlafIcon("slaf_check_selected",SIZE);
    return checkBoxSelectedIcon;
  }

  public static Icon getCheckBoxUnselectedIcon()
  {
    if(checkBoxUnselectedIcon==null)
      checkBoxUnselectedIcon=getSlafIcon("slaf_check_unselected",SIZE);
    return checkBoxUnselectedIcon;
  }

  public static Icon getRadioButtonIcon()
  {
    if(radioButtonIcon==null)
      radioButtonIcon=new RadioButtonIcon();
    return radioButtonIcon;
  }

  public static Icon getRadioButtonSelectedIcon()
  {
    if(radioButtonSelectedIcon==null)
      radioButtonSelectedIcon=getSlafIcon("slaf_radio_selected",SIZE);
    return radioButtonSelectedIcon;
  }

  public static Icon getRadioButtonUnselectedIcon()
  {
    if(radioButtonUnselectedIcon==null)
      radioButtonUnselectedIcon=getSlafIcon("slaf_radio_unselected",SIZE);
    return radioButtonUnselectedIcon;
  }

  public static Icon getOptionPaneErrorIcon()
  {
    if(optionPaneErrorIcon==null)
      optionPaneErrorIcon=getSlafIcon("slaf_error",SIZE);
    return optionPaneErrorIcon;
  }

  public static Icon getOptionPaneInformationIcon()
  {
    if(optionPaneInformationIcon==null)
      optionPaneInformationIcon=getSlafIcon("slaf_information",SIZE);
    return optionPaneInformationIcon;
  }

  public static Icon getOptionPaneWarningIcon()
  {
    if(optionPaneWarningIcon==null)
      optionPaneWarningIcon=getSlafIcon("slaf_warning",SIZE);
    return optionPaneWarningIcon;
  }

  public static Icon getOptionPaneQuestionIcon()
  {
    if(optionPaneQuestionIcon==null)
      optionPaneQuestionIcon=getSlafIcon("slaf_question",SIZE);
    return optionPaneQuestionIcon;
  }

//******************************************************

  private static class EmptyIcon
    implements Icon, UIResource, Serializable
  {
    private int w_;
    private int h_;

    public EmptyIcon(int _w, int _h)
    {
      w_=_w;
      h_=_h;
    }

    public void paintIcon(Component c, Graphics g, int x, int y)
    {
      /*
      g.translate(x,y);
      g.setColor (Color.red);
      g.drawLine(0,0,w_-1,h_-1);
      g.drawLine(0,h_-1,w_-1,0);
      g.translate(-x,-y);
      */
    }
	
    public int getIconWidth()  { return w_; }
    public int getIconHeight() { return h_; }
  }

  private static class DefaultIcon
    implements Icon, UIResource, Serializable
  {
    private int size_=SIZE;

    public DefaultIcon(int _size)
    {
      size_=_size;
    }

    public void paintIcon(Component c, Graphics g, int x, int y)
    {
      g.translate(x,y);
      g.setColor (Color.blue);
      g.drawLine(0,0,size_-1,size_-1);
      g.drawLine(0,size_-1,size_-1,0);
      g.translate(-x,-y);
    }
	
    public int getIconWidth()  { return size_; }
    public int getIconHeight() { return size_; }
  }

  private static class CheckBoxIcon
    implements Icon, UIResource, Serializable
  {
    public void paintIcon(Component c, Graphics g, int x, int y)
    {
      JCheckBox   b    =(JCheckBox) c;
      ButtonModel model=b.getModel();

      boolean isSelected=model.isSelected();
      boolean isEnabled =model.isEnabled();
      boolean isRollover=model.isRollover();
      //boolean isPressed =model.isPressed();
      //boolean isArmed   =model.isArmed();
	    
      g.translate(x,y);

      Icon icon=null;

      if(!isEnabled)
      {
	if(isSelected)
	  icon=SlafManager.getIcon("CheckBox.disabledSelectedIcon"); 
	else
	  icon=SlafManager.getIcon("CheckBox.disabledUnselectedIcon"); 
      }
      else
      if(isRollover)
      {
	if(isSelected)
	  icon=SlafManager.getIcon("CheckBox.rolloverSelectedIcon"); 
	else
	  icon=SlafManager.getIcon("CheckBox.rolloverUnselectedIcon"); 
      }

      if(icon==null) // if(isEnabled)
      {
	if(isSelected)
	  icon=SlafManager.getIcon("CheckBox.selectedIcon"); 
	else
	  icon=SlafManager.getIcon("CheckBox.unselectedIcon"); 
      }

      if(icon!=null) icon.paintIcon(c,g,0,0);
      g.translate(-x,-y);
    }

    public int getIconWidth()
    {
      int  r=SIZE;
      Icon i=SlafManager.getIcon("CheckBox.selectedIcon");
      if(i!=null) r=i.getIconWidth();
      return r;
    }
    
    public int getIconHeight()
    {
      int  r=SIZE;
      Icon i=SlafManager.getIcon("CheckBox.selectedIcon");
      if(i!=null) r=i.getIconHeight();
      return r;
    }
  }

  private static class RadioButtonIcon
    implements Icon, UIResource, Serializable
  {
    public void paintIcon(Component c, Graphics g, int x, int y)
    {
      JRadioButton b=(JRadioButton) c;
      ButtonModel  model=b.getModel();

      boolean isSelected=model.isSelected();
      boolean isEnabled =model.isEnabled();
      boolean isRollover=model.isRollover();
      //boolean isPressed =model.isPressed();
      //boolean isArmed   =model.isArmed();
	    
      g.translate(x,y);

      Icon icon=null;

      if(!isEnabled)
      {
	if(isSelected)
	  icon=SlafManager.getIcon("RadioButton.disabledSelectedIcon"); 
	else
	  icon=SlafManager.getIcon("RadioButton.disabledUnselectedIcon"); 
      }
      else
      if(isRollover)
      {
	if(isSelected)
	  icon=SlafManager.getIcon("RadioButton.rolloverSelectedIcon"); 
	else
	  icon=SlafManager.getIcon("RadioButton.rolloverUnselectedIcon"); 
      }

      if(icon==null) // if(isEnabled)
      {
	if(isSelected)
	  icon=SlafManager.getIcon("RadioButton.selectedIcon"); 
	else
	  icon=SlafManager.getIcon("RadioButton.unselectedIcon"); 
      }

      if(icon!=null) icon.paintIcon(c,g,0,0);
      g.translate(-x,-y);
    }

    public int getIconWidth()
    {
      int  r=SIZE;
      Icon i=SlafManager.getIcon("RadioButton.selectedIcon");
      if(i!=null) r=i.getIconWidth();
      return r;
    }
    
    public int getIconHeight()
    {
      int  r=SIZE;
      Icon i=SlafManager.getIcon("RadioButton.selectedIcon");
      if(i!=null) r=i.getIconHeight();
      return r;
    }
  }

  private static class CheckBoxMenuItemIcon
    implements Icon, UIResource, Serializable
  {
    public void paintIcon(Component c, Graphics g, int x, int y)
    {
      JMenuItem   b    =(JMenuItem) c;
      ButtonModel model=b.getModel();

      boolean isSelected=model.isSelected();
      //boolean isEnabled =model.isEnabled();
      //boolean isPressed =model.isPressed();
      //boolean isArmed   =model.isArmed();
	    
      g.translate(x,y);

      Icon icon=null;

      // if(isEnabled)
      {
	if(isSelected)
	  icon=SlafManager.getIcon("CheckBoxMenuItem.selectedIcon");
	else
	  icon=SlafManager.getIcon("CheckBoxMenuItem.unselectedIcon"); 
      }

      if(icon!=null) icon.paintIcon(c,g,0,0);
      g.translate(-x,-y);
    }

    public int getIconWidth()
    {
      int  r=SIZE;
      Icon i=SlafManager.getIcon("CheckBoxMenuItem.selectedIcon");
      if(i!=null) r=i.getIconWidth();
      return r;
    }
    
    public int getIconHeight()
    {
      int  r=SIZE;
      Icon i=SlafManager.getIcon("CheckBoxMenuItem.selectedIcon");
      if(i!=null) r=i.getIconHeight();
      return r;
    }
  }

  private static class RadioButtonMenuItemIcon
    implements Icon, UIResource, Serializable
  {
    public void paintIcon(Component c, Graphics g, int x, int y)
    {
      JMenuItem   b    =(JMenuItem) c;
      ButtonModel model=b.getModel();

      boolean isSelected=model.isSelected();
      //boolean isEnabled =model.isEnabled();
      //boolean isPressed =model.isPressed();
      //boolean isArmed   =model.isArmed();
	    
      g.translate(x,y);

      Icon icon=null;

      // if(isEnabled)
      {
	if(isSelected)
	  icon=SlafManager.getIcon("RadioButtonMenuItem.selectedIcon"); 
	else
	  icon=SlafManager.getIcon("RadioButtonMenuItem.unselectedIcon"); 
      }

      if(icon!=null) icon.paintIcon(c,g,0,0);
      g.translate(-x,-y);
    }

    public int getIconWidth()
    {
      int  r=SIZE;
      Icon i=SlafManager.getIcon("RadioButtonMenuItem.selectedIcon");
      if(i!=null) r=i.getIconWidth();
      return r;
    }
    
    public int getIconHeight()
    {
      int  r=SIZE;
      Icon i=SlafManager.getIcon("RadioButtonMenuItem.selectedIcon");
      if(i!=null) r=i.getIconHeight();
      return r;
    }
  }
}
