/**
 * @modification $Date: 2004/01/26 18:17:33 $
 * @statut       unstable
 * @file         com/memoire/slaf/SlafInternalFrameUI.java
 * @version      0.20
 * @author       Guillaume Desnoix
 * @email        guillaume-desnoix@memoire.com
 * @license      GNU General Public License 2 (GPL2)
 * @copyright    1999-2004 Guillaume Desnoix
 */

package com.memoire.slaf;
import com.memoire.slaf.*;

import java.awt.*;
import java.awt.event.*;
import java.util.*;
import javax.swing.*;
import javax.swing.border.*;
import javax.swing.event.*;
import javax.swing.plaf.basic.*;
import java.util.EventListener;
import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyVetoException;
import javax.swing.plaf.*;
import java.io.Serializable;

public class SlafInternalFrameUI extends BasicInternalFrameUI
{
  private SlafInternalFrameTitlePane titlePane;

  private PropertyChangeListener pclistener;//,blistener;
  private ComponentListener      clistener;

  private static final Border EMPTY0000 =new EmptyBorder(0,0,0,0);
  private static final String IS_PALETTE="JInternalFrame.isPalette";

  public SlafInternalFrameUI(JInternalFrame b)
  {
    super(b);
  }
  
  public static ComponentUI createUI(JComponent c)
  {
    return new SlafInternalFrameUI((JInternalFrame)c);
  }

  public void installUI(JComponent c)
  { 
    super.installUI(c);

    SlafUtils.setOpaque(c);

    //frame=(JInternalFrame)c;

    LookAndFeel.installColorsAndFont(c,
				     "InternalFrame.background",
				     "InternalFrame.foreground",
				     "InternalFrame.font");
    //LookAndFeel.installBorder(c,"InternalFrame.border");

    Object paletteProp=frame.getClientProperty(IS_PALETTE);
    if(paletteProp!=null) setPalette(((Boolean)paletteProp).booleanValue());
    else                  setPalette(false);

    //blistener=new BListener();
    setContentBorder(frame.getContentPane());    

    pclistener=new PCListener();
    frame.addPropertyChangeListener(pclistener);

    clistener=new CListener();
    frame.addComponentListener(clistener);
  }
  
  public void uninstallUI(JComponent c)
  {                  
    frame.removePropertyChangeListener(pclistener);
    frame.removeComponentListener(clistener);

    unlinify();
    unsetContentBorder(frame.getContentPane());

    /*
    if(isAlphaPalette())
    {
      Vector v=SlafUtils.getAllSubComponents(c);
      for(int i=v.size()-1; i>=0; i--)
      {
        Object o=v.elementAt(i);
	SlafUtils.clearTransparent((JComponent)o);
	((JComponent)o).putClientProperty("SLAF_NO_TEXTURE",null);
      }
    }
    */

    c.setOpaque(true);
    super.uninstallUI(c);
  }

  public void unlinify()
  {
    Dimension as=frame.getSize();
    Dimension ms=titlePane.getMinimumFrameSize();

    if(as.height==ms.height)
      titlePane.linify();
  }

  private void setContentBorder(Object _o)
  {
    if(_o instanceof JComponent)
    {
      JComponent c=(JComponent)_o;
      Border     b=c.getBorder();
      if((b==null)||(b instanceof UIResource))
	c.setBorder(EMPTY0000);

      /*
      c.removePropertyChangeListener(blistener);

      Border b=c.getBorder();
      if((b==null)||(b instanceof UIResource))
      {
	orig_    =null;
	compound_=SlafManager.getBorder("InternalFrame.contentBorder",null);
	if(compound_==null)
	{
	  compound_=new BorderUIResource(new EmptyBorder(0,0,0,0));
	  System.err.println("*** IF.contentBorder not defined");
	}
	System.err.println("*** setContentBorder1 "+compound_+" ("+c.getName()+")");
	compound_=new LineBorder(Color.green,10);
      }
      else
      {
	orig_    =b;
	compound_=SlafManager.getBorder("InternalFrame.contentBorder",null);
	if(compound_!=null)
	  compound_=new BorderUIResource(new CompoundBorder(orig_,compound_));
	else
	{
	  compound_=new BorderUIResource(new EmptyBorder(0,0,0,0));
	  System.err.println("*** IF.contentBorder not defined");
	}

	System.err.println("*** setContentBorder2 "+compound_);
      }

      c.setBorder(compound_);
      c.addPropertyChangeListener(blistener);
    */
    }
  }

  private void unsetContentBorder(Object _o)
  {
    if(_o instanceof JComponent)
    {
      JComponent c=(JComponent)_o;
      Border     b=c.getBorder();
      if((b==EMPTY0000)||(b instanceof UIResource))
	c.setBorder(null);

      /*
      c.removePropertyChangeListener(blistener);
      Border b=c.getBorder();
      if((b==compound_)||(b==null)||(b instanceof UIResource))
      {
	c.setBorder(orig_);
	orig_    =null;
	compound_=null;
      }
      System.err.println("*** unsetContentBorder "+b+" ("+c.getName()+")");
      */
    }
  }


  protected JComponent createNorthPane(JInternalFrame frame)
  {
    titlePane=new SlafInternalFrameTitlePane(frame);
    return titlePane;
  }

  /*
  protected JComponent createSouthPane(JInternalFrame frame)
  {
    return new JLabel("hello");
  }
  */

  protected MouseInputAdapter createBorderListener(JInternalFrame w)
  {
    return new SlafBorderListener();
  }

  public boolean isPalette()
  {
    return ((titlePane!=null) ? titlePane.isPalette() : false);
  }

  public void setPalette(boolean isPalette)
  {
    titlePane.setPalette(isPalette);

    String r=(isPalette
	      ? "InternalFrame.paletteBorder"
	      : "InternalFrame.border");

    if(  isPalette
       &&!titlePane.shouldBeVisible()
       &&(SlafManager.getBorder("InternalFrame.untitledPaletteBorder")!=null))
      r="InternalFrame.untitledPaletteBorder";      

    /*
    if(isPalette)
    {
      System.err.println("TITLE      : "+frame.getTitle());
      System.err.println("ICONIFIABLE: "+frame.isIconifiable());
      System.err.println("SHOULD BE  : "+titlePane.shouldBeVisible());
      System.err.println("SET-BORDER: "+r);
    }
    */

    LookAndFeel.installBorder(frame,r);

    forcePaletteTransparency();

    Dimension d=titlePane.getMinimumFrameSize();
    frame.setMinimumSize(d);
    frame.invalidate();
    frame.doLayout();
    frame.validate();
  }

  public boolean isAlphaPalette()
  {
    return   isPalette()
	   &&(SlafManager.getInteger("InternalFrame.alpha",-1)!=-1);
  }

  protected void forcePaletteTransparency()
  {
    if(isAlphaPalette())
    {
      Vector v=SlafUtils.getAllSubComponents(frame);

      for(int i=v.size()-1; i>=0; i--)
      {
	Object o=v.elementAt(i);
	if(  (o instanceof JPanel)
	   //||(o instanceof AbstractButton)
	   ||(o instanceof JViewport)
	   ||(o instanceof JScrollPane)
	   ||(o instanceof JScrollBar)
	   ||(o instanceof JList)
	   ||(o instanceof JTree)
	   ||(o instanceof JTabbedPane)
	   ||(o instanceof SlafInternalFrameTitlePane)
	   ||(o instanceof JInternalFrame)
	  )
	{
	  SlafUtils.forceTransparent((JComponent)o);
	  ((JComponent)o).putClientProperty("SLAF_NO_TEXTURE",Boolean.TRUE);
	}
	if(((o instanceof JButton)||(o instanceof JToggleButton))
	   &&(((JComponent)o).getParent() instanceof JPanel))
	{
	  SlafUtils.forceTransparent((JComponent)o);
	  ((JComponent)o).putClientProperty("SLAF_NO_TEXTURE",Boolean.TRUE);
	  ((AbstractButton)o).setBorderPainted(false);
	  //((JButton)o).setRolloverEnabled(false);
	}
      }

      /*
      SlafUtils.forceTransparent(titlePane);
      titlePane
	  .putClientProperty("SLAF_NO_TEXTURE",Boolean.TRUE);
      */
    }
  }

  protected void paintBackground(Graphics g, JInternalFrame b)
  {
    Rectangle r=SlafUtils.getBackgroundRect  (b);
    // int    t=SlafUtils.NORMAL;
    Color     c=SlafUtils.getBackground(b);

    // if(b.isSelected()) t=SlafUtils.DARK;
    SlafUtils.paintBackground(g,r,b,c); // t

    if(isAlphaPalette())
    {
      c=b.getBackground();
      //r=SlafUtils.getViewRect(b);
      int alpha=SlafManager.getInteger("InternalFrame.alpha",-1);
      if(alpha!=-1)
      {
	g.setColor(new Color(c.getRed(),c.getGreen(),c.getBlue(),alpha));
	g.fillRect(r.x,r.y,r.width,r.height);
	g.setColor(new Color(0,0,0,32));
	g.fillRect(r.x,r.y,r.width,r.height);
      }
    }
  }
  
  public void update(Graphics g, JComponent c)
  {
    /*
    if(isPalette())
    {
      System.err.println("###### ALPHA");
      g=new SlafAlphaGraphics(g);
    }
    */

    paintBackground(g,(JInternalFrame)c);
    paint(g,c);
  }

  public void paint(Graphics g, JComponent c)
  {
    super.paint(g,c);
      
    // TMP @GDX
    /*
    Border b=c.getBorder();
    if(b!=null)
    {
      Rectangle r=SlafUtils.getFullRect(c);
      b.paintBorder(c,g,r.x,r.y,r.width,r.height);
    }
    */
  }

  /*
  class BListener implements PropertyChangeListener
  {
    public BListener()
    {
    }

    public void propertyChange(PropertyChangeEvent _evt)
    {
      System.err.println("*** blistener: "+_evt.getPropertyName()+" "+_evt.getSource());
      if(  (content_==_evt.getSource())
	 &&"border".equals(_evt.getPropertyName()))
      {
	unsetContentBorder(content_);
	setContentBorder(content_);
      }
    }
  }
  */

  class PCListener implements PropertyChangeListener
  {
    boolean sbv_;

    public PCListener()
    {
      sbv_=titlePane.shouldBeVisible();
    }

    public void propertyChange(PropertyChangeEvent _evt)
    {
      String name=_evt.getPropertyName();

      if(name.equals(IS_PALETTE))
      {
	if(_evt.getNewValue()!=null)
	  setPalette(((Boolean)_evt.getNewValue()).booleanValue());
	else
	  setPalette(false);
      }
      else
      if(name.equals(JInternalFrame.CONTENT_PANE_PROPERTY))
      {
	//System.err.println("*** pclistener: content");
	Object ov=_evt.getOldValue();
	unsetContentBorder(ov);
	Object nv=_evt.getNewValue();
	//content_=nv;
	setContentBorder(nv);
	forcePaletteTransparency();
      }
      else
      //if(SlafUtils.jdk()<=1.3)
      {
	// with JDK1.3, the property icon(ifi)able is not fired
	//if(Boolean.TRUE.equals(frame.getClientProperty(IS_PALETTE)))
	//setPalette(true);
	//}

	if(sbv_!=titlePane.shouldBeVisible())
        {
	  setPalette(Boolean.TRUE.equals(frame.getClientProperty(IS_PALETTE)));
	  sbv_=!sbv_;
	}
      }
    }
  }

  class CListener implements ComponentListener
  {
    public void componentShown(ComponentEvent _evt)
    {
      forcePaletteTransparency();
    }

    public void componentHidden (ComponentEvent _evt) { }
    public void componentResized(ComponentEvent _evt) { }

    public void componentMoved  (ComponentEvent _evt)
    {
      if(isAlphaPalette()||!frame.isOpaque()) frame.repaint(500);
    }
  }

  class SlafBorderListener extends BorderListener
  {
    private boolean first_ =true;
    private Point   anchor_=null;
    private Point   loc_   =null;

    public void mousePressed(MouseEvent e)
    {
      first_=true;
      super.mousePressed(e);
    }

    public void mouseReleased(MouseEvent e)
    {
      if(!first_)
	frame.getDesktopPane().getDesktopManager()
	  .endDraggingFrame(frame);
      super.mouseReleased(e);
    }

    public void mouseClicked(MouseEvent e)
    {
      if(  (e.getClickCount()>1)
	 &&(e.getSource()==getNorthPane())
	 &&"linify".equals
	   (SlafManager.getString("InternalFrameTitlePane.doubleClick")))
	titlePane.linify();
      else
	super.mouseClicked(e);
    }

    public void mouseDragged(MouseEvent e)
    {
      if(  (e.getSource()==frame)&&(getNorthPane()!=null)
	 &&(!getNorthPane().isVisible()||(getNorthPane().getHeight()==0)))
      {
        //e=new MouseEvent
	//  (getNorthPane(),e.getID(),e.getWhen(),e.getModifiers(),
	//  e.getX(),e.getY(),e.getClickCount(),e.isPopupTrigger());

	if(first_)
	{
	  first_=false;
	  anchor_=new Point(e.getX(),e.getY());
	  loc_   =frame.getLocation();
	  frame.getDesktopPane().getDesktopManager()
	    .beginDraggingFrame(frame);
	}
	else
	{
	  loc_=frame.getLocation();
	  frame.getDesktopPane().getDesktopManager()
	    .dragFrame(frame,
		       loc_.x+e.getX()-anchor_.x,
		       loc_.y+e.getY()-anchor_.y);
	}
      }

      super.mouseDragged(e);
    }
  }
}
