/**
 * @modification $Date: 2004/01/26 18:17:33 $
 * @statut       unstable
 * @file         com/memoire/slaf/SlafMixFilter.java
 * @version      0.20
 * @author       Guillaume Desnoix
 * @email        guillaume-desnoix@memoire.com
 * @license      GNU General Public License 2 (GPL2)
 * @copyright    1999-2004 Guillaume Desnoix
 */

package com.memoire.slaf;
import com.memoire.slaf.*;

import java.awt.*;
import java.awt.image.*;

public class SlafMixFilter extends RGBImageFilter
{
  private int a1_,r1_,g1_,b1_;
  private int a2_,r2_,g2_,b2_;
  private int wi_,hi_,wc_,hc_;
  private boolean fading_;

  public SlafMixFilter(int rgb2)
  {
    this(rgb2&0xFF000000,rgb2,0,0,false);
  }

  public SlafMixFilter(int rgb1, int rgb2)
  {
    this(rgb1,rgb2,0,0,false);
  }

  public SlafMixFilter(int rgb1, int rgb2, int wi, int hi)
  {
    this(rgb1,rgb2,wi,hi,false);
  }

  public SlafMixFilter(int rgb1, int rgb2, int wi, int hi, boolean fading)
  {
    a1_=(rgb1 & 0xff000000) >>24;
    r1_=(rgb1 & 0x00ff0000) >>16;
    g1_=(rgb1 & 0x0000ff00) >> 8;
    b1_=(rgb1 & 0x000000ff)     ;
    a2_=(rgb2 & 0xff000000) >>24;
    r2_=(rgb2 & 0x00ff0000) >>16;
    g2_=(rgb2 & 0x0000ff00) >> 8;
    b2_=(rgb2 & 0x000000ff)     ;

    if(a1_<0) a1_+=256;
    if(a2_<0) a2_+=256;

    wi_=wi-1;
    hi_=hi-1;

    fading_=fading;
    if((wi_<=0)||(hi_<=0)) fading_=false;

    if(fading_)
    {
      wc_=Math.min(Math.max(0,wi_/2),8);
      hc_=Math.min(Math.max(0,hi_/2),8);
    }
    else
    {
      wc_=0;
      hc_=0;
    }

    // canFilterIndexColorModel=true;
  }

  public int filterRGB(int x, int y, int rgb)
  {
    int a=(rgb & 0xff000000) >>24;
    int r=(rgb & 0x00ff0000) >>16;
    int g=(rgb & 0x0000ff00) >> 8;
    int b=(rgb & 0x000000ff)     ;

    int h=(299*r+587*g+114*b)/1000;
    int p=((255-h)*a1_+h*a2_)/255;
    //if(p<  0) System.err.println("MIXF: p<0");
    //if(p>255) System.err.println("MIXF: p>255");

    if(fading_)//(wi_>0)&&(hi_>0))
    {
           if(x<wc_    ) p=Math.max(0,p-8*(wc_-x));
      else if(x>wi_-wc_) p=Math.max(0,p-8*(wc_-wi_+x));
           if(y<hc_    ) p=Math.max(0,p-8*(hc_-y));
      else if(y>hi_-hc_) p=Math.max(0,p-8*(hc_-hi_+y));
    }

    if(p>0)
    {
      r=((255-p)*r+p*((255-h)*r1_+h*r2_)/255)/255;
      g=((255-p)*g+p*((255-h)*g1_+h*g2_)/255)/255;
      b=((255-p)*b+p*((255-h)*b1_+h*b2_)/255)/255;
    }

    return (a<<24) | (r<<16) | (g<<8) | b;
  }
}
