/**
 * @modification $Date: 2004/01/26 18:17:33 $
 * @statut       unstable
 * @file         com/memoire/slaf/SlafScrollBarUI.java
 * @version      0.20
 * @author       Guillaume Desnoix
 * @email        guillaume-desnoix@memoire.com
 * @license      GNU General Public License 2 (GPL2)
 * @copyright    1999-2004 Guillaume Desnoix
 */

package com.memoire.slaf;
import com.memoire.slaf.*;

import java.awt.*;
import java.awt.event.*;
import java.io.Serializable;
import java.beans.*;

import javax.swing.*;
import javax.swing.border.*;
import javax.swing.event.*;
import javax.swing.plaf.*;
import javax.swing.plaf.basic.BasicScrollBarUI;

public class SlafScrollBarUI extends BasicScrollBarUI
{
  protected JButton increaseButton;
  protected JButton decreaseButton;

  protected static int scrollBarWidth;

  protected ScrollBarListener scrollBarListener;
  protected RolloverListener  rolloverListener;

  public static final String FREE_STANDING_PROP="JScrollBar.isFreeStanding";
  protected boolean isFreeStanding=true;

  protected boolean isRollover=false;
  protected boolean isPressed =false;

  public static ComponentUI createUI( JComponent c )
  {
    return new SlafScrollBarUI();
  }

  public void installUI(JComponent c)
  {
    scrollBarWidth=SlafManager.getInteger("ScrollBar.width");
    // SlafUtils.trace("%%% SBW="+scrollBarWidth);

    if((c.getBackground()==null)||(c.getBackground() instanceof UIResource))
      c.setBackground(SlafManager.getColor("ScrollBar.background"));
    
    super.installUI(c);
    SlafUtils.setOpaque(c);

    scrollBarListener=new ScrollBarListener();
    c.addPropertyChangeListener(scrollBarListener);

    if(SlafManager.getBoolean
       ("ScrollBar.rollover",
	SlafManager.getBoolean("Theme.rollover",true)))
    {
      rolloverListener=new RolloverListener(c);
      c.addMouseListener(rolloverListener);
    }

    scrollBarListener.handlePropertyChange
      (c.getClientProperty(FREE_STANDING_PROP));
  }

  public void uninstallUI(JComponent c)
  {
    super.uninstallUI(c);
    c.removePropertyChangeListener(scrollBarListener);
    if(rolloverListener!=null) c.removeMouseListener(rolloverListener);
  }

  protected void configureScrollBarColors()
  {
  }

  public Dimension getPreferredSize(JComponent c)
  {
    if(scrollbar.getOrientation()==JScrollBar.VERTICAL)
      return new Dimension(scrollBarWidth,scrollBarWidth*4+10);
    else  // Horizontal
      return new Dimension(scrollBarWidth*4+10,scrollBarWidth);
  }

  protected JButton createDecreaseButton( int orientation )
  {
    decreaseButton=new SlafScrollButton
      (orientation,scrollBarWidth,isFreeStanding);
    return decreaseButton;
  }
  
  protected JButton createIncreaseButton( int orientation )
  {
    increaseButton=new SlafScrollButton
      (orientation,scrollBarWidth,isFreeStanding);
    return increaseButton;
  }

  /*
  public Rectangle getFullRect(JComponent b)
  {
    Dimension d=b.getSize();
    Rectangle r=new Rectangle(d);
    return r;
  }
  */

  protected Color getBackground(JComponent b)
  {
    Color r=b.getBackground();
    
    if(b.isEnabled()&&(isRollover||isPressed))
      r=SlafUtils.brighten(r,1);
    else
    if(!b.isOpaque()&&(b.getParent()!=null))
      r=b.getParent().getBackground();

    return r;
  }

  protected void paintBackground(Graphics g, JComponent b)
  {
    Rectangle r=SlafUtils.getFullRect(b);
    Color     c=getBackground(b);
    SlafUtils.paintBackground(g,r,b,c);
  }
  
  public void update(Graphics g, JComponent c)
  {
    paintBackground(g,c);
    paint(g,c);
  }

  protected void paintTrack( Graphics g, JComponent c, Rectangle trackBounds )
  {
    if((trackBounds.width==0)||(trackBounds.height==0))
      return;

    g.translate(trackBounds.x,trackBounds.y);

    boolean inactive=Boolean.TRUE.equals
      (c.getClientProperty("SLAF_INACTIVE_SCROLLBAR"));
    
    Color sbbg=getBackground(c);

    Color tkbg=SlafManager.getColor
      (inactive ? "ScrollTrack.inactiveBackground" : "ScrollTrack.background",
       SlafManager.getColor
       ("ScrollTrack.background",
	SlafManager.getColor("ScrollBar.trackBackground",sbbg)));

    boolean o=SlafManager.getBoolean
      ("ScrollTrack.opaque",
       SlafManager.getBoolean("ScrollBar.trackOpaque",true));

    if(scrollbar.getOrientation()==JScrollBar.VERTICAL)
    {
      if(!isFreeStanding) trackBounds.width+= 2;

      if(o)
      {
	g.setColor(tkbg);
	g.fillRect(0,0,trackBounds.width,trackBounds.height);
      }

      Border b=SlafManager.getBorder
	(inactive
	 ? "ScrollTrack.inactiveVerticalBorder"
	 : "ScrollTrack.verticalBorder",
	 SlafManager.getBorder
	 ("ScrollTrack.verticalBorder",
	  SlafManager.getBorder
	  ("ScrollTrack.border",
	   SlafManager.getBorder("ScrollBar.trackBorder"))));

      Rectangle r=new Rectangle(0,0,trackBounds.width,trackBounds.height);
      int t=SlafUtils.ENABLED;
      //if(isRollover) t=SlafUtils.ROLLOVER;
      //if(isPressed ) t=SlafUtils.PRESSED;
      SlafUtils.paintBackground(g,r,c,tkbg,t,"ScrollTrack");

      g.setColor(sbbg);

      if(b!=null)
	b.paintBorder(c,g,0,0,trackBounds.width,trackBounds.height);

      if(SlafManager.getBoolean("ScrollBar.lineVisible",true))
      {
	int x0=(trackBounds.width+1)/2-1;

	b=SlafManager.getBorder("ScrollBar.lineBorder");
	if(b!=null)
	  b.paintBorder(c,g,x0,1,3,trackBounds.height-2);

	if(SlafManager.getBoolean("ScrollBar.lineOpaque",true))
	{
	  g.setColor(sbbg.darker().darker());
	  g.drawLine(x0+1,2,x0+1,trackBounds.height-3);
	}
      }

      if(!isFreeStanding) trackBounds.width-= 2;
    }
    else  // HORIZONTAL
    {
      if(!isFreeStanding) trackBounds.height+=2;

      if(o)
      {
	g.setColor(tkbg);
	g.fillRect(0,0,trackBounds.width,trackBounds.height);
      }

      Border b=SlafManager.getBorder
	(inactive
	 ? "ScrollTrack.inactiveHorizontalBorder"
	 : "ScrollTrack.horizontalBorder",
	 SlafManager.getBorder
	  ("ScrollTrack.horizontalBorder",
	   SlafManager.getBorder
	   ("ScrollTrack.border",
	    SlafManager.getBorder("ScrollBar.trackBorder"))));

      Rectangle r=new Rectangle(0,0,trackBounds.width,trackBounds.height);
      int t=SlafUtils.ENABLED;
      //if(isRollover) t=SlafUtils.ROLLOVER;
      //if(isPressed ) t=SlafUtils.PRESSED;
      SlafUtils.paintBackground(g,r,c,tkbg,t,"ScrollTrack");

      g.setColor(sbbg);

      if(b!=null)
	b.paintBorder(c,g,0,0,trackBounds.width,trackBounds.height);

      if(SlafManager.getBoolean("ScrollBar.lineVisible",true))
      {
	int y0=(trackBounds.height+1)/2-1;

	b=SlafManager.getBorder("ScrollBar.lineBorder");
	if(b!=null)
	  b.paintBorder(c,g,1,y0,trackBounds.width-2,3);

	if(SlafManager.getBoolean("ScrollBar.lineOpaque",true))
	{
	  g.setColor(sbbg.darker().darker());
	  g.drawLine(2,y0+1,trackBounds.width-3,y0+1);
	}
      }

      if(!isFreeStanding) trackBounds.height-=2;
    }
    
    g.translate(-trackBounds.x, -trackBounds.y);
  }

  protected void paintThumb(Graphics g,JComponent c,Rectangle thumbBounds )
  {
    if((thumbBounds.width==0)||(thumbBounds.height==0))
      return;

    int     gap;
    Color   bg;
    boolean o;

    g.translate(thumbBounds.x, thumbBounds.y);

    o=SlafManager.getBoolean
      ("ScrollThumb.opaque",
       SlafManager.getBoolean("ScrollBar.thumbOpaque",true));

    gap=SlafManager.getInteger("ScrollBar.gap",3);

    bg=SlafManager.getColor
      ("ScrollThumb.background",
       SlafManager.getColor("ScrollBar.thumbBackground"));

    if(bg!=null)
    {
      if(c.isEnabled()&&(isRollover||isPressed))
	bg=SlafUtils.brighten(bg,1);
    }
    else
    {
      bg=getBackground(c);
      if(c.isEnabled()) bg=SlafUtils.darken(bg);
    }
    g.setColor(bg);

    if(scrollbar.getOrientation()==JScrollBar.VERTICAL)
    {
      if(!isFreeStanding) thumbBounds.width+=2;

      if(o)
	g.fillRect(gap,0,thumbBounds.width-2*gap,thumbBounds.height);

      Rectangle r=new Rectangle
	(gap,0,thumbBounds.width-2*gap,thumbBounds.height);
      int t=SlafUtils.ENABLED;
      if(c.isEnabled())
      {
	if(isRollover) t=SlafUtils.ROLLOVER;
	if(isPressed ) t=SlafUtils.PRESSED;
      }
      SlafUtils.paintBackground(g,r,c,bg,t,"ScrollThumb",true);

      Border b=null;
      if(isPressed)
	b=SlafManager.getBorder
	  ("ScrollThumb.pressedVerticalBorder",
	   SlafManager.getBorder("ScrollThumb.pressedBorder"));
      else
      if(isRollover)
	b=SlafManager.getBorder
	  ("ScrollThumb.rolloverVerticalBorder",
	   SlafManager.getBorder("ScrollThumb.rolloverBorder"));

      if(b==null)
	b=SlafManager.getBorder
	  ("ScrollThumb.verticalBorder",
	   SlafManager.getBorder
	   ("ScrollThumb.border",
	    SlafManager.getBorder("ScrollBar.thumbBorder")));

      if(b!=null)
	b.paintBorder(c,g,gap,0,thumbBounds.width-2*gap,thumbBounds.height);

      Icon i=SlafManager.getIcon("ScrollBar.verticalIcon");
      if(i!=null)
	i.paintIcon(c,g,(thumbBounds.width-i.getIconWidth())/2,
		    (thumbBounds.height-i.getIconHeight())/2);

      if(!isFreeStanding) thumbBounds.width-=2;
    }
    else  // HORIZONTAL
    {
      if(!isFreeStanding) thumbBounds.height+= 2;

      if(o)
	g.fillRect(0,gap,thumbBounds.width,thumbBounds.height-2*gap);
      
      Rectangle r=new Rectangle
	(0,gap,thumbBounds.width,thumbBounds.height-2*gap);
      int t=SlafUtils.ENABLED;
      if(c.isEnabled())
      {
	if(isRollover) t=SlafUtils.ROLLOVER;
	if(isPressed ) t=SlafUtils.PRESSED;
      }
      SlafUtils.paintBackground(g,r,c,bg,t,"ScrollThumb",true);

      Border b=null;
      if(isPressed)
	b=SlafManager.getBorder
	  ("ScrollThumb.pressedHorizontalBorder",
	   SlafManager.getBorder("ScrollThumb.pressedBorder"));
      else
      if(isRollover)
	b=SlafManager.getBorder
	  ("ScrollThumb.rolloverHorizontalBorder",
	   SlafManager.getBorder("ScrollThumb.rolloverBorder"));

      if(b==null)
	b=SlafManager.getBorder
	  ("ScrollThumb.horizontalBorder",
	   SlafManager.getBorder
	   ("ScrollThumb.border",
	    SlafManager.getBorder("ScrollBar.thumbBorder")));

      if(b!=null)
	b.paintBorder(c,g,0,gap,thumbBounds.width,thumbBounds.height-2*gap);

      Icon i=SlafManager.getIcon("ScrollBar.horizontalIcon");
      if(i!=null)
	i.paintIcon(c,g,(thumbBounds.width-i.getIconWidth())/2,
		    (thumbBounds.height-i.getIconHeight())/2);

      if(!isFreeStanding) thumbBounds.height-=2;
    }
    
    g.translate(-thumbBounds.x, -thumbBounds.y);
  }

  protected Dimension getMinimumThumbSize()
  {
    Dimension r=SlafManager.getSize("ScrollBar.minimumThumbSize");
    if(r==null) r=new Dimension(scrollBarWidth,scrollBarWidth);
    return r;
  }		

  protected void layoutHScrollbar(JScrollBar _scrollbar)  
  {
    int type=SlafManager.getInteger("ScrollBar.horizontalType",0);

      Dimension size  =_scrollbar.getSize();
      Insets    insets=_scrollbar.getInsets();

      final int h=size.height-insets.top-insets.bottom;
      final int y=insets.top;
      int x,w;

      final int decrW=decrButton.getPreferredSize().width;
      final int incrW=incrButton.getPreferredSize().width;

      int decrX,incrX;
      int thumbX,thumbW;

      if(type==1)
      {
	decrX=insets.left;
	incrX=decrX+decrW;
	x=incrX+incrW;
	//thumbX=size.width-insets.right;
      }
      else
      if(type==2)
      {
	incrX=size.width-insets.right-incrW;
	decrX=incrX-decrW;
	x=insets.left;
	//thumbX=decrX;
      }
      else
      {
	decrX=insets.left;
	incrX=size.width-insets.right-incrW;
	x=decrX+decrW;
	//thumbX=incrX;
      }
        
      w=size.width-insets.left-insets.right-incrW-decrW;

      decrButton.setBounds(decrX,y,decrW,h);
      incrButton.setBounds(incrX,y,incrW,h);
      trackRect .setBounds(x,y,w,h);

      decrButton.setVisible(w>=0);
      incrButton.setVisible(w>=0);

      float min   =_scrollbar.getMinimum();
      float max   =_scrollbar.getMaximum();
      float extent=_scrollbar.getVisibleAmount();
      float range =max-min;
      float value =_scrollbar.getValue();

      thumbW=(range<=0.0f) 
	? getMaximumThumbSize().width
	: (int)(w*(extent/range));
      thumbW=Math.max(thumbW,getMinimumThumbSize().width);
      thumbW=Math.min(thumbW,getMaximumThumbSize().width);

      if(thumbW>=w)
      {
	setThumbBounds(0,0,0,0);
	if(!Boolean.TRUE.equals
	   (_scrollbar.getClientProperty("SLAF_INACTIVE_SCROLLBAR")))
	{
	  _scrollbar.putClientProperty("SLAF_INACTIVE_SCROLLBAR",Boolean.TRUE);
	  decrButton.repaint();
	  incrButton.repaint();
	}
      }
      else
      {
	if(!Boolean.FALSE.equals
	   (_scrollbar.getClientProperty("SLAF_INACTIVE_SCROLLBAR")))
	{
	  _scrollbar.putClientProperty("SLAF_INACTIVE_SCROLLBAR",Boolean.FALSE);
	  decrButton.repaint();
	  incrButton.repaint();
	}

	//thumbX=x;
	if(value<(max-extent))
	{
	  float thumbRange=w-thumbW;
	  thumbX=x+(int)(0.5f+(thumbRange
			       *((value-min)/(range-extent))));
	  //thumbY+=decrButtonY + decrButtonH;
	}
	else thumbX=x+w-thumbW;  

	setThumbBounds(thumbX,y,thumbW,h);
      }
  }

  protected void layoutVScrollbar(JScrollBar _scrollbar)  
  {
    int type=SlafManager.getInteger("ScrollBar.verticalType",0);

      Dimension size  =_scrollbar.getSize();
      Insets    insets=_scrollbar.getInsets();

      final int w=size.width-insets.left-insets.right;
      final int x=insets.left;
      int y,h;

      final int decrH=decrButton.getPreferredSize().height;
      final int incrH=incrButton.getPreferredSize().height;

      int decrY,incrY;
      int thumbY,thumbH;

      if(type==1)
      {
	decrY=insets.top;
	incrY=decrY+decrH;
	y=incrY+incrH;
	//thumbY=size.height-insets.bottom;
      }
      else
      if(type==2)
      {
	incrY=size.height-insets.bottom-incrH;
	decrY=incrY-decrH;
	y=insets.top;
	//thumbY=decrY;
      }
      else
      {
	decrY=insets.top;
	incrY=size.height-insets.bottom-incrH;
	y=decrY+decrH;
	//thumbY=incrY;
      }

      h=size.height-insets.top-insets.bottom-incrH-decrH;

      decrButton.setBounds(x,decrY,w,decrH);
      incrButton.setBounds(x,incrY,w,incrH);
      trackRect .setBounds(x,y,w,h);

      decrButton.setVisible(h>=0);
      incrButton.setVisible(h>=0);

      float min   =_scrollbar.getMinimum();
      float max   =_scrollbar.getMaximum();
      float extent=_scrollbar.getVisibleAmount();
      float range =max-min;
      float value =_scrollbar.getValue();

      thumbH=(range<=0.0f) 
	? getMaximumThumbSize().height
	: (int)(h*(extent/range));
      thumbH=Math.max(thumbH,getMinimumThumbSize().height);
      thumbH=Math.min(thumbH,getMaximumThumbSize().height);
        
      if(thumbH>=h)
      {
	setThumbBounds(0,0,0,0);
	if(!Boolean.TRUE.equals
	   (_scrollbar.getClientProperty("SLAF_INACTIVE_SCROLLBAR")))
	{
	  _scrollbar.putClientProperty("SLAF_INACTIVE_SCROLLBAR",Boolean.TRUE);
	  decrButton.repaint();
	  incrButton.repaint();
	}
      }
      else
      {
	if(!Boolean.FALSE.equals
	   (_scrollbar.getClientProperty("SLAF_INACTIVE_SCROLLBAR")))
	{
	  _scrollbar.putClientProperty("SLAF_INACTIVE_SCROLLBAR",Boolean.FALSE);
	  decrButton.repaint();
	  incrButton.repaint();
	}

	//thumbY-=thumbH;  
	//thumbY=y;
	if(value<(max-extent))
	{
	  float thumbRange=h-thumbH;
	  thumbY=y+(int)(0.5f+(thumbRange
			       *((value-min)/(range-extent))));
	  //thumbY+=decrButtonY + decrButtonH;
	}
	else thumbY=y+h-thumbH;

	setThumbBounds(x,thumbY,w,thumbH);
      }
  }

  protected TrackListener createTrackListener()
  {
    return new SlafTrackListener();
  }

  protected class SlafTrackListener
    extends TrackListener
  {
    public void mouseDragged(MouseEvent e)
    {
      if(!scrollbar.isEnabled()||!isDragging)	return;
      
      BoundedRangeModel model=scrollbar.getModel();
      Rectangle thumbR=getThumbBounds();
      float trackLength;
      int thumbMin, thumbMax, thumbPos;
      
      if(scrollbar.getOrientation()==JScrollBar.VERTICAL)
      {
	thumbMin=trackRect.y;
	thumbMax=trackRect.y+trackRect.height-getThumbBounds().height;
	thumbPos=Math.min(thumbMax,Math.max(thumbMin,(e.getY()-offset)));
	setThumbBounds(thumbR.x, thumbPos, thumbR.width, thumbR.height);
	trackLength=getTrackBounds().height;
      }
      else
      {
	thumbMin=trackRect.x;
	thumbMax=trackRect.x+trackRect.width-getThumbBounds().width;
	thumbPos=Math.min(thumbMax,Math.max(thumbMin,(e.getX()-offset)));
	setThumbBounds(thumbPos, thumbR.y, thumbR.width, thumbR.height);
	trackLength = getTrackBounds().width;
      }
      
      if(thumbPos==thumbMax)
      {
	scrollbar.setValue(model.getMaximum()-model.getExtent());
      }
      else
      if(thumbPos==thumbMin)
      {
	scrollbar.setValue(model.getMinimum());
      }
      else
      {
	float valueMax  =model.getMaximum()-model.getExtent();
	float valueMin  =model.getMinimum();
	float valueRange=valueMax-valueMin;
	float thumbValue=thumbPos-thumbMin;
	float thumbRange=thumbMax-thumbMin;
	int value=(int)(0.5+((thumbValue/thumbRange)*valueRange)+valueMin);
	scrollbar.setValue(value);
      }
    }
  }

  class RolloverListener extends MouseAdapter
  {
    private JComponent c_;

    public RolloverListener(JComponent _c)
    {
      c_=_c;
    }

    public void mousePressed(MouseEvent _evt)
    {
      isPressed=true;
      c_.repaint();
    }

    public void mouseReleased(MouseEvent _evt)
    {
      isPressed=false;
      c_.repaint();
    }

    public void mouseEntered(MouseEvent _evt)
    {
      isRollover=true;
      c_.repaint();
    }

    public void mouseExited(MouseEvent _evt)
    {
      isRollover=false;
      c_.repaint();
    }
  }

  class ScrollBarListener implements PropertyChangeListener, Serializable
  {
    public void propertyChange(PropertyChangeEvent e)
    {
      String name=e.getPropertyName();
      if(name.equals(FREE_STANDING_PROP))
	handlePropertyChange(e.getNewValue());
    }

    public void handlePropertyChange( Object newValue )
    {
      if(newValue!=null)
      {
	boolean temp = ((Boolean)newValue).booleanValue();
	boolean becameFlush = temp == false && isFreeStanding == true;
	boolean becameNormal = temp == true && isFreeStanding == false;
		
	isFreeStanding = temp;

	if(becameFlush)
	  toFlush();
	else
	if(becameNormal)
	  toFreeStanding();
      }
      else
      {
	if(!isFreeStanding)
	{
	  isFreeStanding=true;
	  toFreeStanding();
	}
	  
	// This commented-out block is used for testing flush scrollbars.
	/*
	  if(isFreeStanding)
	  {
	    isFreeStanding=false;
	    toFlush();
	  }
	*/
      }
      
      if(increaseButton!=null)
      {
	// increaseButton.setFreeStanding( isFreeStanding );
      }
      if(decreaseButton!=null)
      {
	// decreaseButton.setFreeStanding( isFreeStanding );
      }	    
    }
    
    protected void toFlush()
    {
      scrollBarWidth -= 2;
    }
    
    protected void toFreeStanding()
    {
      scrollBarWidth += 2;
    }
  }
}
