/**
 * @modification $Date: 2004/05/11 19:06:50 $
 * @statut       unstable
 * @file         com/memoire/slaf/SlafSliderUI.java
 * @version      0.20
 * @author       Guillaume Desnoix
 * @email        guillaume-desnoix@memoire.com
 * @license      GNU General Public License 2 (GPL2)
 * @copyright    1999-2004 Guillaume Desnoix
 */

package com.memoire.slaf;
import com.memoire.slaf.*;


import java.awt.*;
import java.awt.event.*;
import java.beans.*;

import javax.swing.*;
import javax.swing.event.*;
import javax.swing.plaf.*;
import javax.swing.border.*;
import javax.swing.plaf.basic.*;

public class SlafSliderUI extends BasicSliderUI
{
  protected final int TICK_BUFFER = 4;

  // protected static int sliderWidth;
  // protected static int trackWidth;
  // protected static int tickLength;

  protected RolloverListener rolloverListener;

  protected boolean isRollover=false;
  protected boolean isPressed =false;

  public static ComponentUI createUI(JComponent c)
  {
    return new SlafSliderUI();
  }
  
  public SlafSliderUI()
  {
    super( null );
  }

  public void installUI(JComponent c)
  {
    // sliderWidth=SlafManager.getInteger("Slider.width");
    // SlafUtils.trace("%%% SW="+sliderWidth);

    // trackWidth=((Integer)SlafManager.get("Slider.trackWidth")).intValue();
    // tickLength=((Integer)SlafManager.get("Slider.majorTickLength")).intValue();

    /*
      if((c.getBackground()==null)||(c.getBackground() instanceof UIResource))
      c.setBackground(SlafManager.getColor("Slider.background"));
    */

    super.installUI(c);
    SlafUtils.setOpaque(c);
    Dimension ps=c.getPreferredSize();
    c.setSize(ps);
    //c.setPreferredSize(ps);

    if(SlafManager.getBoolean
       ("Slider.rollover",
	SlafManager.getBoolean("Theme.rollover",true)))
    {
      rolloverListener=new RolloverListener(c);
      c.addMouseListener(rolloverListener);
    }

    scrollListener.setScrollByBlock(false);
  }
  
  public void uninstallUI(JComponent c)
  {
    super.uninstallUI(c);
    c.removeMouseListener(rolloverListener);
  }

  /*
  public Rectangle getFullRect(JComponent b)
  {
    Dimension d=b.getSize();
    Rectangle r=new Rectangle(d);
    return r;
  }
  */

  public Dimension getPreferredHorizontalSize()
  {
    return new Dimension(200,SlafManager.getInteger("Slider.width",21));
  }

  public Dimension getPreferredVerticalSize()
  {
    return new Dimension(SlafManager.getInteger("Slider.width",21),200);
  }

  public Dimension getMinimumHorizontalSize()
  {
    return new Dimension(36,SlafManager.getInteger("Slider.width",21));
  }

  public Dimension getMinimumVerticalSize()
  {
    return new Dimension(SlafManager.getInteger("Slider.width",21),36);
  }

  protected Color getBackground(JComponent b)
  {
    Color r=SlafManager.getColor("Slider.trackBackground",b.getBackground());
    
    if(b.isEnabled()&&(isRollover||isPressed))
      r=SlafUtils.brighten(r,1);
    else
    if(!b.isOpaque()&&(b.getParent()!=null))
      if(!SlafManager.getBoolean("Slider.trackOpaque",false))
	r=b.getParent().getBackground();

    return r;
  }

  protected void paintBackground(Graphics g, JComponent b)
  {
    /*
    Rectangle r;
    Color     c;

    if(b.isOpaque())
    {
      r=SlafUtils.getFullRect(b);
      c=b.getBackground();
      g.setColor(c);
      g.fillRect(r.x,r.y,r.width,r.height);
    }

    r=trackRect;
    c=getBackground(b);
    SlafUtils.paintBackground(g,r,b,c);
    */

    Rectangle r=SlafUtils.getFullRect(b);
    Color     c=getBackground(b);
    SlafUtils.paintBackground(g,r,b,c);
  }
  
  public void paintOver(Graphics g, JComponent c)
  {
    Rectangle r=new Rectangle(thumbRect);

    int gap=SlafManager.getInteger("Slider.gap",3);
    if(slider.getOrientation()==JSlider.HORIZONTAL)
      { r.y+=gap; r.height-=2*gap; }
    else
      { r.x+=gap; r.width-=2*gap; }
    
    SlafUtils.paintOver(g,r,c,SlafUtils.getBackground(c));
    // g.setColor(Color.red);
    // g.fillRect(r.x,r.y,r.width,r.height);
  }

  public void update(Graphics g, JComponent c)
  {
    paintBackground(g,c);
    paint(g,c);
    paintOver(g,c);
  }

  public void paintThumb(Graphics g)
  {
    int     gap;
    Color   bg;
    Border  b;
    boolean o;

    g.translate(thumbRect.x,thumbRect.y);

    b=SlafManager.getBorder
	("SliderThumb.border",
	 SlafManager.getBorder("Slider.thumbBorder"));

    o=SlafManager.getBoolean
	("SliderThumb.opaque",
	 SlafManager.getBoolean("Slider.thumbOpaque",true));

    gap=SlafManager.getInteger("Slider.gap",3);

    bg=SlafManager.getColor
	("SliderThumb.background",
	 SlafManager.getColor("Slider.thumbBackground"));

    if(bg!=null)
    {
      if(slider.isEnabled())
      {
	     if(isPressed)  bg=SlafUtils.darken(bg,1);
	else if(isRollover) bg=SlafUtils.brighten(bg,1);
      }
    }
    else
    {
      bg=getBackground(slider);
      if(slider.isEnabled()) bg=SlafUtils.darken(bg);
    }
    g.setColor(bg);

    if(slider.getOrientation()==JSlider.VERTICAL)
    {
      if(o)
	g.fillRect(gap,0,thumbRect.width-2*gap,thumbRect.height);

      Rectangle r=new Rectangle
	(gap,0,thumbRect.width-2*gap,thumbRect.height);
      int t=SlafUtils.ENABLED;
      if(slider.isEnabled())
      {
	if(isRollover) t=SlafUtils.ROLLOVER;
	if(isPressed ) t=SlafUtils.PRESSED;
      }
      SlafUtils.paintBackground(g,r,slider,bg,t,"SliderThumb",true);

      Icon i=null;
      if(isPressed)
	i=SlafManager.getIcon("Slider.pressedVerticalIcon",
			      SlafManager.getIcon("Slider.pressedIcon"));
      else
      if(isRollover)
	i=SlafManager.getIcon("Slider.rolloverVerticalIcon",
			      SlafManager.getIcon("Slider.rolloverIcon"));

      if(i==null)
	i=SlafManager.getIcon("Slider.verticalIcon",
			      SlafManager.getIcon("Slider.icon"));

      if(i!=null)
	i.paintIcon(slider,g,(thumbRect.width-i.getIconWidth())/2,
		    (thumbRect.height-i.getIconHeight())/2);

      if(b!=null)
	b.paintBorder(slider,g,gap,0,thumbRect.width-2*gap,thumbRect.height);
    }
    else
    {
      if(o)
	g.fillRect(0,gap,thumbRect.width,thumbRect.height-2*gap);

      Rectangle r=new Rectangle
	(0,gap,thumbRect.width,thumbRect.height-2*gap);
      int t=SlafUtils.ENABLED;
      /*
      if(slider.isEnabled())
      {
	if(isRollover) t=SlafUtils.ROLLOVER;
	if(isPressed ) t=SlafUtils.PRESSED;
      }
      */
      SlafUtils.paintBackground(g,r,slider,bg,t,"Slider",true);

      Icon i=null;
      if(isPressed)
	i=SlafManager.getIcon("Slider.pressedHorizontalIcon",
			      SlafManager.getIcon("Slider.pressedIcon"));
      else
      if(isRollover)
	i=SlafManager.getIcon("Slider.rolloverHorizontalIcon",
			      SlafManager.getIcon("Slider.rolloverIcon"));

      if(i==null)
	i=SlafManager.getIcon("Slider.horizontalIcon",
			      SlafManager.getIcon("Slider.icon"));

      if(i!=null)
	i.paintIcon(slider,g,(thumbRect.width-i.getIconWidth())/2,
		    (thumbRect.height-i.getIconHeight())/2);

      if(b!=null)
	b.paintBorder(slider,g,0,gap,thumbRect.width,thumbRect.height-2*gap);
    }

    /*
    if(slider.getOrientation()==JSlider.HORIZONTAL)
      horizThumbIcon.paintIcon( slider, g, 0, 0 );
    else
      vertThumbIcon.paintIcon( slider, g, 0, 0 );
    */

    g.translate(-thumbRect.x,-thumbRect.y);
  }

  public void paintTrack(Graphics g)
  {
    if((trackRect.width==0)||(trackRect.height==0))
      return;

    g.translate(trackRect.x,trackRect.y);
    
    Color  slbg=getBackground(slider);

    Color  tkbg=SlafManager.getColor
      ("SliderTrack.background",
       SlafManager.getColor("Slider.trackBackground",slbg));

    boolean o=SlafManager.getBoolean
      ("SliderTrack.opaque",
       SlafManager.getBoolean("Slider.trackOpaque",true));

    if(slider.getOrientation()==JSlider.VERTICAL)
    {
      if(o)
      {
	g.setColor(tkbg);
	g.fillRect(0,0,trackRect.width,trackRect.height);
      }

      Border b=SlafManager.getBorder
	  ("SliderTrack.verticalBorder",
	   SlafManager.getBorder
	   ("SliderTrack.border",
	    SlafManager.getBorder("Slider.trackBorder")));

      Rectangle r=new Rectangle(0,0,trackRect.width,trackRect.height);
      int t=SlafUtils.ENABLED;
      //if(isRollover) t=SlafUtils.ROLLOVER;
      //if(isPressed ) t=SlafUtils.PRESSED;
      SlafUtils.paintBackground(g,r,slider,tkbg,t,"SliderTrack");

      g.setColor(slbg);

      if(b!=null)
	b.paintBorder(slider,g,0,0,trackRect.width,trackRect.height);

      if(SlafManager.getBoolean("Slider.lineVisible",true))
      {
	int x0=(trackRect.width+1)/2-1;

	b=SlafManager.getBorder
	  ("Slider.verticalLineBorder",
	   SlafManager.getBorder("Slider.lineBorder"));

	if(b!=null)
	{
	  Insets bi=b.getBorderInsets(slider);
	  b.paintBorder
	    (slider,g,x0+1-bi.left,1,bi.left+bi.right,trackRect.height-2);
	}

	if(SlafManager.getBoolean("Slider.lineOpaque",true))
	{
	  g.setColor(slbg.darker().darker());
	  g.drawLine(x0+1,2,x0+1,trackRect.height-3);
	}
      }
    }
    else
    {
      if(o)
      {
	g.setColor(tkbg);
	g.fillRect(0,0,trackRect.width,trackRect.height);
      }

      Border b=SlafManager.getBorder
	  ("SliderTrack.horizontalBorder",
	   SlafManager.getBorder
	   ("SliderTrack.border",
	    SlafManager.getBorder("Slider.trackBorder")));

      Rectangle r=new Rectangle(0,0,trackRect.width,trackRect.height);
      int t=SlafUtils.ENABLED;
      //if(isRollover) t=SlafUtils.ROLLOVER;
      //if(isPressed ) t=SlafUtils.PRESSED;
      SlafUtils.paintBackground(g,r,slider,tkbg,t,"SliderTrack");

      g.setColor(slbg);

      if(b!=null)
	b.paintBorder(slider,g,0,0,trackRect.width,trackRect.height);

      if(SlafManager.getBoolean("Slider.lineVisible",true))
      {
	int y0=(trackRect.height+1)/2-1;

	b=SlafManager.getBorder
	  ("Slider.horizontalLineBorder",
	   SlafManager.getBorder("Slider.lineBorder"));

	if(b!=null)
	{
	  Insets bi=b.getBorderInsets(slider);
	  b.paintBorder(slider,g,1,y0+1-bi.top,
			trackRect.width-2,bi.top+bi.bottom);
	}

	if(SlafManager.getBoolean("Slider.lineOpaque",true))
	{
	  g.setColor(slbg.darker().darker());
	  g.drawLine(2,y0+1,trackRect.width-3,y0+1);
	}
      }
     }

    g.translate(-trackRect.x,-trackRect.y);
  }
        
  public void paintFocus(Graphics g)
  {
    if(SlafUtils.hasFocus(slider))
    {
      SlafUtils.paintFocus(g,slider,trackRect,tickRect,thumbRect);

      /*
      Rectangle r = slider.getBounds();
      r.x = 0;
      r.y = 0;
      if(slider.getBorder() != null)
      {
	r = getFullContentArea();
      }
      g.setColor(getFocusColor());
      g.drawRect(r.x+2,r.y+2,r.width-5,r.height-5);
      */
    }
  }

  public void paintTicks(Graphics g)
  {
    Rectangle tickBounds = tickRect;
    int i;
    int maj, min, max;
    int w = tickBounds.width;
    int h = tickBounds.height;
    int centerEffect, tickHeight;

    //if(slider.isOpaque())
    //{
    //g.setColor(slider.getBackground());
    //g.fillRect(tickBounds.x,tickBounds.y,
    //	 tickBounds.width,tickBounds.height);  
    //}

    g.setColor(slider.getForeground());

    maj = slider.getMajorTickSpacing();
    min = slider.getMinorTickSpacing();

    if(slider.getOrientation()==JSlider.HORIZONTAL)
    {
      g.translate(0,tickBounds.y);

      int value = slider.getMinimum();
      int xPos = 0;

      if ( slider.getMinorTickSpacing() > 0 ) {
	while ( value <= slider.getMaximum() ) {
	  xPos = xPositionForValue( value );
	  paintMinorTickForHorizSlider( g, tickBounds, xPos );
	  value += slider.getMinorTickSpacing();
	}
      }
      
      if ( slider.getMajorTickSpacing() > 0 ) {
	value = slider.getMinimum();
	
	while ( value <= slider.getMaximum() ) {
	  xPos = xPositionForValue( value );
	  paintMajorTickForHorizSlider( g, tickBounds, xPos );
	  value += slider.getMajorTickSpacing();
	}
      }

      g.translate( 0, -tickBounds.y);
    }
    else {
      g.translate(tickBounds.x, 0);
      
      int value = slider.getMinimum();
      int yPos = 0;
      
      if ( slider.getMinorTickSpacing() > 0 ) {
	while ( value <= slider.getMaximum() ) {
	  yPos = yPositionForValue( value );
	  paintMinorTickForVertSlider( g, tickBounds, yPos );
	  value += slider.getMinorTickSpacing();
	}
      }
      
      if ( slider.getMajorTickSpacing() > 0 ) {
	value = slider.getMinimum();
	
	while ( value <= slider.getMaximum() ) {
	  yPos = yPositionForValue( value );
	  paintMajorTickForVertSlider( g, tickBounds, yPos );
	  value += slider.getMajorTickSpacing();
	}
      }
      g.translate(-tickBounds.x, 0);
    }
  }

  /*
  public void calculateThumbBounds()
  {
    int thumbLength=13;

    if(slider.getOrientation()==JSlider.HORIZONTAL)
    {
      int thumbX=xPositionForValue(slider.getValue())-thumbRect.width/2;
      int thumbY=trackRect.y;
      int thumbW=trackRect.height;
      thumbRect.setBounds(thumbX,thumbY,thumbLength,thumbW);
    }
    else
    {
      int thumbY=yPositionForValue(slider.getValue())-thumbRect.height/2;
      int thumbX=trackRect.x;
      int thumbW=trackRect.width;
      thumbRect.setBounds(thumbX,thumbY,thumbW,thumbLength);
    }
  }
  */

  protected Dimension getThumbSize()
  {
    int w=SlafManager.getInteger("SliderThumb.width" ,16);
    int h=SlafManager.getInteger("SliderThumb.height",16);

    Dimension ms=SlafManager.getSize("Slider.maximumThumbSize",null);
    if(ms!=null)
    {
      if(w>ms.width ) w=ms.width;
      if(h>ms.height) h=ms.height;
    }

    if(slider.getOrientation()==JSlider.VERTICAL)
    {
      int tmp=w;
      w=h;
      h=tmp;
    }
    
    return new Dimension(w,h);
  }

  protected int getTickLength()
  {
    return SlafManager.getInteger("Slider.majorTickLength");
  }

  /*
  public Dimension getPreferredSize(JComponent c)
  {
    if(slider.getOrientation()==JSlider.VERTICAL)
      return new Dimension(sliderWidth+
			   (slider.getPaintTicks() ? getTickLength() : 0),
			   sliderWidth*4+10);
    else  // Horizontal
      return new Dimension(sliderWidth*4+10,sliderWidth+
			   (slider.getPaintTicks() ? getTickLength() : 0));
  }
  */

  /*
  public int getTickSpace()
  {
    return slider.getOrientation()==JSlider.HORIZONTAL
      ? tickLength+TICK_BUFFER+1
      : tickLength+TICK_BUFFER+3;
  }
  */

  /*
  protected int getTrackWidth()
  {
    return trackWidth;
  }
  */

  /*
  protected int getTrackLength()
  {
    //Rectangle interiorRect = getFullContentArea();
    
    if ( slider.getOrientation() == JSlider.HORIZONTAL ) {
      return trackRect.width - (trackBuffer * 2);
    }
    return trackRect.height - (trackBuffer * 2);
  }
  */

  /*
  protected int getThumbOverhang()
  {
    return SlafManager.getInteger("Slider.overhang",4);
  }
  */

  /*
  protected void scrollDueToClickInTrack(int dir)
  {
    scrollByUnit(dir);
  }
  */

  /*  
  protected void paintMinorTickForHorizSlider
    (Graphics g, Rectangle tickBounds, int x)
  {
    g.setColor(slider.isEnabled()
	       ? slider.getForeground()
	       : slider.getBackground().darker());

    g.drawLine(x,TICK_BUFFER,x,TICK_BUFFER+(getTickLength()/2));
  }

  protected void paintMajorTickForHorizSlider
    (Graphics g, Rectangle tickBounds, int x)
  {
    g.setColor(slider.isEnabled()
	       ? slider.getForeground()
	       : slider.getBackground().darker());
    g.drawLine(x,TICK_BUFFER,x,TICK_BUFFER+(getTickLength()-1));
  }
  
  protected void paintMinorTickForVertSlider
    (Graphics g, Rectangle tickBounds, int y)
  {
    g.setColor(slider.isEnabled()
	       ? slider.getForeground()
	       : slider.getBackground().darker());
    g.drawLine(TICK_BUFFER,y,TICK_BUFFER+(getTickLength()/2),y);
  }

  protected void paintMajorTickForVertSlider
    (Graphics g, Rectangle tickBounds, int y)
  {
    g.setColor(slider.isEnabled()
	       ? slider.getForeground()
	       : slider.getBackground().darker());
    g.drawLine(TICK_BUFFER,y,TICK_BUFFER+getTickLength(),y);
  }
  */

  class RolloverListener extends MouseAdapter
  {
    private JComponent c_;

    public RolloverListener(JComponent _c)
    {
      c_=_c;
    }

    public void mousePressed(MouseEvent _evt)
    {
      isPressed=true;
      c_.repaint();
    }

    public void mouseReleased(MouseEvent _evt)
    {
      isPressed=false;
      c_.repaint();
    }

    public void mouseEntered(MouseEvent _evt)
    {
      isRollover=true;
      c_.repaint();
    }

    public void mouseExited(MouseEvent _evt)
    {
      isRollover=false;
      c_.repaint();
    }
  }
}
