/**
 * @modification $Date: 2004/01/26 18:17:34 $
 * @statut       unstable
 * @file         com/memoire/slaf/SlafTabbedPaneUI.java
 * @version      0.20
 * @author       Guillaume Desnoix
 * @email        guillaume-desnoix@memoire.com
 * @license      GNU General Public License 2 (GPL2)
 * @copyright    1999-2004 Guillaume Desnoix
 */

package com.memoire.slaf;
import com.memoire.slaf.*;

import java.awt.*;
import java.awt.event.*;
import java.beans.*;
import java.io.Serializable; 
import java.util.*;

import javax.swing.*;
import javax.swing.border.*;
import javax.swing.event.*;
import javax.swing.plaf.*;

public class SlafTabbedPaneUI extends TabbedPaneUI
       implements LayoutManager, ChangeListener, Serializable, SwingConstants
{
  // Class variables 

  protected static int spacingHeight       = 0; //4;
  protected static int spacingWidth        = 3; //4;
  protected static int iconSpacingWidth    = 3;
  //protected static int selectedTabRaisePad = 3; // 3;
  //protected static int selectedTabWidenPad = 4; // 4;

  protected static Insets contentBorderInsets  = new Insets(3,3,3,3);
  protected static Insets tabBorderInsets      = new Insets(3,3,3,3);

  protected static Insets tabsOnTopTabInsets   = new Insets(0,0,5,10);
  protected static Insets tabsOnLeftTabInsets  = new Insets(0,0,10,5);
  protected static Insets tabsOnBottomTabInsets= new Insets(0,0,5,10);
  protected static Insets tabsOnRightTabInsets = new Insets(0,0,5,10);

  // Instance variables

  protected int xNudge_ = 0;
  protected int yNudge_ = 0;
  
  protected Color tabHighlight;
  protected Color tabShadow;
  protected Color tabDarkShadow;
  protected Color focus;

  // Transient variables (recalculated as events arrive)

  protected int tabRuns[] = new int[10];
  protected int runCount;
  protected int selectedRun;
  protected Rectangle rects[] = new Rectangle[0];
  
  protected int maxTabHeight;
  protected int maxTabWidth;
  
  protected Component visibleComponent;

  // Listeners
  protected MouseListener          mouseGetter;
  protected FocusListener          focusGetter;
  protected ChangeListener         tabChangeListener;
  protected PropertyChangeListener propertyChangeListener;
  
  // UI creation

  public static ComponentUI createUI(JComponent c)
  {
    return new SlafTabbedPaneUI();
  }

  // UI Installation/De-installation
    
  public void installUI(JComponent c)
  {
    c.setLayout(this);
    installDefaults(c);      
    installListeners(c);
    installKeyboardActions(c);
    SlafUtils.setOpaque(c);

    int tp=SlafManager.getInteger("TabbedPane.tabPlacement");
    if(tp!=0) ((JTabbedPane)c).setTabPlacement(tp);
  }

  public void uninstallUI(JComponent c)
  {
    uninstallKeyboardActions(c);
    uninstallListeners(c);
    uninstallDefaults(c);
    c.setLayout(null); 
  }

  protected void installDefaults(JComponent c)
  {
    LookAndFeel.installColorsAndFont
      (c,"TabbedPane.background","TabbedPane.foreground","TabbedPane.font");
    LookAndFeel.installBorder(c,"TabbedPane.border");
    tabHighlight =SlafManager.getColor("TabbedPane.tabHighlight");
    tabShadow    =SlafManager.getColor("TabbedPane.tabShadow");
    tabDarkShadow=SlafManager.getColor("TabbedPane.tabDarkShadow");
    focus        =SlafManager.getColor("TabbedPane.focus");
  }

  protected void uninstallDefaults(JComponent c)
  {
  }

  protected void installListeners(JComponent c)
  {
    if((mouseGetter=createMouseListener(c))!=null)
      c.addMouseListener(mouseGetter);

    if((focusGetter=createFocusListener(c))!=null)
      c.addFocusListener(focusGetter);

    if ((tabChangeListener=createChangeListener(c))!=null)
      ((JTabbedPane)c).addChangeListener(tabChangeListener);

    if((propertyChangeListener=createPropertyChangeListener(c))!=null)
      c.addPropertyChangeListener(propertyChangeListener);
  }

  protected void uninstallListeners(JComponent c) {
    if(mouseGetter!=null)
    {
      c.removeMouseListener(mouseGetter);
      mouseGetter=null;
    }
    if(focusGetter!=null)
    {
      c.removeFocusListener(focusGetter);
      focusGetter=null;
    }
    if(tabChangeListener!=null)
    {
      ((JTabbedPane)c).removeChangeListener(tabChangeListener);
    }        
    if(propertyChangeListener!=null)
    {
      c.removePropertyChangeListener(propertyChangeListener);
    }        
  }

  protected MouseListener createMouseListener(JComponent c)
  {
    return new MouseGetter();
  }
  
  protected FocusListener createFocusListener(JComponent c)
  {
    return new FocusGetter();
  }

  protected ChangeListener createChangeListener(JComponent c) {
    return this;
  }

  protected PropertyChangeListener createPropertyChangeListener(JComponent c)
  {
    return new PropertyGetter();
  }

  protected void installKeyboardActions(JComponent c) {
    final JTabbedPane pane = (JTabbedPane)c;
        c.registerKeyboardAction(
	    new AbstractAction() {
                public void actionPerformed(ActionEvent e) {
	            navigateSelectedTab(pane, pane.getTabPlacement(), EAST);
                }
                public boolean isEnabled() { 
	          return true; 
                }
            },
            KeyStroke.getKeyStroke(KeyEvent.VK_RIGHT,0), 
            JComponent.WHEN_FOCUSED);
        c.registerKeyboardAction(
            new AbstractAction() {
                public void actionPerformed(ActionEvent e) {
                    navigateSelectedTab(pane, pane.getTabPlacement(), WEST);
                }
                public boolean isEnabled() { 
                    return true; 
                }
            }, 
            KeyStroke.getKeyStroke(KeyEvent.VK_LEFT,0), 
            JComponent.WHEN_FOCUSED);
        c.registerKeyboardAction(
            new AbstractAction() {
                public void actionPerformed(ActionEvent e) {
                    navigateSelectedTab(pane, pane.getTabPlacement(), NORTH);
                }

                public boolean isEnabled() { 
                    return true; 
                }
            },
            KeyStroke.getKeyStroke(KeyEvent.VK_UP,0), 
            JComponent.WHEN_FOCUSED);
        c.registerKeyboardAction(
            new AbstractAction() {
                public void actionPerformed(ActionEvent e) {
                    navigateSelectedTab(pane, pane.getTabPlacement(), SOUTH);
                }

                public boolean isEnabled() { 
                    return true; 
                }
            },
            KeyStroke.getKeyStroke(KeyEvent.VK_DOWN,0), 
            JComponent.WHEN_FOCUSED);
    }

    protected void uninstallKeyboardActions(JComponent c) {
        c.resetKeyboardActions();
    }



  // UI Rendering 

  public void paint(Graphics g, JComponent c)
  {
    JTabbedPane pane =(JTabbedPane)c;
    int selectedIndex=pane.getSelectedIndex();
    int tabPlacement =pane.getTabPlacement();
    int tabCount     =pane.getTabCount();

    if(tabCount != rects.length)
      calculateLayoutInfo(pane); 
       
    // Paint content border
    paintContentBorder(g, pane, tabPlacement, selectedIndex);

    Rectangle iconRect = new Rectangle();
    Rectangle textRect = new Rectangle();
    Rectangle clipRect = g.getClipBounds();  
    
    Insets insets = pane.getInsets();

    //g.translate(insets.left, insets.top);

    // Paint tabRuns of tabs from back to front
    for (int i = runCount - 1; i >= 0; i--)
    {
      int start = tabRuns[i];
      int next = tabRuns[(i == runCount - 1)? 0 : i+1];
      int end = (next != 0? next - 1: tabCount - 1);
      for (int j = start; j <= end; j++) {
	if (rects[j].intersects(clipRect)) {
	  paintTab(g, pane, tabPlacement, rects, j, iconRect, textRect);
	}
      }
    }
    
    // Paint selected tab if its in the front run
    // since it may overlap other tabs
    if(selectedIndex >= 0 && getRunForTab(pane, tabCount, selectedIndex) == 0)
    {
      if (rects[selectedIndex].intersects(clipRect))
	paintTab(g,pane,tabPlacement,rects,selectedIndex,iconRect,textRect);
    }
    //g.translate(-insets.left, -insets.top);
  }

  protected void paintTab(Graphics g, JTabbedPane pane, int tabPlacement,
			  Rectangle[] rects, int tabIndex, 
			  Rectangle iconRect, Rectangle textRect)
  {
    Rectangle tabRect = rects[tabIndex];
    int selectedIndex = pane.getSelectedIndex();
    boolean isSelected = (selectedIndex==tabIndex);

    xNudge_=calculateXNudge(pane,tabPlacement,tabIndex,isSelected);
    yNudge_=calculateYNudge(pane,tabPlacement,tabIndex,isSelected);

    //Insets j=getContentBorderInsets(pane,tabPlacement);
    Rectangle old=g.getClipBounds();
    int cx,cy,cw,ch;
    cx=cy=cw=ch=0;

    if(isSelected)
      switch(tabPlacement)
      {
        case LEFT  : cw=2;  break;
        case RIGHT : cx=-2; cw+=2; break;
        case BOTTOM: cy=-2; ch+=2; break;
        case TOP   :
        default    : ch=2;  break;
      }
    else
      switch(tabPlacement)
      {
        case RIGHT : cx=5; break;
        case BOTTOM: cy=5; break;
      }

    g.setClip(new Rectangle
	(tabRect.x+cx,tabRect.y+cy,tabRect.width+cw,tabRect.height+ch));

    paintTabBackground(g, pane, tabPlacement, tabIndex, tabRect.x, tabRect.y, 
		       tabRect.width, tabRect.height, isSelected);
    paintTabBorder(g, pane, tabPlacement, tabIndex, tabRect.x, tabRect.y, 
		   tabRect.width, tabRect.height, isSelected);

    if(isSelected&&(tabIndex==0)&&(tabPlacement==TOP))
    {
      Icon corner=SlafManager.getIcon("TabbedPane.selectedFirstCornerIcon");
      if(corner!=null)
      {
	//g.setColor(Color.red);
	corner.paintIcon
	  (pane,g,tabRect.x+cx,
	   tabRect.y+cy+tabRect.height+ch-corner.getIconHeight());
      }
    }
	
    String title = pane.getTitleAt(tabIndex);
    Font font = pane.getFont();
    FontMetrics metrics = g.getFontMetrics(font);
    Icon icon = getIconForTab(pane, tabIndex);

    layoutLabel(tabPlacement, metrics, title, icon, 
		tabRect, iconRect, textRect, isSelected);

    paintText(g, pane, tabPlacement, font, metrics, 
	      tabIndex, title, textRect, isSelected);

    paintIcon(g, pane, tabPlacement, tabIndex, icon, iconRect, isSelected );
    
    if(SlafManager.getBoolean("TabbedPane.focusPainted",true))
      paintFocusIndicator(g,pane,tabPlacement,rects,tabIndex,
			  iconRect,textRect,isSelected);

    //g.setColor(Color.red);
    //g.drawRect(tabRect.x,tabRect.y,tabRect.width,tabRect.height);

    g.setClip(old);

    //g.setColor(Color.yellow);
    //g.drawRect(tabRect.x-2,tabRect.y-2,tabRect.width+4,tabRect.height+4);
  }

  protected void layoutLabel(int tabPlacement, FontMetrics metrics, 
			     String title, Icon icon, 
			     Rectangle tabRect, Rectangle iconRect, 
			     Rectangle textRect, boolean isSelected )
  {
    textRect.x = textRect.y = iconRect.x = iconRect.y = 0;
    SwingUtilities.layoutCompoundLabel(metrics, title, icon,
				       SwingUtilities.CENTER,
				       SwingUtilities.LEFT, // CENTER,
				       SwingUtilities.CENTER,
				       SwingUtilities.RIGHT,
				       tabRect,
				       iconRect,
				       textRect,
				       iconSpacingWidth);
  }

  protected void paintIcon(Graphics g, JTabbedPane pane, int tabPlacement,
			   int tabIndex, Icon icon, Rectangle iconRect, 
			   boolean isSelected )
  {
    if(icon!=null)
      icon.paintIcon(pane,g,iconRect.x+xNudge_,iconRect.y+yNudge_);
  }

  protected void paintText(Graphics g, JTabbedPane pane, int tabPlacement,
			   Font font, FontMetrics metrics, int tabIndex,
			   String title, Rectangle textRect, 
			   boolean isSelected)
  {
    g.setFont(font);

    if (pane.isEnabledAt(tabIndex))
    {
      Color fg=pane.getForegroundAt(tabIndex);
      if(isSelected&&(fg instanceof ColorUIResource))
	  fg=SlafManager.getColor
	      ("TabbedPaneTab.selectionForeground",
	       SlafManager.getColor
	       ("TabbedPane.selectionTabForeground",
		fg.brighter()));
      g.setColor(fg);
      g.drawString(title,
		   textRect.x+xNudge_,
		   textRect.y+metrics.getAscent()+yNudge_);
    }
    else
    { // tab disabled
      g.setColor(pane.getBackgroundAt(tabIndex).brighter());
      g.drawString(title, 
		   textRect.x+xNudge_ +1,
		   textRect.y+metrics.getAscent()+yNudge_ +1);
      g.setColor(pane.getBackgroundAt(tabIndex).darker());
      g.drawString(title, 
		   textRect.x+xNudge_,
		   textRect.y+metrics.getAscent()+yNudge_);
    }
  }

  protected int calculateXNudge
  (JTabbedPane _pane, int _tabPlacement, int _tabIndex, boolean _isSelected)
  {
      Insets i=getTabBorderInsets(_pane,_isSelected);
      int    r=0;

      switch(_tabPlacement)
      {
        case LEFT:
	  r=i.left+1; // ok
	  break;
	case RIGHT:
	  r=i.left+4;
	  break;
	case BOTTOM:
	  r=i.left+1; //-i.bottom-1;
	  break;
	case TOP:
	default:
	  r=i.left+1; // ok
	  break;
      }

      return r;
  }

  protected int calculateYNudge(JTabbedPane _pane, int _tabPlacement,
				int _tabIndex, boolean _isSelected)
  {
      Insets i=getTabBorderInsets(_pane,_isSelected);
      int    r=0;

      switch(_tabPlacement)
      {
        case LEFT:
	  r=0;
	  break;
	case RIGHT:
	  r=0;
	  break;
	case BOTTOM:
	    r=i.top-1;
	  break;
	case TOP:
	default:
	  r=0;
	  break;
      }
      return r;
  }

  protected void paintFocusIndicator(Graphics g, JTabbedPane pane,
				     int tabPlacement,
				     Rectangle[] rects, int tabIndex, 
				     Rectangle iconRect, Rectangle textRect,
				     boolean isSelected)
  {
    if (SlafUtils.hasFocus(pane)&&isSelected)
    {
      Rectangle fr=rects[tabIndex];
      Insets    in=getTabBorderInsets(pane,isSelected);
      Rectangle vr=new Rectangle
	(fr.x+in.left,fr.y+in.top,
	 fr.width-in.left-in.right,fr.height-in.top-in.bottom);
      Rectangle tr=new Rectangle
	(textRect.x+xNudge_,textRect.y+yNudge_,textRect.width,textRect.height);
      Rectangle ir=new Rectangle
	(iconRect.x+xNudge_,iconRect.y+yNudge_,iconRect.width,iconRect.height);
      SlafUtils.paintFocus(g,pane,vr,tr,ir);
    }

    /*
      Rectangle tabRect=textRect;
      if (SlafUtils.hasFocus(pane)&&isSelected)
      {
      int x, y, w, h;
      g.setColor(focus);
      g.drawRect(tabRect.x,tabRect.y,tabRect.width,tabRect.height);
      }
    */

    /*
    Rectangle tabRect = rects[tabIndex];

    if(SlafUtils.hasFocus(pane)&&isSelected)
    {
      int x, y, w, h;
      g.setColor(focus);

      switch(tabPlacement)
      {
      case LEFT:
	x = tabRect.x     +4; // 3
	y = tabRect.y     +4; // 3
	w = tabRect.width -9; // 5
	h = tabRect.height-8; // 6
	break;
      case RIGHT:
	x = tabRect.x     +4; // 2
	y = tabRect.y     +4; // 3;
	w = tabRect.width -9; // 5;
	h = tabRect.height-8; // 6;
	break;
      case BOTTOM:
	x = tabRect.x     +4; // 3
	y = tabRect.y     +6; // 2
	w = tabRect.width -8; // 6
	h = tabRect.height-9; // 5
	break;
      case TOP:
      default:
	x = tabRect.x     +4;
	y = tabRect.y     +4;
	w = tabRect.width -8;
	h = tabRect.height-7;
      }
      // SlafGraphicsUtils.drawDashedRect(g, x, y, w, h);
      g.drawLine(x,y+h-1,x+w-1,y+h-1);
    }
    */
  }

  private Rectangle getFocusedTabBounds(JTabbedPane _pane)
  {
    if(rects.length!= _pane.getTabCount())
      calculateLayoutInfo(_pane);
    int i=_pane.getSelectedIndex();
    return (i==-1) ? null : rects[i];
  }

  protected void paintTabBorder
    (Graphics g, JTabbedPane pane, int tabPlacement,
     int tabIndex,
     int x, int y, int w, int h, 
     boolean isSelected )
  {
    Border b=getTabBorder(isSelected);
    if(b!=null)
    {
      Insets i=getTabBorderInsets(pane,isSelected);
      //Insets j=getContentBorderInsets(pane,tabPlacement);
      int cx,cy,cw,ch;
      cx=cy=cw=ch=0;
      switch(tabPlacement)
      {
      case LEFT  : cw=i.right +4; break; // ok
      case RIGHT : cx=-i.left; cw=-cx; break; // ok
      case BOTTOM: cy=-i.top; ch=-cy; break;
      case TOP   :
      default    : ch=i.bottom+4; break; // ok
      }
      b.paintBorder(pane,g,x+cx,y+cy,w+cw,h+ch);
    }
  }

  protected void paintTabBackground
    (Graphics _g, JTabbedPane _pane, int _tabPlacement,
     int _tabIndex, int _x, int _y, int _w, int _h, boolean _isSelected)
  {
    // essai
    /*
    Insets i=getTabBorderInsets(_pane,_isSelected);
    _x+=i.left;
    _y+=i.top;
    _w+=i.left+i.right;
    _h+=i.top+i.bottom;
    */

    Rectangle r =new Rectangle(_x,_y,_w,_h);
    Color     bg=_pane.getBackgroundAt(_tabIndex);
    int       t =SlafUtils.ENABLED;

    if(_isSelected)
    {
      if(bg instanceof ColorUIResource)
      {
	bg=SlafManager.getColor
	    ("TabbedPaneTab.selectionBackground",
	     SlafManager.getColor
	     ("TabbedPane.selectionTabBackground",null));
	if(bg==null)
	{
	  t=SlafUtils.SELECTED;
	  bg=SlafManager.getColor
	      ("TabbedPaneTab.background",
	       SlafManager.getColor
	       ("TabbedPane.tabBackground",null));
	}
      }

      switch(_tabPlacement)
      {
        case LEFT  : r.width+=2; break;
        case RIGHT : r.x-=2; r.width+=2; break;
        case BOTTOM: r.y-=2; r.height+=2; break;
        case TOP   :
        default    : r.height+=2; break;
      }
    }
    else
    {
      if(bg instanceof ColorUIResource)
      {
	bg=SlafManager.getColor
	    ("TabbedPaneTab.background",
	     SlafManager.getColor
	     ("TabbedPane.tabBackground",null));
      }
    }

    if(bg==null)
	bg=SlafManager.getColor("TabbedPane.tabBackground",null);
    if(bg==null)
	bg=_pane.getBackground();
    //SlafManager.getColor("TabbedPane.background");
 
    //SlafUtils.trace("RECT-TAB="+r);
    SlafUtils.paintBackground(_g,r,_pane,bg,t);//,"TabbedPaneTab",true);
  }

  protected void paintContentBorder
    (Graphics g, JTabbedPane pane, int tabPlacement, int selectedIndex)
  {
    Rectangle bounds = pane.getBounds();
    Insets insets = pane.getInsets();
    int height = bounds.height - (insets.top+insets.bottom);
    int width = bounds.width - (insets.left+insets.right);

    int x = 0;
    int y = 0;
    int w = width;
    int h = height;

    switch(tabPlacement)
    {
    case LEFT:
      x = totalTabWidth(pane, tabPlacement, runCount);
      w -= x;
      break;
    case RIGHT:
      w -= totalTabWidth(pane, tabPlacement, runCount);
      break;            
    case BOTTOM: 
      h -= totalTabHeight(pane, tabPlacement, runCount);
      break;
    case TOP:
    default:
      y = totalTabHeight(pane, tabPlacement, runCount);
      h -= y;
    }                
    
//   Color bg=pane.getBackground();
//   g.setColor(bg.brighter());
//   paintContentBorderTopEdge(g, tabPlacement, selectedIndex, x, y, w, h);
//   paintContentBorderLeftEdge(g, tabPlacement, selectedIndex, x, y, w, h); 
//   g.setColor(bg.darker());
//   paintContentBorderBottomEdge(g, tabPlacement, selectedIndex, x, y, w, h);
//   paintContentBorderRightEdge(g, tabPlacement, selectedIndex, x, y, w, h); 

    Border b=getContentBorder();
    if(b!=null) b.paintBorder(pane,g,x,y,w,h);
  }
	 
    protected void paintContentBorderTopEdge(Graphics g, int tabPlacement,
                                         int selectedIndex, 
                                         int x, int y, int w, int h) {

      // g.setColor(tabHighlight);

        if (tabPlacement != TOP || selectedIndex < 0 || 
            (rects[selectedIndex].y+rects[selectedIndex].height+1 < y)) {
            g.drawLine(x, y, x+w-2, y);
        } else {
            Rectangle selRect = rects[selectedIndex];

            g.drawLine(x, y, selRect.x - 1, y);
            if (selRect.x+selRect.width < x+w - 2) {
                g.drawLine(selRect.x+selRect.width, y, 
                           x+w-2, y);
            } else {
	      // g.setColor(tabShadow); 
                g.drawLine(x+w-2, y, x+w-2, y);
            }
        }
    }

    protected void paintContentBorderLeftEdge(Graphics g, int tabPlacement,
                                               int selectedIndex,
                                               int x, int y, int w, int h) { 
      // g.setColor(tabHighlight); 
        if (tabPlacement != LEFT || selectedIndex < 0 ||
           (rects[selectedIndex].x+rects[selectedIndex].width+1< x)) {
            g.drawLine(x, y, x, y+h-2);
        } else {
            Rectangle selRect = rects[selectedIndex];

            g.drawLine(x, y, x, selRect.y - 1);
            if (selRect.y+selRect.height < y+h - 2) {
                g.drawLine(x, selRect.y+selRect.height, 
                           x, y+h-2);
            } 
        }
    }

    protected void paintContentBorderBottomEdge(Graphics g, int tabPlacement,
                                               int selectedIndex,
                                               int x, int y, int w, int h) { 
      // g.setColor(tabShadow);
        if (tabPlacement != BOTTOM || selectedIndex < 0 ||
            (rects[selectedIndex].y - 1 > h)) {
            g.drawLine(x+1, y+h-2, x+w-2, y+h-2);
            // g.setColor(tabDarkShadow);
            // g.drawLine(x, y+h-1, x+w-1, y+h-1);
        } else {
            Rectangle selRect = rects[selectedIndex];

            g.drawLine(x+1, y+h-2, selRect.x - 1, y+h-2);
            g.setColor(tabDarkShadow);
            g.drawLine(x, y+h-1, selRect.x - 1, y+h-1);
            if (selRect.x+selRect.width < x+w - 2) {
	      // g.setColor(tabShadow);
                g.drawLine(selRect.x+selRect.width, y+h-2, x+w-2, y+h-2);
                // g.setColor(tabDarkShadow);
                // g.drawLine(selRect.x+selRect.width, y+h-1, x+w-1, y+h-1);
            } 
        }

    }

    protected void paintContentBorderRightEdge(Graphics g, int tabPlacement,
                                               int selectedIndex,
                                               int x, int y, int w, int h) { 

      // g.setColor(tabShadow);
        if (tabPlacement != RIGHT || selectedIndex < 0 ||
            rects[selectedIndex].x - 1 > w) {
            g.drawLine(x+w-2, y+1, x+w-2, y+h-3);
            // g.setColor(tabDarkShadow);
            // g.drawLine(x+w-1, y, x+w-1, y+h-1);
        } else {
            Rectangle selRect = rects[selectedIndex];

            g.drawLine(x+w-2, y+1, x+w-2, selRect.y - 1);
            // g.setColor(tabDarkShadow);
            g.drawLine(x+w-1, y, x+w-1, selRect.y - 1);

            if (selRect.y+selRect.height < y+h - 2) {
	      // g.setColor(tabShadow);
                g.drawLine(x+w-2, selRect.y+selRect.height, 
                           x+w-2, y+h-2);
                // g.setColor(tabDarkShadow);
                // g.drawLine(x+w-1, selRect.y+selRect.height, x+w-1, y+h-2);
            } 
        }
    }
    

  // LayoutManager methods

  public Dimension getPreferredSize(JComponent c) {
    JTabbedPane pane = (JTabbedPane)c; 
    return calculateSize(pane, pane.getTabPlacement(), false);
  }

  public Dimension getMinimumSize(JComponent c) {
    JTabbedPane pane = (JTabbedPane)c; 
    return calculateSize(pane, pane.getTabPlacement(), true);
  }


  protected Dimension calculateSize(JTabbedPane pane, int tabPlacement,
				    boolean minimum)
  { 
    Insets insets = pane.getInsets();
    Insets borderInsets = getContentBorderInsets(pane, tabPlacement);

    Dimension zeroSize = new Dimension(0,0);
    int height = borderInsets.top+borderInsets.bottom; 
    int width = borderInsets.left+borderInsets.right;
    int cWidth = 0;
    int cHeight = 0;

    for(int i = 0; i < pane.getTabCount(); i++)
    {
      Component component = pane.getComponentAt(i);
      Dimension size = zeroSize;
      if (component instanceof JComponent)
      { 
	if (component.getFont() == null) { 
	  component.setFont(pane.getFont()); 
	}
	size = minimum? component.getMinimumSize() : 
	  component.getPreferredSize();
	
	if (size != null) {
	  cHeight = Math.max(size.height, cHeight);
	  cWidth = Math.max(size.width, cWidth);
	}
      }
    }

    width += cWidth;
    height += cHeight;
    int tabExtent = 0;
    
    switch(tabPlacement)
    {
    case LEFT:
    case RIGHT:
      tabExtent = preferredTotalTabWidth(pane, tabPlacement,
					 getMetrics(pane), height);
      width += tabExtent;

      // @GDX ensure all tabs visible
      FontMetrics metrics=pane.getFontMetrics(pane.getFont());
      int is=SlafManager.getInteger("TabbedPane.interTabSpace",5);
      int ht=pane.getTabCount()*(is+maxTabHeight(pane,metrics));
      if(is>0) { if(ht>is) ht-=is; }
      else     { ht+=-is; }
      // SlafUtils.trace("#### TP.TC: "+pane.getTabCount());
      // SlafUtils.trace("#### TP.PH: "+height+" - "+ht);
      height=Math.max(height,ht);

      break;
    case TOP:
    case BOTTOM:
    default:
      tabExtent = preferredTotalTabHeight(pane, tabPlacement,
					  getMetrics(pane), width);
      height+=tabExtent;
      break;
    }
    
    return new Dimension(width+insets.left+insets.right, 
			 height+insets.bottom+insets.top);
  }
    
  public Dimension getMaximumSize(JComponent c)
  {
    return new Dimension(Short.MAX_VALUE,Short.MAX_VALUE);
  }

  public void addLayoutComponent(String name, Component comp) { }
    
  public void removeLayoutComponent(Component comp) { }
    
  public Dimension preferredLayoutSize(Container parent)
  {
    return new Dimension(parent.getBounds().width, 
			 parent.getBounds().height);
    // return getPreferredSize((JTabbedPane)parent);
  }

  public Dimension minimumLayoutSize(Container parent)
  {
    return new Dimension(0,0);
    // return getMinimumSize((JTabbedPane)parent);
  }

  public void layoutContainer(Container parent)
  {
    JTabbedPane pane = (JTabbedPane)parent;
    layoutTabbedPane(pane, pane.getTabPlacement());
  }

  protected void layoutTabbedPane(JTabbedPane pane, int tabPlacement)
  {
    Insets insets = pane.getInsets();
    Insets borderInsets = getContentBorderInsets(pane, tabPlacement);
    int selectedIndex = pane.getSelectedIndex();
    Window window = null;
    Component focusedComponent;

    if(selectedIndex<0)
    {
      if(visibleComponent!=null)
      {
	// The last tab was removed, so remove the component
	setVisibleComponent(pane,null);
      }
    }
    else
    {
      int cx, cy, cw, ch;
      int totalTabWidth = 0;
      int totalTabHeight = 0;

      Component selectedComponent = pane.getComponentAt(selectedIndex);
      boolean shouldChangeFocus = false;

      if(selectedComponent!=null)
      {
	if(selectedComponent.getParent()==null)
	{
	  if(visibleComponent!=null)
	  {
	    if(SwingUtilities.findFocusOwner(visibleComponent)!=null)
	    {
	      shouldChangeFocus = true;
	    }
	  }
	}
	calculateLayoutInfo(pane); 
	Rectangle bounds = pane.getBounds();

	setVisibleComponent(pane, selectedComponent); 

	cx=cy=cw=ch=0;

	switch(tabPlacement)
	{
	case LEFT:
	  totalTabWidth = totalTabWidth(pane, tabPlacement, runCount);
	  cx = insets.left+totalTabWidth+borderInsets.left;
	  cy = insets.top+borderInsets.top;
	  break;
	case RIGHT:
	  totalTabWidth = totalTabWidth(pane, tabPlacement, runCount);
	  cx = insets.left+borderInsets.left;
	  cy = insets.top+borderInsets.top;
	  break;
	case BOTTOM:
	  totalTabHeight = totalTabHeight(pane, tabPlacement, runCount);
	  cx = insets.left+borderInsets.left;
	  cy = insets.top+borderInsets.top;
	  break;                   
	case TOP:
	default:
	  totalTabHeight = totalTabHeight(pane, tabPlacement, runCount);
	  cx = insets.left+borderInsets.left;
	  cy = insets.top+totalTabHeight+borderInsets.top
	    -Math.max(0,SlafManager.getInteger("TabbedPane.padSelected",0));
	  ch = 0;//-borderInsets.top;          // ???
	}
	
	cw+=bounds.width - totalTabWidth -
	   insets.left - insets.right -
	   borderInsets.left - borderInsets.right;
	ch+=bounds.height - totalTabHeight -
	   insets.top - insets.bottom -
	   borderInsets.top - borderInsets.bottom;
	
	selectedComponent.setBounds(cx, cy, cw, ch);
	selectedComponent.validate();
	if (shouldChangeFocus)
	{
	  if (selectedComponent.isFocusTraversable())
	  {
	    selectedComponent.requestFocus();
	    shouldChangeFocus = false;
	  }
	  else
	  if (selectedComponent instanceof JComponent)
	  {
	    if (((JComponent)selectedComponent).requestDefaultFocus())
	      shouldChangeFocus = false;
	  }
	  
	  if (shouldChangeFocus)
	  {
	    /* Need to change the focus but we have no candidate.
	     * Let's set the focus on the tabbed pane itself
	     */
	    pane.requestFocus();
	  }
	}
      }
    }
  }


// TabbedPaneUI methods

    public void stateChanged(ChangeEvent e) {
        JTabbedPane pane = (JTabbedPane)e.getSource();
        pane.doLayout();
        pane.repaint();
    }

    protected Component getVisibleComponent(JTabbedPane pane) {
        int count = pane.getTabCount();

        while (count-- > 0) {
            Component component = pane.getComponentAt(count);

            if (component != null && component.getParent() == pane) {
                return component;
            }
        }
        return null;
    }

  protected void setVisibleComponent(JTabbedPane pane, Component component)
  {
    if(visibleComponent==component) return;

    if(visibleComponent!=null)
      visibleComponent.setVisible(false);

    if(component!=null)
    {
      Color bg=SlafManager.getColor("TabbedPane.contentBackground");
      if(bg!=null)
      {
	if(component.getBackground() instanceof ColorUIResource)
	  component.setBackground(bg);

	if(component instanceof Container)
        {
	  Vector v=SlafUtils.getAllSubComponents((Container)component);
	  int    l=v.size();
	  for(int i=1;i<l;i++)
	  {
	    Component c=(Component)v.elementAt(i);
	    if(  (c instanceof JPanel)
	       &&(c.getBackground() instanceof ColorUIResource))
	      c.setBackground(bg);
	  }
	}
      }

      component.setVisible(true);
    }
    visibleComponent = component;
  }

  protected void calculateLayoutInfo(JTabbedPane pane)
  {
    calculateLayoutInfo(pane, pane.getTabPlacement());
  }

  protected void calculateLayoutInfo(JTabbedPane pane, int tabPlacement)
  {
    Font font = pane.getFont();
    int tabCount = pane.getTabCount();
    int selectedIndex = pane.getSelectedIndex();
    
    assureRectsCreated(tabCount);

    arrangeTabs(pane, tabPlacement, tabCount, font); 
            
    padSelectedTab(pane, tabPlacement, selectedIndex);
  }

  protected void assureRectsCreated(int tabCount)
  {
    int rectArrayLen = rects.length; 
    if (tabCount != rectArrayLen )
    {
      Rectangle[] tempRectArray = new Rectangle[tabCount];
      System.arraycopy(rects, 0, tempRectArray, 0, 
		       Math.min(rectArrayLen, tabCount));
      rects = tempRectArray;
      for (int rectIndex = rectArrayLen; rectIndex < tabCount; rectIndex++) {
	rects[rectIndex] = new Rectangle();
      }
    } 
  }

  protected void expandTabRunsArray()
  {
    int rectLen = tabRuns.length;
    int[] newArray = new int[rectLen+10];
    System.arraycopy(tabRuns, 0, newArray, 0, runCount);
    tabRuns = newArray;
  }

  protected void arrangeTabs
  (JTabbedPane _pane, int _tabPlacement, int _tabCount, Font _font)
  {
    // FontMetrics metrics = Toolkit.getDefaultToolkit().getFontMetrics(font);
    FontMetrics metrics=_pane.getFontMetrics(_font);
    Dimension size     =_pane.getSize();
    Insets insets      =_pane.getInsets(); 
    Insets tabInsets   =getTabAreaInsets(_pane,_tabPlacement);
    int fontHeight     =metrics.getHeight();
    int selectedIndex  =_pane.getSelectedIndex();
    int overlay;
    int i, j;
    int x, y;
    int returnAt;
    boolean verticalTabRuns=(_tabPlacement==LEFT||_tabPlacement==RIGHT);

    switch(_tabPlacement)
    {
    case LEFT:
      maxTabWidth = maxTabWidth(_pane, metrics);
      x = insets.left+tabInsets.left;
      y = insets.top+tabInsets.top;
      returnAt = size.height - (insets.bottom+tabInsets.bottom);
      break;
    case RIGHT:
      maxTabWidth = maxTabWidth(_pane, metrics);
      x = size.width - insets.right - tabInsets.right - maxTabWidth;
      y = insets.top+tabInsets.top;
      returnAt = size.height - (insets.bottom+tabInsets.bottom);
      break;
    case BOTTOM:
      maxTabHeight = maxTabHeight(_pane, metrics);
      x = insets.left+tabInsets.left;
      y = size.height - insets.bottom - tabInsets.bottom - maxTabHeight;
      returnAt = size.width - (insets.right+tabInsets.right);
      break;
    case TOP:
    default:
      maxTabHeight = maxTabHeight(_pane, metrics);
      x=insets.left+tabInsets.left;
      y=insets.top+tabInsets.top
	+Math.max(0,SlafManager.getInteger("TabbedPane.padSelected",0));
      returnAt=size.width-(insets.right+tabInsets.right);
    }
    
    overlay=getTabOverlay(_pane, _tabPlacement);
    
    runCount = 0;
    selectedRun = -1;
    
    Rectangle rect;
    // Run through tabs and partition them into runs
    for(i=0;i<_tabCount;i++)
    {
      rect = rects[i];

      if(!verticalTabRuns)
      {
	// Tabs on TOP or BOTTOM....
	if(i>0)
	{
	  rect.x=rects[i-1].x+rects[i-1].width;
	  //rect.x+=5;
	  rect.x+=SlafManager.getInteger("TabbedPane.interTabSpace",5);
	}
	else
	{
	  tabRuns[0] = 0;
	  runCount = 1;
	  maxTabWidth = 0;
	  rect.x = x;
	}
	rect.width = tabWidth(_pane, i, metrics);
	maxTabWidth = Math.max(maxTabWidth, rect.width);
	
	// Never move a TAB down a run if it is in the first column. 
	// Even if there isn't enough room, moving it to a fresh 
	// line won't help.
	if((rect.x!=2+insets.left)&&(rect.x+rect.width>returnAt))
	{
	  if(runCount>tabRuns.length-1)
	    expandTabRunsArray();

	  tabRuns[runCount] = i;
	  runCount++;
	  rect.x = x;
	}
	// Initialize y position in case there's just one run
	rect.y = y;
	rect.height = maxTabHeight /* - 2*/
	  -Math.max(0,SlafManager.getInteger("TabbedPane.padSelected",0));
      } else {
	// Tabs on LEFT or RIGHT...
	if(i>0)
	{
	  rect.y = rects[i-1].y+rects[i-1].height;
	  // rect.y+=5;
	  rect.y+=SlafManager.getInteger("TabbedPane.interTabSpace",5);

	}
	else
	{
	  tabRuns[0] = 0;
	  runCount = 1;
	  maxTabHeight = 0;
	  rect.y = y;
	}
	rect.height = tabHeight(_pane, i, fontHeight);
	maxTabHeight = Math.max(maxTabHeight, rect.height);
	
	/***
	// Never move a TAB over a run if it is in the first run. 
	// Even if there isn't enough room, moving it to a fresh 
	// column won't help.
	if (rect.y != 2+insets.top && rect.y+rect.height > returnAt) {
	  if (runCount > tabRuns.length - 1) {
	    expandTabRunsArray();
	  }
	  tabRuns[runCount] = i;
	  runCount++;
	  rect.y = y;
	}
	***/

	// Initialize x position in case there's just one column
	rect.x = x;
	rect.width = maxTabWidth/* - 2*/;
      }            
      if (i == selectedIndex) {
	selectedRun = runCount - 1;
      }
    }
    
    
    if (runCount > 1) {
      // Re-distribute tabs in case last run has leftover space
      normalizeTabRuns(_pane, _tabPlacement, _tabCount, verticalTabRuns? y : x, returnAt);
      
      selectedRun = getRunForTab(_pane, _tabCount, selectedIndex);
      
      // Rotate run array so that selected run is first
      rotateTabRuns(_pane, _tabPlacement, selectedRun);
    }
    
    // Step through runs from back to front to calculate
    // tab y locations and to pad runs appropriately
    for (i = runCount - 1; i >= 0; i--) {
      int start = tabRuns[i];
      int next = tabRuns[i == (runCount - 1)? 0 : i+1];
      int end = (next != 0? next - 1 : _tabCount - 1);
      if (!verticalTabRuns) {
	for (j = start; j <= end; j++) {
	  rect = rects[j];
	  rect.y = y;
	  rect.x += getRunIndent(_pane, _tabPlacement, i);
	}
	if (shouldPadRun(_pane, _tabPlacement, i)) {
	  padRun(_pane, _tabPlacement, start, end, returnAt);
	}
	if (_tabPlacement == BOTTOM) {
	  y -= (maxTabHeight - overlay);
	} else {
	  y += (maxTabHeight - overlay);
	}
      } else {
	for (j = start; j <= end; j++) {
	  rect = rects[j];
	  rect.x = x;
	  rect.y += getRunIndent(_pane, _tabPlacement, i);
	}
	if (shouldPadRun(_pane, _tabPlacement, i)) {
	  padRun(_pane, _tabPlacement, start, end, returnAt);
	}
	if (_tabPlacement == RIGHT) {
                    x -= (maxTabWidth - overlay);
	} else {
	  x += (maxTabWidth - overlay);
	}
      }      
    }
  }
  
  protected int getTabOverlay(JTabbedPane pane, int tabPlacement)
  {
    return 0;
    //SlafManager.getInteger("TabbedPane.runTabOverlay",0);
  }
  
  protected int getRunForTab(JTabbedPane pane, int tabCount, int tabIndex)
  {
    for (int i = 0; i < runCount; i++) {
      int first = tabRuns[i];
      int last = lastIndexInRun(i, tabCount);
      if (tabIndex >= first && tabIndex <= last) {
	return i;
      }
    }
    return 0;
  }

  /* 
   * Rotates the run-index array so that the selected run is run[0]
   */
  protected void rotateTabRuns(JTabbedPane pane, int tabPlacement,
			       int selectedRun)
  {
    for (int i = 0; i < selectedRun; i++) {
      int save = tabRuns[0];
      for (int j = 1; j < runCount; j++) { 
	tabRuns[j - 1] = tabRuns[j];
      }
      tabRuns[runCount-1] = save;
    }        
  }

  protected int getRunIndent(JTabbedPane pane, int tabPlacement, int run)
  {
    return SlafManager.getInteger("TabbedPane.runIndent",0);
  }
  
  protected void normalizeTabRuns
      (JTabbedPane pane, int tabPlacement, int tabCount, 
       int start, int max)
  {
      //return;

      boolean verticalTabRuns=((tabPlacement==LEFT)||(tabPlacement==RIGHT));
      int run = runCount - 1;
      boolean keepAdjusting = true;
      double weight = 1.25;

        while (keepAdjusting) {
            int last = lastIndexInRun(run, tabCount);
            int prevLast = lastIndexInRun(run-1, tabCount);
            int end;
            int prevLastLen;

            if (!verticalTabRuns) {
                end = rects[last].x+rects[last].width;
                prevLastLen = (int)(maxTabWidth*weight);
            } else {
                end = rects[last].y+rects[last].height;
                prevLastLen = (int)(maxTabHeight*weight*2);
            }
 
            if (max - end > prevLastLen) {

                tabRuns[run] = prevLast;
                if (!verticalTabRuns) {
                    rects[prevLast].x = start;
                } else {
                    rects[prevLast].y = start;
                }
                for (int i = prevLast+1; i <= last; i++) {
                    if (!verticalTabRuns) {
                        rects[i].x = rects[i-1].x+rects[i-1].width;
                    } else {
                        rects[i].y = rects[i-1].y+rects[i-1].height;
                    }
                } 
 
            } else if (run == runCount - 1) {
                keepAdjusting = false;
            }
            if (run - 1 > 0) {
                run -= 1;
            } else {
                run = runCount - 1;
                weight += .25;
            }
        }                       
    }

    protected int lastIndexInRun(int run, int tabCount) {
        if (runCount == 1) {
            return tabCount - 1;
        }
        int nextRun = (run == runCount - 1? 0 : run+1);
        if (tabRuns[nextRun] == 0) {
            return tabCount - 1;
        }
        return tabRuns[nextRun]-1;
    }

    protected boolean shouldPadRun(JTabbedPane pane, int tabPlacement, int run) {
        return runCount > 1;
    }

    protected void padRun(JTabbedPane pane, int tabPlacement, 
                          int start, int end, int max) {
        Rectangle lastRect = rects[end];
        if (tabPlacement == TOP || tabPlacement == BOTTOM) {
            int runWidth = (lastRect.x+lastRect.width) - rects[start].x;
            int deltaWidth = max - (lastRect.x+lastRect.width);
            float factor=(float)deltaWidth/(float)runWidth;

            for (int j = start; j <= end; j++) {
                Rectangle pastRect = rects[j];
                if (j > start) {
                    pastRect.x = rects[j-1].x+rects[j-1].width;
                }
                pastRect.width += Math.round((float)pastRect.width * factor);
            }
            lastRect.width = max - lastRect.x;
        } else {
            int runHeight = (lastRect.y+lastRect.height) - rects[start].y;
            int deltaHeight = max - (lastRect.y+lastRect.height);
            float factor = (float)deltaHeight / (float)runHeight;

            for (int j = start; j <= end; j++) {
                Rectangle pastRect = rects[j];
                if (j > start) {
                    pastRect.y = rects[j-1].y+rects[j-1].height;
                }
                pastRect.height += Math.round((float)pastRect.height * factor);
            }
            lastRect.height = max - lastRect.y;
        }
    } 

  protected void padSelectedTab(JTabbedPane pane,
				int tabPlacement,int selectedIndex)
  {
    if(selectedIndex>=0)
    {
      int i=SlafManager.getInteger("TabbedPane.padSelected");
      if(i!=0)
      {
        Rectangle selRect = rects[selectedIndex];

	switch(tabPlacement)
	{
	case LEFT:
	    selRect.x     -=i;
	    selRect.width +=i;
	    break;
	case RIGHT:
	    selRect.width +=i;
	    break;
	case BOTTOM:
	    selRect.height+=i;
	    break;
	case TOP:
	default:
	    selRect.y     -=i;
	    selRect.height+=i;
	}
      }
    }
  } 

  protected int tabHeight(JTabbedPane pane, int tabIndex, int fontHeight)
  {
    int height = fontHeight;
    Icon icon = getIconForTab(pane, tabIndex);
    
    if (icon != null) {
      height = Math.max(height, icon.getIconHeight());
    }
    height += 2*spacingHeight+2;

    Insets i=getTabBorderInsets(pane,(tabIndex==pane.getSelectedIndex()));
    height+=i.top+i.bottom;   

    return height;
  } 

  protected int maxTabHeight(JTabbedPane pane)
  { 
    return maxTabHeight(pane, getMetrics(pane));
  }
  
  protected int maxTabHeight(JTabbedPane pane, FontMetrics metrics)
  {
    int tabCount = pane.getTabCount();
    int result = 0; 
    int fontHeight = metrics.getHeight();
    for(int i = 0; i < tabCount; i++) {
      result = Math.max(tabHeight(pane, i, fontHeight), result);
    }
    return result; 
  }

  protected int tabWidth(JTabbedPane pane, int tabIndex, FontMetrics metrics)
  {
        String title = pane.getTitleAt(tabIndex);
        Icon icon = getIconForTab(pane, tabIndex);
        int width = 2*spacingWidth+3;

        if (icon != null) {
            width += icon.getIconWidth()+iconSpacingWidth;
        }
        width += metrics.stringWidth(title);

	Insets i=getTabBorderInsets(pane,(tabIndex==pane.getSelectedIndex()));
	width+=i.left+i.right;

        return width;
    }

    protected int maxTabWidth(JTabbedPane pane) {
        return maxTabWidth(pane, getMetrics(pane));
    }
    
    protected int maxTabWidth(JTabbedPane pane, FontMetrics metrics) {
        int tabCount = pane.getTabCount();
        int result = 0; 
        for(int i = 0; i < tabCount; i++) {
            result = Math.max(tabWidth(pane, i, metrics), result);
        }
        return result; 
    }

    protected Icon getIconForTab(JTabbedPane pane, int tabIndex) {
        return (!pane.isEnabledAt(tabIndex) && 
                          pane.getDisabledIconAt(tabIndex) != null)?
                          pane.getDisabledIconAt(tabIndex) : pane.getIconAt(tabIndex);
    }

  private Border contentBorder_=
      SlafManager.getBorder("TabbedPane.contentBorder");

  private Border tabBorder_=
      SlafManager.getBorder("TabbedPane.tabBorder");

  private Border selectedTabBorder_=
      SlafManager.getBorder("TabbedPane.selectedTabBorder",
			    SlafManager.getBorder("TabbedPane.tabBorder"));

  protected Border getContentBorder()
  {
    return contentBorder_;
  }

  protected Border getTabBorder(boolean _selected)
  {
    return _selected ? selectedTabBorder_ : tabBorder_;
  }

  protected Insets getContentBorderInsets(JTabbedPane pane, int tabPlacement)
  {
    Border b=getContentBorder();
    Insets r=null;

    if(b!=null) r=b.getBorderInsets(pane);
    else        r=contentBorderInsets;

    return r;
  }

  protected Insets getTabBorderInsets(JTabbedPane pane, boolean _selected)
  {
    Border b=getTabBorder(_selected);
    Insets r=null;

    if(b!=null) r=b.getBorderInsets(pane);
    else        r=tabBorderInsets;

    return r;
  }

  protected Insets getTabAreaInsets(JTabbedPane pane, int tabPlacement)
  {
    Insets insets;

    switch(tabPlacement)
    {
    case LEFT:
	insets = tabsOnLeftTabInsets;
	break;
    case BOTTOM:
	insets = tabsOnBottomTabInsets;
	break;
    case RIGHT:
	insets = tabsOnRightTabInsets;
	break;
    case TOP:
    default:
	insets = tabsOnTopTabInsets;
    }

    return insets;
  }
    
  protected FontMetrics getMetrics(JTabbedPane pane)
  {
    Font        font   =pane.getFont();
    FontMetrics metrics=pane.getFontMetrics(font);
    // return Toolkit.getDefaultToolkit().getFontMetrics(font);
    return metrics;
  }
    
  public Rectangle getTabBounds(JTabbedPane pane, int i)
  {  
    calculateLayoutInfo(pane); 
    return rects[i];
  }

  public int getTabRunCount(JTabbedPane pane)
  {
    return runCount;
  }

  protected int totalTabHeight(JTabbedPane pane, int tabPlacement,
			       int rowCount)
  {
    Insets tabInsets = getTabAreaInsets(pane, tabPlacement);
    int overlay = getTabOverlay(pane, tabPlacement);
    return rowCount * (maxTabHeight-overlay)+-5+//overlay+
      tabInsets.top+tabInsets.bottom;
  }

  protected int totalTabWidth(JTabbedPane pane, int tabPlacement,
			      int columnCount)
  { 
    Insets tabInsets = getTabAreaInsets(pane, tabPlacement);
    int overlay = getTabOverlay(pane, tabPlacement);
    return columnCount * (maxTabWidth-overlay)+-5 +
      tabInsets.left+tabInsets.right;
  }

  protected int preferredTotalTabHeight(JTabbedPane pane, int tabPlacement,
					FontMetrics metrics, int width)
  {
    int tabCount = pane.getTabCount();
    int total = 0;

    if(tabCount>0)
    {
      int rows=1;
      int x   =0;
      
      maxTabHeight=maxTabHeight(pane,metrics);
      
      for(int i=0; i<tabCount; i++)
      {
	int tabWidth=tabWidth(pane,i,metrics);
	// multi-rows
	if((x!=0)&&(x+tabWidth>width))
	{ 
	  rows++;
	  x=0;
	}
	else x+=tabWidth;
      }

      total = totalTabHeight(pane, tabPlacement, rows);
    }
    return total;
  }

  protected int preferredTotalTabWidth(JTabbedPane pane, int tabPlacement,
				       FontMetrics metrics, int height)
  {
    int tabCount = pane.getTabCount();
    int total = 0;

    if (tabCount > 0)
    {
      int columns = 1;
      int y = 0;
      int fontHeight = metrics.getHeight();

      maxTabWidth = maxTabWidth(pane, metrics);
      
      for (int i = 0; i < tabCount; i++) {
	int tabHeight = tabHeight(pane, i, fontHeight);
	
	/*
	  if (y != 0 && y+tabHeight > height) { 
	  columns++;
	  y = 0;
	  }
	*/
      }                   
      total = totalTabWidth(pane, tabPlacement, columns);
    }
    return total;
  }

  // Tab Navigation methods

  protected void navigateSelectedTab(JTabbedPane pane, int tabPlacement,
				     int direction)
  {
    int current = pane.getSelectedIndex();
    int tabCount = pane.getTabCount();
    int offset;

    switch(tabPlacement) {
    case LEFT:
    case RIGHT:
      switch(direction) {
      case NORTH:
	selectPrevTab(pane, current);
	break;
      case SOUTH:
	selectNextTab(pane, current);
	break;
      case WEST:
	offset = getTabRunOffset(pane, tabPlacement, tabCount, current, false);
	selectAdjacentRunTab(pane, tabPlacement, current, offset);
	break;
      case EAST:
	offset = getTabRunOffset(pane, tabPlacement, tabCount, current, true);
	selectAdjacentRunTab(pane, tabPlacement, current, offset);
	break;
      default:
      }
      break;
    case BOTTOM:
    case TOP:
    default:
      switch(direction) {
      case NORTH:
	offset = getTabRunOffset(pane, tabPlacement, tabCount, current, false);
	selectAdjacentRunTab(pane, tabPlacement, current, offset);
	break;
      case SOUTH:
	offset = getTabRunOffset(pane, tabPlacement, tabCount, current, true);
	selectAdjacentRunTab(pane, tabPlacement, current, offset);
	break;
      case EAST:
	selectNextTab(pane, current);
	break;
      case WEST:
	selectPrevTab(pane, current);
	break;
      default:
      }
    }
  }

    public int getPrevTabIndex(JTabbedPane pane, int base) {
        int tabIndex = base - 1;
        if (tabIndex < 0) {
            tabIndex = pane.getTabCount() - 1;
        }
        return tabIndex;            
    }

    public int getNextTabIndex(JTabbedPane pane, int base) {
        return (base+1)%pane.getTabCount();
    }

    public void selectNextTab(JTabbedPane pane, int current) {
        int tabIndex = getNextTabIndex(pane, current);
        
        while (!pane.isEnabledAt(tabIndex) && tabIndex != current) {
            tabIndex = getNextTabIndex(pane, tabIndex);
        }
        pane.setSelectedIndex(tabIndex);
    }

    public void selectPrevTab(JTabbedPane pane, int current) {
        int tabIndex = getPrevTabIndex(pane, current);
        
        while (!pane.isEnabledAt(tabIndex) && tabIndex != current) {
            tabIndex = getPrevTabIndex(pane, tabIndex);
        }
        pane.setSelectedIndex(tabIndex);
    }

    protected void selectAdjacentRunTab(JTabbedPane pane, int tabPlacement, 
                                        int tabIndex, int offset) {
        if ( runCount < 2 ) {
            return; 
        }
        int newIndex;
        Rectangle r = rects[tabIndex]; 
        switch(tabPlacement) {
          case LEFT:
          case RIGHT:
              newIndex = tabForCoordinate(pane, r.x+r.width/2+offset,
                                          r.y+r.height/2);
              break;
          case BOTTOM:  
          case TOP:
          default:
              newIndex = tabForCoordinate(pane, r.x+r.width/2, 
                                        r.y+r.height/2+offset);
        }
        if (newIndex != -1) {
            while (!pane.isEnabledAt(newIndex) && newIndex != tabIndex) {
                newIndex = getNextTabIndex(pane, newIndex);
            }        
            pane.setSelectedIndex(newIndex);
        }
    }

    protected int getTabRunOffset(JTabbedPane pane, int tabPlacement, int tabCount, 
                                  int tabIndex, boolean forward) {
        int run = getRunForTab(pane, tabCount, tabIndex);
        int offset;
        switch(tabPlacement) {
          case LEFT:
              if (run == 0) {
                  offset = (forward? 
                            -(totalTabWidth(pane, tabPlacement, runCount)-maxTabWidth(pane)) :
                            -(maxTabWidth(pane)));
              } else if (run == runCount - 1) {
                  offset = (forward?
                            maxTabWidth(pane) :
                            totalTabWidth(pane, tabPlacement, runCount)-maxTabWidth(pane));
              } else {
                  offset = (forward? maxTabWidth(pane) : -(maxTabWidth(pane)));
              }
              break;
          case RIGHT:
              if (run == 0) {
                  offset = (forward? 
                            maxTabWidth(pane) :
                            totalTabWidth(pane, tabPlacement, runCount)-maxTabWidth(pane));
              } else if (run == runCount - 1) {
                  offset = (forward?
                            -(totalTabWidth(pane, tabPlacement, runCount)-maxTabWidth(pane)) :
                            -(maxTabWidth(pane)));
              } else {
                  offset = (forward? maxTabWidth(pane) : -(maxTabWidth(pane)));
              } 
              break;
          case BOTTOM: 
              if (run == 0) {
                  offset = (forward? 
                            maxTabHeight(pane) :
                            totalTabHeight(pane, tabPlacement, runCount)-maxTabHeight(pane));
              } else if (run == runCount - 1) {
                  offset = (forward?
                            -(totalTabHeight(pane, tabPlacement, runCount)-maxTabHeight(pane)) :
                            -(maxTabHeight(pane)));
              } else {
                  offset = (forward? maxTabHeight(pane) : -(maxTabHeight(pane)));
              } 
              break;
          case TOP:
          default:
              if (run == 0) {
                  offset = (forward? 
                            -(totalTabHeight(pane, tabPlacement, runCount)-maxTabHeight(pane)) :
                            -(maxTabHeight(pane)));
              } else if (run == runCount - 1) {
                  offset = (forward?
                            maxTabHeight(pane) :
                            totalTabHeight(pane, tabPlacement, runCount)-maxTabHeight(pane));
              } else {
                  offset = (forward? maxTabHeight(pane) : -(maxTabHeight(pane)));
              }
        }
        return offset;
    }
    
    public int tabForCoordinate(JTabbedPane pane, int x, int y) {
        int tabCount = pane.getTabCount();
        if (tabCount != rects.length) { 
              calculateLayoutInfo(pane); 
        }

        for (int i = 0; i < tabCount; i++) {
            if (rects[i].contains(x, y)) {
                return i;
            }
        }

        return -1;
    }
        

  // Mouse click on tab
  protected class MouseGetter
    extends MouseAdapter implements Serializable
  {
    public void mousePressed(MouseEvent _evt)
    {
      JTabbedPane p=(JTabbedPane)_evt.getSource();
      int         i=tabForCoordinate(p,_evt.getX(),_evt.getY());
      
      if((i>=0)&&p.isEnabledAt(i))
      {
	if(i==p.getSelectedIndex())
	{
	  p.requestFocus();
	  p.repaint(rects[i]);
	}
	else
	{
	  p.setSelectedIndex(i);
	  //p.invalidate();
	}
      }
    }
  }

  // Focus
  protected class FocusGetter
    extends FocusAdapter implements Serializable
  {
    public void focusGained(FocusEvent _evt)
    {
      JTabbedPane p=(JTabbedPane)_evt.getSource();
      Rectangle   r=getFocusedTabBounds(p);
      if(r!=null) p.repaint(r);
    }
            
    public void focusLost(FocusEvent _evt)
    {
      JTabbedPane p=(JTabbedPane)_evt.getSource();
      Rectangle   r=getFocusedTabBounds(p);
      if(r!=null) p.repaint(r);
    }
  }

  // Tab Placement
  protected static class PropertyGetter
    implements PropertyChangeListener
  {
    public void propertyChange(PropertyChangeEvent _evt)
    {
      if("tabPlacement".equals(_evt.getPropertyName()))
      {
	int tp=SlafManager.getInteger("TabbedPane.tabPlacement");
	if(tp!=0)
	{
	  if(!(new Integer(tp).equals(_evt.getNewValue())))
            ((JTabbedPane)_evt.getSource()).setTabPlacement(tp);
	}
      }
    }
  }
}
