/**
 * @modification $Date: 2004/01/26 18:17:34 $
 * @statut       unstable
 * @file         com/memoire/slaf/SlafTextAreaUI.java
 * @version      0.20
 * @author       Guillaume Desnoix
 * @email        guillaume-desnoix@memoire.com
 * @license      GNU General Public License 2 (GPL2)
 * @copyright    1999-2004 Guillaume Desnoix
 */

package com.memoire.slaf;
import com.memoire.slaf.*;

import java.awt.*;
import java.awt.event.*;
import java.beans.*;

import javax.swing.*;
import javax.swing.text.*;
import javax.swing.plaf.*;
import javax.swing.plaf.basic.*;

public class SlafTextAreaUI extends BasicTextAreaUI
{
  protected RolloverListener rolloverListener;
  protected boolean isRollover=false;
  protected boolean isPressed =false;

  public static ComponentUI createUI(JComponent c)
  {
    return new SlafTextAreaUI();
  }

  public void installUI(JComponent c)
  {
    super.installUI(c);
    SlafUtils.setOpaque(c);
    SlafUtils.installFocusBorder(c,"TextArea",true);

    if(SlafManager.getBoolean
       ("TextArea.rollover",
	SlafManager.getBoolean("Theme.rollover",true)))
    {
      rolloverListener=new RolloverListener(c);
      c.addMouseListener(rolloverListener);
    }
  }

  public void uninstallUI(JComponent c)
  {
    super.uninstallUI(c);
    if(rolloverListener!=null) c.removeMouseListener(rolloverListener);
  }

  /*
  protected Color getBackground(JComponent b)
  {
    Color r=b.getBackground();
    
    if(b.isEnabled()&&(isRollover||isPressed))
      r=SlafUtils.brighten(r,1);
    else
    if(!b.isOpaque()&&(b.getParent()!=null))
      r=b.getParent().getBackground();

    return r;
  }
  */

  protected void paintBackground(Graphics g)
  {
  }

  protected void paintBackground(Graphics g, JComponent b)
  {
    Rectangle r=SlafUtils.getFullRect(b);
    Color     c=SlafUtils.getBackground(b);
    int       t=SlafUtils.ENABLED;

    if(b.isEnabled())
    {
      // if(b.hasFocus()) t=SlafUtils.VERY_LIGHT;
      // else
      if(isRollover&&getComponent().isEditable()) t=SlafUtils.ROLLOVER;
    }
    else
    {
      if(c instanceof ColorUIResource)
	c=SlafManager.getColor("TextArea.disabledBackground",c);
    }

    SlafUtils.paintBackground(g,r,b,c,t);
  }
  
  protected void paintSafely(Graphics g)
  {
    paintBackground(g,getComponent());
    super.paintSafely(g);
  }

  /*
  public void update(Graphics g, JComponent c)
  {
    paintBackground(g,c);

    JTextComponent editor=getComponent();
    Color          old   =editor.getDisabledTextColor();
    if(!editor.isEnabled())
      editor.setDisabledTextColor(getBackground(editor).darker());

    paint(g,c);
    editor.setDisabledTextColor(old);
  }
  */

  class RolloverListener extends MouseAdapter
  {
    private JComponent c_;

    public RolloverListener(JComponent _c)
    {
      c_=_c;
    }

    public void mousePressed(MouseEvent _evt)
    {
      isPressed=true;
      c_.repaint();
    }

    public void mouseReleased(MouseEvent _evt)
    {
      isPressed=false;
      c_.repaint();
    }

    public void mouseEntered(MouseEvent _evt)
    {
      isRollover=true;
      c_.repaint();
    }

    public void mouseExited(MouseEvent _evt)
    {
      isRollover=false;
      c_.repaint();
    }
  }
}
