/**
 * @modification $Date: 2004/01/26 18:17:34 $
 * @statut       unstable
 * @file         com/memoire/slaf/SlafToggleButtonUI.java
 * @version      0.20
 * @author       Guillaume Desnoix
 * @email        guillaume-desnoix@memoire.com
 * @license      GNU General Public License 2 (GPL2)
 * @copyright    1999-2004 Guillaume Desnoix
 */

package com.memoire.slaf;
import com.memoire.slaf.*;

import java.awt.*;
import java.awt.event.*;

import javax.swing.*;
import javax.swing.border.*;
import javax.swing.plaf.*;
import javax.swing.plaf.basic.*;
import javax.swing.text.View;

public class SlafToggleButtonUI extends BasicToggleButtonUI
{
  private static final SlafToggleButtonUI
    togglebuttonui=new SlafToggleButtonUI();

  public static ComponentUI createUI(JComponent b)
  {
    return togglebuttonui;
  }

  public void installUI(JComponent c)
  {
    super.installUI(c);
    SlafUtils.setOpaque(c);
    SlafUtils.setRollover((AbstractButton)c);
    SlafUtils.installFocusBorder(c,"ToggleButton",true);
  }

  public void paint(Graphics g, JComponent c)
  {
    AbstractButton b = (AbstractButton) c;
    ButtonModel model = b.getModel();
	
    Dimension size = b.getSize();
    FontMetrics fm = g.getFontMetrics();

    Insets i = c.getInsets();

    Rectangle viewRect = new Rectangle(size);

    viewRect.x += i.left;
    viewRect.y += i.top;
    viewRect.width -= (i.right + viewRect.x);
    viewRect.height -= (i.bottom + viewRect.y);
    
    Rectangle iconRect = new Rectangle();
    Rectangle textRect = new Rectangle();

    Font f = c.getFont();
    g.setFont(f);

    // layout the text and icon
    String text = SwingUtilities.layoutCompoundLabel
	(b, fm, b.getText(), b.getIcon(),
	 b.getVerticalAlignment(), b.getHorizontalAlignment(),
	 b.getVerticalTextPosition(), b.getHorizontalTextPosition(),
	 viewRect, iconRect, textRect,
	 ((b.getText() == null) ? 0 : getDefaultTextIconGap(b)));

    /*
    g.setColor(b.getBackground());

    if (model.isArmed() && model.isPressed() || model.isSelected())
    {
      paintButtonPressed(g,b);
    } else if (b.isOpaque())
    {
      Insets insets = b.getInsets();
      Insets margin = b.getMargin();
	    
      g.fillRect
	  (insets.left - margin.left,
	   insets.top - margin.top, 
	   size.width - (insets.left-margin.left)
	   - (insets.right - margin.right),
	   size.height - (insets.top-margin.top)
	   - (insets.bottom - margin.bottom));
    }
    */

    if(b.getIcon() != null) { 
	paintIcon(g, b, iconRect);
    }
	
    if(text != null && !text.equals("")) {
	View v = (View) c.getClientProperty(BasicHTML.propertyKey);
	if (v != null) {
	    v.paint(g, textRect);
	} else {
	    paintText(g, b, textRect, text);
	}
    }
    
    if(SlafUtils.hasFocus(b)&&b.isFocusPainted())
      paintFocus(g, b, viewRect, textRect, iconRect);
  }

  protected void paintBackground(Graphics g, AbstractButton b)
  {
    Rectangle r=SlafUtils.getFullRect         (b);
    int       t=SlafUtils.getStatus           (b);
    Color     c=SlafUtils.getAdaptedBackground(b,"ToggleButton");
    SlafUtils.paintBackground(g,r,b,c,t);

    /*
    Rectangle r=SlafUtils.getFullRect  (b);
    int       t=SlafUtils.getStatus    (b);
    Color     c=SlafUtils.getBackground(b);
    if(b.isSelected())
      c=//SlafManager.getColor("ToggleButton.selectionBackground",
	SlafUtils.getAdaptedBackground(b);
    SlafUtils.paintBackground(g,r,b,c,t);
    */
  }
  
  public void update(Graphics g, JComponent c)
  {
    paintBackground(g,(AbstractButton)c);
    paint(g, c);
    SlafUtils.paintOver(g,(AbstractButton)c);
  }

  protected void paintFocus
    (Graphics g, AbstractButton b,
     Rectangle viewRect, Rectangle textRect, Rectangle iconRect)
  {
    SlafUtils.paintFocus(g,b,viewRect,textRect,iconRect);

    /*
    Rectangle r=SlafUtils.getViewRect(b);
    g.setColor(SlafManager.getColor("ToggleButton.focus"));
    g.drawLine(r.x, r.y+r.height-1, r.x+r.width-1, r.y+r.height-1);
    */
  }

  protected Icon getIcon(AbstractButton b)
  {
    ButtonModel    m=b.getModel();
    Icon           a=null;

    if(!m.isEnabled())
      a=b.getDisabledIcon();
    else
    if(m.isPressed()&&m.isArmed())
    {
      a=b.getPressedIcon();
      if(a==null)
	a=b.getSelectedIcon();
    }
    else
    if(m.isSelected())
      a=b.getSelectedIcon();
    else
    if(b.isRolloverEnabled()&&m.isRollover())
      a=b.getRolloverIcon();
	      
    if(a==null)
      a=b.getIcon();

    return a;
  }

  protected void paintIcon
    (Graphics g, AbstractButton b, Rectangle iconRect)
  {
    getIcon(b).paintIcon(b,g,iconRect.x,iconRect.y);
  }

  // TMP
  protected void paintButtonPressed(Graphics g, AbstractButton b)
  {
    /*
    Color pbg=SlafManager.getColor("ToggleButton.selectionBackground");

    if(pbg!=null)
    {
      Rectangle r=SlafUtils.getFullRect(b);
      g.setColor(pbg);
      g.fillRect(r.x,r.y,r.width-1,r.height-1);
    }
    */
  }

  protected void paintText
    (Graphics g, AbstractButton b, Rectangle textRect, String text)
  {
    SlafUtils.drawString(g,text,
			 textRect.x,
			 textRect.y,"ToggleButton",b);
    /*
    // AbstractButton b =(AbstractButton) c;			     
    ButtonModel    m =b.getModel();
    FontMetrics    fm=g.getFontMetrics();

    if(m.isEnabled())
    {
      Color fg=SlafUtils.getAdaptedForeground(b);
      g.setColor(fg);
      BasicGraphicsUtils.drawString(g,text, m.getMnemonic(),
				    textRect.x,
				    textRect.y+fm.getAscent());
    }
    else
    {
      Color df=SlafManager.getColor("ToggleButton.disabledForeground");
      Color ds=SlafManager.getColor("ToggleButton.disabledShadow");

      if(df!=null)
      {
	if(ds!=null)
	{
	  g.setColor(ds);
	  BasicGraphicsUtils.drawString(g,text,m.getMnemonic(),
					textRect.x+1,
					textRect.y+fm.getAscent()+1);
	}
	g.setColor(df);
	BasicGraphicsUtils.drawString(g,text,m.getMnemonic(),
				      textRect.x,
				      textRect.y+fm.getAscent());
      }
      else
      {
	df=SlafUtils.getAdaptedBackground(b);
	g.setColor(SlafUtils.brighten(df));
	BasicGraphicsUtils.drawString(g,text,m.getMnemonic(),
				      textRect.x+1,
				      textRect.y+fm.getAscent()+1);
	if(ds==null) ds=SlafUtils.darken(df);
	g.setColor(ds);
	BasicGraphicsUtils.drawString(g,text,m.getMnemonic(),
				      textRect.x,
				      textRect.y+fm.getAscent());
      }
    }
    */
  }
}
