/**
 * @modification $Date: 2004/01/26 18:17:34 $
 * @statut       unstable
 * @file         com/memoire/slaf/SlafToolTipUI.java
 * @version      0.20
 * @author       Guillaume Desnoix
 * @email        guillaume-desnoix@memoire.com
 * @license      GNU General Public License 2 (GPL2)
 * @copyright    1999-2004 Guillaume Desnoix
 */

package com.memoire.slaf;
import  com.memoire.slaf.*;

import java.awt.*;
import java.beans.*;
import java.io.Serializable;

import javax.swing.*;
import javax.swing.border.*;
import javax.swing.plaf.*;

public class SlafToolTipUI
       extends ToolTipUI
  implements Serializable, PropertyChangeListener
{
  private static final SlafToolTipUI tooltipUI=new SlafToolTipUI(); 

  public static ComponentUI createUI(JComponent c)
  {
    return tooltipUI;
  }

  public SlafToolTipUI()
  {
    super();
  }

  public void installUI(JComponent c)
  {
    installDefaults(c);
    installListeners(c);
    SlafUtils.setOpaque(c);
  }

  public void uninstallUI(JComponent c)
  {
    uninstallDefaults(c);
    uninstallListeners(c);
  }

  protected void installDefaults(JComponent c)
  {
    LookAndFeel.installColorsAndFont(c,"ToolTip.background",
				       "ToolTip.foreground",
				       "ToolTip.font");
    LookAndFeel.installBorder(c,"ToolTip.border");
  }
    
  protected void uninstallDefaults(JComponent c)
  {
    LookAndFeel.uninstallBorder(c);
  }

  protected void installListeners(JComponent c)
  {
    c.addPropertyChangeListener(this);
  }

  protected void uninstallListeners(JComponent c)
  {
    c.removePropertyChangeListener(this);
  }

  public void propertyChange(PropertyChangeEvent _evt)
  {
    JComponent c=(JComponent)_evt.getSource();
    Container  q=c;
    for(int i=0;i<4;i++)
    {
      if((q instanceof JComponent)&&((JComponent)q).isOpaque()) q=null;
      if(q!=null) q=q.getParent();
    }

    Boolean b=((q!=null)
	       &&"javax.swing.Popup$HeavyWeightWindow"
	       .equals(q.getClass().getName()))
      ? Boolean.TRUE
      : Boolean.FALSE;


    Container p=c.getParent();
    if(p instanceof JComponent)
    {
      ((JComponent)p).putClientProperty("SLAF_OPAQUE_PANEL",b);
      ((JComponent)p).setOpaque(false);
      if(!b.equals(c.getClientProperty("SLAF_NO_SHADOW")))
      {
	c.putClientProperty("SLAF_NO_SHADOW",b);
	/*
	if((q!=null)&&!q.isShowing())
	{
	  //System.err.println("RESIZE WINDOW");
	  c.invalidate();
	  c.setSize(c.getPreferredSize());
	  q.invalidate();
	  q.setSize(q.getPreferredSize());
	}
	*/
      }
    }

    /*
    Container p=((JComponent)_evt.getSource()).getParent();
    if((p instanceof JComponent)&&((JComponent)p).isOpaque())
    {
      //((JComponent)p).setOpaque(false);
      ((JComponent)p).setOpaque(!p.isLightweight());
      //((JComponent)p).setBorder(new EmptyBorder(0,0,0,0));
    }
    */
  }

  protected void paintBackground(Graphics g, JComponent b)
  {
    /*
      Rectangle r=SlafUtils.getFullRect  (b);
      Color     c=SlafUtils.getBackground(b);
      SlafUtils.paintBackground(g,r,b,c);
    */
    SlafUtils.paintBackground(g,b);
  }
  
  public void update(Graphics g, JComponent b)
  {
    paintBackground(g,b);
    paint(g,b);
  }

  public void paint(Graphics g, JComponent c)
  {
    String text=((JToolTip)c).getTipText();

    if((text!=null)&&!"".equals(text))
    {
      Insets      insets =c.getInsets();
      Font        font   =c.getFont();
      FontMetrics metrics=c.getToolkit().getFontMetrics(font);
      Dimension   size   =c.getSize();

      g.setColor(c.getForeground());
      g.setFont(font);
      g.drawString(text,insets.left,insets.top+metrics.getAscent());
    }
  }

  public Dimension getPreferredSize(JComponent c)
  {
    Insets      insets =c.getInsets();
    Font        font   =c.getFont();
    FontMetrics metrics=Toolkit.getDefaultToolkit().getFontMetrics(font);

    // fix for bug 4094153
    String tipText=((JToolTip)c).getTipText();
    if(tipText==null)
      return new Dimension(insets.left+insets.right,
			   insets.top+insets.bottom+
			   metrics.getHeight());
    else
      return new Dimension(insets.left+insets.right+
			   metrics.stringWidth(tipText),
			   insets.top+insets.bottom+
			   metrics.getHeight());
  }

  public Dimension getMinimumSize(JComponent c)
  {
    return getPreferredSize(c);
  }

  public Dimension getMaximumSize(JComponent c)
  {
    return getPreferredSize(c);
  }
}


