/**
 * @modification $Date: 2004/03/03 19:56:47 $
 * @statut       unstable
 * @file         com/memoire/slaf/SlafUtils.java
 * @version      0.20
 * @author       Guillaume Desnoix
 * @email        guillaume-desnoix@memoire.com
 * @license      GNU General Public License 2 (GPL2)
 * @copyright    1999-2004 Guillaume Desnoix
 */

package com.memoire.slaf;
import com.memoire.slaf.*;

import java.awt.*;
import java.awt.image.*;
import java.beans.*;
import java.net.*;
import java.util.*;
import javax.swing.*;
import javax.swing.border.*;
import javax.swing.plaf.*;
import javax.swing.plaf.basic.*;

public class SlafUtils
{
  // Misc

  private static double jdk_=jdk_version();

  public static final double jdk_version()
  {
    double r=1.1;
    try
    {
      r=new Double(System.getProperty("java.version")
		   .substring(0,3)).doubleValue();
    }
    catch(Exception ex)
    {
      System.err.println("Unknown JDK version. Switched to 1.1.");
    }
    return r;
  }

  public static final double jdk()
  {
    return jdk_;
  }

  public static final int max(int a, int b, int c)
  {
    return Math.max(Math.max(a,b),c);
  }

  public static final int min(int a, int b, int c)
  {
    return Math.min(Math.min(a,b),c);
  }

  public static final int avg(int a, int b, int c)
  {
    return (a+b+c)/3;
  }

  public static final int max(int a, int b, int c, int d)
  {
    return Math.max(max(a,b,c),d);
  }

  public static final int min(int a, int b, int c, int d)
  {
    return Math.min(min(a,b,c),d);
  }

  public static final int avg(int a, int b, int c, int d)
  {
    return (a+b+c+d)/4;
  }

  public static final int max(int a, int b, int c, int d, int e)
  {
    return Math.max(max(a,b,c,d),e);
  }

  public static final int min(int a, int b, int c, int d, int e)
  {
    return Math.min(min(a,b,c,d),e);
  }

  public static final int max(int a, int b, int c, int d, int e, int f)
  {
    return Math.max(max(a,b,c,d,e),f);
  }

  public static final int min(int a, int b, int c, int d, int e, int f)
  {
    return Math.max(min(a,b,c,d,e),f);
  }

  // Debug

  private static boolean debug_=amanda();

  public static final void    setDebug(boolean _debug) { debug_=_debug; }
  public static final boolean getDebug()               { return debug_; }

  private static final boolean amanda()
  {
    boolean r=false;
    try { r="172.17.250.86".equals
	    (InetAddress.getLocalHost().getHostAddress()); }
    catch(UnknownHostException ex) { }
    return r;
  }

  public static final void trace(Object _o)
  {
    if(debug_) System.err.println("SLF: "+_o);
  }

  public static final void error(Object _o)
  {
    System.err.println("SLF: "+_o);
  }

  /*
  public static final void check(boolean _test, String _message)
  {
    if(debug_&&!_test)
    {
      System.err.println("SLF: check not verified, "+_message);
      new RuntimeException().printStackTrace();
      System.err.println("-----");
    }
  }
  */

  // Color

  public static final Color contrast(Color _c)
  {
    Color r=null;

    if(max(_c.getRed(),_c.getGreen(),_c.getBlue())<160)
      r=Color.white;
    else
      r=Color.black;

    if(_c instanceof ColorUIResource)
      r=new ColorUIResource(r);

    return r;
  }

  public static final Color brighten(Color _c)
    { return intensity(_c,+2); }

  public static final Color darken(Color _c)
    { return intensity(_c,-2); }

  public static final Color brighten(Color _c,int d)
    { return intensity(_c,+d); }

  public static final Color darken(Color _c,int d)
    { return intensity(_c,-d); }

  // Intensity

  public static final Color intensity(Color _c, int _d)
  {
    Color r=new Color(intensity(_c.getRGB(),_d));

    if(_c instanceof ColorUIResource)
      r=new ColorUIResource(r);

    return r;
  }

  public static final int intensity(int rgb, int d)
  {
    int a=(rgb & 0xff000000) >>24;
    int r=(rgb & 0x00ff0000) >>16;
    int g=(rgb & 0x0000ff00) >> 8;
    int b=(rgb & 0x000000ff)     ;

    if(d>0)
    {
      int i=d;
      while(i>0)
      {
	r=r*10/9;
	g=g*10/9;
	b=b*10/9;
	i--;
      }
    }
    else
    {  
      int i=-d;
      while(i>0)
      {
	r=r*9/10;
	g=g*9/10;
	b=b*9/10;
	i--;
      }
    }

    if(r>255) r=255;
    if(g>255) g=255;
    if(b>255) b=255;

    return (a<<24) | (r<<16) | (g<<8) | b;
  }

  private static final class IntensityFilter
                       extends RGBImageFilter
  {
    private int n_;

    public IntensityFilter(int _n)
    {
      n_=_n;
      // canFilterIndexColorModel=true;
    }

    public int filterRGB(int x, int y, int rgb)
    {
      return intensity(rgb,n_);
    }
  }

  /*
  public static final Color brighten(Color _c)
    { return brighten(_c,2); }

  public static final Color darken(Color _c)
    { return darken(_c,2); }

  public static final Color brighten(Color _c, int _i)
    { return intensity(_c,_i); }

  public static final Color darken(Color _c, int _i)
    { return intensity(_c,-_i); }
  */

  public static final Image applyFilter(ImageFilter f, JComponent c, Image i)
  {
    return SlafResource.ensureImageIsLoaded
      (c.createImage
       (new FilteredImageSource
	(i.getSource(),f)),i);
  }

  // Image

  public static void paintImage
  (Image image, Component c, Graphics g,
   Rectangle view, Insets insets, boolean center)
  {
    int wi=image.getWidth(c);
    int hi=image.getHeight(c);

    int x     =view.x;
    int y     =view.y;
    int width =view.width;
    int height=view.height;

    int top   =insets.top;
    int left  =insets.left;
    int bottom=insets.bottom;
    int right =insets.right;

    g.translate(x,y);

    //W
    if((left>=0)&&(hi-bottom>=top))
    g.drawImage(image,0,top,left,height-bottom,
		0,top,left,hi-bottom,c);
    //S
    if((wi-right>=left)&&(bottom>=0))
    g.drawImage(image,left,height-bottom,width-right,height,
		left,hi-bottom,wi-right,hi,c);
    //N
    if((top>=0)&&(wi-right>=left))
    g.drawImage(image,left,0,width-right,top,
		left,0,wi-right,top,c);
    //E
    if((right>=0)&&(hi-bottom>=top))
    g.drawImage(image,width-right,top,width,height-bottom,
		wi-right,top,wi,hi-bottom,c);
    //SW
    if((left>=0)&&(bottom>=0))
    g.drawImage(image,0,height-bottom,left,height,
		0,hi-bottom,left,hi,c);
    //NW
    if((left>=0)&&(top>=0))
    g.drawImage(image,0,0,left,top,
		0,0,left,top,c);
    //SE
    if((right>=0)&&(bottom>=0))
    g.drawImage(image,width-right,height-bottom,width,height,
		wi-right,hi-bottom,wi,hi,c);
    //NE
    if((right>=0)&&(top>=0))
    g.drawImage(image,width-right,0,width,top,
		wi-right,0,wi,top,c);
    //CENTER
    if(center&&(wi-right>=left)&&(hi-bottom>=top))
      g.drawImage(image,left,top,width-right,height-bottom,
		left,top,wi-right,hi-bottom,c);

    g.translate(-x,-y);
  }

  // Text

  public static final void drawString(Graphics _g, String _text,
				      int _x, int _y,
				      String _ui, AbstractButton _b)
  {
    if(_b.isEnabled())
      drawEnabledString(_g,_text,_x,_y,_ui,_b);
    else
      drawDisabledString(_g,_text,_x,_y,_ui,_b);
  }

  public static final void drawEnabledString(Graphics _g, String _text,
					     int _x, int _y,
					     String _ui, AbstractButton _b)
  {
    Color fg=getAdaptedForeground(_b,_ui);
    int   ac=_b.getMnemonic();

    _y+=_g.getFontMetrics().getAscent();
    _g.setColor(fg);
    BasicGraphicsUtils.drawString(_g,_text,ac,_x,_y);
  }

  public static final void drawDisabledString(Graphics _g, String _text,
					      int _x, int _y,
					      String _ui, AbstractButton _b)
  {
    Color df=SlafManager.getColor(_ui+".disabledForeground");
    Color ds=SlafManager.getColor(_ui+".disabledShadow");
    int   ac=_b.getMnemonic();

    _y+=_g.getFontMetrics().getAscent();

    if(df!=null)
    {
      if(ds!=null)
      {
	_g.setColor(ds);
	BasicGraphicsUtils.drawString(_g,_text,ac,_x+1,_y+1);
      }
      _g.setColor(df);
      BasicGraphicsUtils.drawString(_g,_text,ac,_x,_y);
    }
    else
    {
      df=getAdaptedBackground(_b);
      _g.setColor(brighten(df));
      BasicGraphicsUtils.drawString(_g,_text,ac,_x+1,_y+1);
      if(ds==null) ds=darken(df);
      _g.setColor(ds);
      BasicGraphicsUtils.drawString(_g,_text,ac,_x,_y);
    }
  }
 
  // Background

  public static final int ENABLED          =0;
  public static final int ROLLOVER         =1;
  public static final int PRESSED          =2;
  public static final int SELECTED         =3;
  public static final int ROLLOVER_SELECTED=4;

  public static final int[] HUE=new int[]
  { 0, +1, -2, -1, 1 }; 

  private static Hashtable[] cache=null;

  public static final void clearCache()
  {
    cache=null;
  }

  public static final Image mixBackground
    (JComponent c,Image i,Color bg, boolean _fading)
  {
    return mixBackground(c,i,bg,_fading,128);
  }

  public static final Image mixBackground
    (JComponent c,Image i,Color bg, boolean _fading,int _alpha)
  {
    if(bg==null) return i;

    _alpha=((_alpha&0xFF)<<24)&0xFF000000;

    return applyFilter
      (//new SlafColorFilter(bg.getRGB()),
       //new SlafHueFilter(255<<24,bg.getRGB()),
       new SlafMixFilter(0x00000000,_alpha|(bg.getRGB()&0x00FFFFFF),
			 i.getWidth(c),i.getHeight(c),_fading),
       c,i);
  }

  public static final Image intensity(JComponent c,Image i,int t)
  {
    int d=HUE[t];

    if(d==0) return i;

    if(cache==null)
    {
      cache=new Hashtable[HUE.length];
      for(int j=0;j<HUE.length;j++)
	cache[j]=new Hashtable(11);
    }

    Image r=(Image)cache[t].get(i);

    if(r==null)
    {
      r=SlafResource.ensureImageIsLoaded
	(c.createImage
	 (new FilteredImageSource
	  (i.getSource(),new IntensityFilter(d))),r);
      cache[t].put(i,r);
    }

    return r;
  }

  public static final void paintBackground
    (Graphics g,JComponent c)
    { paintBackground(g,getFullRect(c),c,getBackground(c),ENABLED); }

  public static final void paintBackground
    (Graphics g,Rectangle r, JComponent c, Color bg)
    { paintBackground(g,r,c,bg,ENABLED); }

  public static final void paintBackground
    (Graphics g,Rectangle r, JComponent c, Color bg, int t)
  {
    String ui=c.getUIClassID();
    if(ui.endsWith("UI")) ui=ui.substring(0,ui.length()-2);
    paintBackground(g,r,c,bg,t,ui,false,false);
  }

  public static final void paintBackground
    (Graphics g,Rectangle r, JComponent c, Color bg, int t, String ui)
  {
    paintBackground(g,r,c,bg,t,ui,false,false);
  }

  public static final void paintBackground
    (Graphics g,Rectangle r, JComponent c,
     Color bg, int t, String ui, boolean ft)
  {
    paintBackground(g,r,c,bg,t,ui,ft,false);
  }

  public static final void paintBackground
    (Graphics g,Rectangle r, JComponent c,
     Color bg, int t, String ui, boolean ft, boolean fbg)
  {
    if(!SlafManager.getBoolean(ui+".backgroundPainted",true)) return;

    if(SlafManager.getBoolean("ToolBar.clean",false)&&
       (c instanceof AbstractButton)&&
       (c.getParent() instanceof JToolBar)&&
       (Boolean.TRUE.equals
	(((JComponent)c.getParent())
	 .getClientProperty("JToolBar.isRollover"))||
	!((AbstractButton)c).isBorderPainted()))
      /*
	||
	(c.getBorder()==null)||
	(c.getBorder() instanceof EmptyBorder)))
      */
    {
      if(c.isOpaque())
      {
	c.setOpaque(false);
	//c.putClientProperty("SLAF_TOOL_WAS_OPAQUE",Boolean.TRUE);
      }

      ButtonModel model =((AbstractButton)c).getModel();

      if((!model.isEnabled())||
	 (!model.isPressed()&&
	  !model.isArmed()&&
	  !model.isRollover()&&
	  !model.isSelected()))
	return;
    }

    int     m=SlafManager.getInteger(ui+".textureType");
    boolean changeIntensity=false;
    Image   i=null;
    boolean o=SlafManager.getBoolean(ui+".opaque",c.isOpaque());

    if(!ft&&Boolean.TRUE.equals(c.getClientProperty("SLAF_NO_TEXTURE")))
    {
      o=c.isOpaque();
      //System.err.println("#### NO TEXTURE FOR "+c.getClass());
    }
    else
    {
      if(!c.isEnabled())
      {
	if(t==SELECTED) i=SlafManager.getImage(ui+".disabledSelectedTexture");
	if(i==null) i=SlafManager.getImage(ui+".disabledTexture");
      }
      else if(t==PRESSED ) i=SlafManager.getImage(ui+".pressedTexture");
      else if(t==ROLLOVER) i=SlafManager.getImage(ui+".rolloverTexture");
      else if(t==SELECTED) i=SlafManager.getImage(ui+".selectedTexture");
      else if(t==ROLLOVER_SELECTED)
	i=SlafManager.getImage(ui+".rolloverSelectedTexture",
			       SlafManager.getImage(ui+".selectedTexture"));
      else
	i=SlafManager.getImage(ui+".enabledTexture");

      if((i==null)&&(c instanceof Adjustable))
      {
	if(((Adjustable)c).getOrientation()==Adjustable.VERTICAL)
          i=SlafManager.getImage(ui+".verticalTexture");
	else
	  i=SlafManager.getImage(ui+".horizontalTexture");
        changeIntensity=true;
      }

      if((i==null)&&(c instanceof JToolBar))
      {
	if(((JToolBar)c).getOrientation()==Adjustable.VERTICAL)
          i=SlafManager.getImage(ui+".verticalTexture");
	else
	  i=SlafManager.getImage(ui+".horizontalTexture");
        changeIntensity=true;
      }

      if(i==null)
      {
        i=SlafManager.getImage(ui+".texture");
        changeIntensity=true;
      }
    }

    if((o==false)&&(i==null)
       &&(c instanceof AbstractButton)
       &&SlafManager.getBoolean(ui+".rollover",true))
      o=((AbstractButton)c).getModel().isRollover();

    // A TESTER
    /*
    if(  (o==false)
       &&(c instanceof AbstractButton)
       &&!((AbstractButton)c).isBorderPainted()
       &&((AbstractButton)c).isContentAreaFilled()
       &&!new Rectangle(0,0,0,0).equals(c.getInsets()))
      o=true;
    */

    int alpha=255;
    if(!o)
    {
      alpha=SlafManager.getInteger(ui+".alpha",alpha);
      if(t==SELECTED) alpha=SlafManager.getInteger(ui+".selectedAlpha",alpha);
      /*
      if(!c.isEnabled())   alpha=SlafManager.getInteger(ui+".disabledAlpha",alpha);
      else if(t==PRESSED ) alpha=SlafManager.getInteger(ui+".pressedAlpha" ,alpha);
      else if(t==ROLLOVER) alpha=SlafManager.getInteger(ui+".rolloverAlpha",alpha);
      else if(t==SELECTED) alpha=SlafManager.getInteger(ui+".selectedAlpha",alpha);
      else                 alpha=SlafManager.getInteger(ui+".enabledAlpha" ,alpha);
      */
    }

    //if(ui.indexOf("Popup")>=0)
    //System.err.println("POPUP: "+ui+" "+o+" "+alpha);

    Object old=null;
    if(alpha<255)
    {
      try
      {
	old=((Graphics2D)g).getComposite();
	((Graphics2D)g).setComposite
	  (AlphaComposite.getInstance
	   (AlphaComposite.SRC_OVER,((float)alpha)/255.f));
      }
      catch(NoClassDefFoundError th) { }
    }

    if(o||((alpha<255)&&(i==null)))
      //||(c.getClientProperty("SLAF_TOOL_WAS_OPAQUE")==Boolean.TRUE))
    {
      Color color=bg;
      if(!fbg&&color.equals(getBackground(c)))
	color=intensity(color,HUE[t]);
      g.setColor(color);
      g.fillRect(r.x,r.y,r.width,r.height);
    }

    if(i!=null)
    {
      r=getTextureRect(c,ui,r);

      int w=i.getWidth (c);
      int h=i.getHeight(c);

      /*
      boolean borderpainted=true;
      try
      {
	if(c instanceof AbstractButton)
	  borderpainted=((AbstractButton)c).isBorderPainted();
	if(c instanceof JMenuBar)
	  borderpainted=((JMenuBar)c).isBorderPainted();
	if(c instanceof JPopupMenu)
	  borderpainted=((JPopupMenu)c).isBorderPainted();
	if(c instanceof JProgressBar)
	  borderpainted=((JProgressBar)c).isBorderPainted();
	if(c instanceof JToolBar)
	  borderpainted=((JToolBar)c).isBorderPainted();
      }
      catch(Throwable th) { }

      if(  !borderpainted
	 &&c.isOpaque()
	 &&!r.equals(new Rectangle(0,0,w,h)))
      {
	Insets insets=c.getInsets();
	Color color=bg;
	if(color.equals(getBackground(c)))
	  color=intensity(color,HUE[t]);
	g.setColor(color);
	g.fillRect(0,0,w,insets.top);
	g.fillRect(0,0,insets.left,h);
	g.fillRect(0,h-1-insets.bottom,w,insets.bottom);
	g.fillRect(w-1-insets.right,0,insets.right,h);
      }
      */

      int x,y,x0,y0;

      if(changeIntensity) i=intensity(c,i,t);

      // if not locationOnScreen
      if(!(c.getBackground() instanceof ColorUIResource))
	i=mixBackground(c,i,c.getBackground(),(m!=2)&&(m!=5));

      Shape old_clip=g.getClip();
      Rectangle bounds=g.getClipBounds();
      if(bounds==null) bounds=r;
      g.setClip(r.intersection(bounds));

      JComponent NULL=null; //c

      switch(m)
      {
      case 0:
	break;
      case 2: // mosaic absolute to top-level ancestor
	{
	  Point p,q;
	  // component must be showing on the screen to determine its location
	  try
	  {
	    p=c.getLocationOnScreen();
	    q=c.getTopLevelAncestor().getLocationOnScreen();
	    p.x-=q.x;
	    p.y-=q.y;
	  }
	  catch(IllegalComponentStateException ex) { p=new Point(0,0); }

	  x=r.x;
	  y=r.y;
	  x0=x/w*w-(p.x%w);
	  y0=y/h*h-(p.y%h);
	  for(y=y0; y<r.y+r.height; y+=h)
	    for(x=x0; x<r.x+r.width; x+=w)
	      g.drawImage(i,x,y,NULL);
	}
	break;
      case 3: // mosaic relative
	x=r.x;
	y=r.y;
	x0=x/w*w;
	y0=y/h*h;
	for(y=y0; y<r.y+r.height; y+=h)
	  for(x=x0; x<r.x+r.width; x+=w)
	    g.drawImage(i,x,y,NULL);
	break;
      case 4: // center
	x=r.x+(r.width-w)/2;
	y=r.y+(r.height-h)/2;
	g.drawImage(i,x,y,NULL);
	break;
      case 5: // mosaic absolute to screen
	{
	  Point p;
	  // component must be showing on the screen to determine its location
	  try { p=c.getLocationOnScreen(); }
	  catch(IllegalComponentStateException ex) { p=new Point(0,0); }
	  
	  x=r.x;
	  y=r.y;
	  x0=x/w*w-(p.x%w);
	  y0=y/h*h-(p.y%h);
	  for(y=y0; y<r.y+r.height; y+=h)
	    for(x=x0; x<r.x+r.width; x+=w)
	      g.drawImage(i,x,y,NULL);
	}
	break;
      case 7:
	{
	  Insets insets=SlafManager.getInsets(ui+".textureInsets");
	  //trace("$$$$ INSETS("+ui+")="+insets);
	  if(insets==null) insets=new Insets(5,5,5,5);
	  paintImage(i,c,g,r,insets,true);
	}
        break;
      case 1:
      default:
	{
	  x=r.x;
	  y=r.y;
	  g.drawImage(i,x,y,r.width,r.height,NULL);
	}
        break;
      }

      /*
      bg=c.getBackground();
      if(!(bg instanceof ColorUIResource))
      {
	g.setColor(new Color(bg.getRed(),bg.getGreen(),bg.getBlue(),128));
	g.fillRect(r.x,r.y,r.width,r.height);
      }
      */

      g.setClip(old_clip);
    }

    if(old!=null)
    {
      try
      {
	((Graphics2D)g).setComposite((Composite)old);
      }
      catch(NoClassDefFoundError th) { }
    }
  }

  // Over

  public static final void paintOver
    (Graphics g,JComponent c)
    { paintOver(g,getViewRect(c),c,getBackground(c)); }

  public static final void paintOver
    (Graphics g,AbstractButton c)
    { paintOver(g,getViewRect(c),c,getAdaptedBackground(c)); }

  public static final void paintOver
    (Graphics g,Rectangle r, JComponent c, Color bg)
  {
    if(!c.isEnabled())
    {
      String ui=c.getUIClassID();
      if(ui.endsWith("UI")) ui=ui.substring(0,ui.length()-2);
      paintOver(g,r,c,bg,ui);
    }
  }

  public static final void paintOver
    (Graphics g,Rectangle r, JComponent c, Color bg, String ui)
  {
    int t=SlafManager.getInteger(ui+".over");
    g.setColor(bg);
    switch(t)
    {
      case 1:
	for(int j=0;j<=r.height;j++)
	  for(int i=(2*j)%3;i<=r.width;i+=3)
	    g.drawLine(r.x+i,r.y+j,r.x+i,r.y+j);
	break;
      case 2:
	for(int j=0;j<=r.height;j+=2)
	  for(int i=(j/2)%2;i<=r.width;i+=2)
	    g.drawLine(r.x+i,r.y+j,r.x+i,r.y+j+1);
	break;
      case 3:
	for(int j=0;j<=r.height;j++)
	  for(int i=j%2;i<=r.width;i+=2)
	    g.drawLine(r.x+i,r.y+j,r.x+i,r.y+j);
	break;
      case 4:
	for(int i=0;i<=r.width;i+=2)
	  g.drawLine(r.x+i,r.y,r.x+i,r.y+r.height);
	break;
      case 5:
	for(int j=0;j<=r.height;j+=2)
	  g.drawLine(r.x,r.y+j,r.x+r.width,r.y+j);
	break;
    }
  }

  // Focus

  public static final void paintFocus
    (Graphics g, JComponent c,
     Rectangle viewRect, Rectangle textRect, Rectangle iconRect)
  {
    String ui=c.getUIClassID();
    if(ui.endsWith("UI")) ui=ui.substring(0,ui.length()-2);
    paintFocus(g,c,viewRect,textRect,iconRect,ui);
  }

  public static final void paintFocus
    (Graphics g, JComponent c,
     Rectangle viewRect, Rectangle textRect, Rectangle iconRect,
     String ui)
  {
    int fr=SlafManager.getInteger
      (ui+".focusRect",SlafManager.getInteger("Theme.focusRect",1));

    if(c instanceof SlafComboBoxButton)
      fr=SlafManager.getInteger("ComboBoxButton.focusRect",fr);

    if(viewRect==null) viewRect=getViewRect(c);
    if(textRect==null) textRect=viewRect;
    if(iconRect==null) iconRect=viewRect;

    Rectangle rr=null;
    switch(fr)
    {
    case  0: rr=getFullRect(c); break;
    case  2: rr=textRect; break;
    case  3: rr=iconRect; break;
    case  4: rr=textRect.union(iconRect); break;
    default: rr=viewRect; break;
    }

    //System.err.println("PAINT-FOCUS: "+rr+" "+c.getClass()+" ui:"+ui);

    Border b=SlafManager.getBorder
      (ui+".focusBorder",SlafManager.getBorder("Theme.focusBorder",null));

    if(c instanceof SlafComboBoxButton)
      b=SlafManager.getBorder("ComboBoxButton.focusBorder",b);

    if(b!=null)
      b.paintBorder(c,g,rr.x,rr.y,rr.width,rr.height);
    else
    {
      Color fc=SlafManager.getColor
	(ui+".focus",SlafManager.getColor("Theme.focus",Color.black));
      Color old=g.getColor();
      g.setColor(fc);
      g.drawLine(rr.x,rr.y+rr.height-1,rr.x+rr.width-1,rr.y+rr.height-1);
      g.setColor(old);
    }
  }

  // Component

  public static final boolean hasFocus(Component c)
  {
    boolean r=false;
    if(jdk()>1.2)
      r=c.hasFocus();
    else
    {
      for(Container p=c.getParent(); p!=null; p=p.getParent())
      {
	if((p instanceof Window)&&(((Window)p).getFocusOwner()==c))
	  r=true;
      }
    }
      
    return r;
  }

  public static final void setOpaque(JComponent c)
    { setOpaque(c,true); }

  public static final void setOpaque(JComponent c, boolean d)
  {
    String ui=c.getUIClassID();
    if(ui.endsWith("UI")) ui=ui.substring(0,ui.length()-2);
    setOpaque(c,ui,d);
  }

  public static final void setOpaque(JComponent c, String ui, boolean d)
  {
    /*
    if(c instanceof JTextArea)
    {
      System.err.println("===== "+SlafManager.get("TextArea.opaque")+" "+d);
      System.err.println("  ==> "+ui+" "+SlafManager.getBoolean(ui+".opaque",d));
    }
    */
    boolean o=SlafManager.getBoolean(ui+".opaque",d);
    if(o!=c.isOpaque()) c.setOpaque(o);
    //c.setDoubleBuffered(c.isOpaque());
  }

  public static final Rectangle getTextureRect(JComponent c)
  {
    return getTextureRect(c,null,null);
  }

  /*
  public static final Rectangle getTextureRect(JComponent c, String ui)
  {
    return getTextureRect(c,null,null);
  }

  public static final Rectangle getTextureRect(JComponent c, Rectangle r)
  {
    return getTextureRect(c,null,null);
  }
  */

  public static final Rectangle getTextureRect(JComponent c, String ui, Rectangle r)
  {
    if(ui==null)
    {
      ui=c.getUIClassID();
      if(ui.endsWith("UI")) ui=ui.substring(0,ui.length()-2);
    }

    if(r==null)
    {
      switch(SlafManager.getInteger(ui+".textureRect",0))
      {
      case  1: r=getViewRect(c); break;
      default: r=getFullRect(c); break;
      }
    }
    else r=new Rectangle(r.x,r.y,r.width,r.height);

    Insets i=SlafManager.getInsets(ui+".textureMargin",null);
    if(i!=null)
    {
      // hack to ignore margins for heavy popups
      if(!Boolean.TRUE.equals(c.getClientProperty("SLAF_NO_SHADOW")))
      {
	r.x+=i.left;
	r.y+=i.top;
	r.width -=i.left+i.right;
	r.height-=i.top+i.bottom;
	//trace("TEXTURE_MARGIN="+i);
      }
    }

    return r;
  }

  public static final Rectangle getBackgroundRect(JComponent c)
  {
    return getBackgroundRect(c,null,null);
  }

  public static final Rectangle getBackgroundRect(JComponent c, String ui)
  {
    return getBackgroundRect(c,null,null);
  }

  public static final Rectangle getBackgroundRect(JComponent c, Rectangle r)
  {
    return getBackgroundRect(c,null,null);
  }

  public static final Rectangle getBackgroundRect(JComponent c, String ui, Rectangle r)
  {
    if(ui==null)
    {
      ui=c.getUIClassID();
      if(ui.endsWith("UI")) ui=ui.substring(0,ui.length()-2);
    }

    if(r==null)
    {
      switch(SlafManager.getInteger(ui+".backgroundRect",0))
      {
      case  1: r=getViewRect(c); break;
      default: r=getFullRect(c); break;
      }
    }

    Insets i=SlafManager.getInsets(ui+".backgroundMargin",null);
    if(i!=null)
    {
      r.x+=i.left;
      r.y+=i.top;
      r.width -=i.left+i.right;
      r.height-=i.top+i.bottom;
      //trace("BACKGROUND_MARGIN="+i);
    }

    return r;
  }

  public static final Rectangle getFullRect(JComponent c)
  {
    Dimension d=c.getSize();
    Rectangle r=new Rectangle(d);
    return r;
  }

  public static final Rectangle getViewRect(JComponent c)
  {
    Insets    i=c.getInsets();
    Rectangle r=getFullRect(c);

    r.x     +=i.left;
    r.y     +=i.top;
    r.width -=(i.right +r.x);
    r.height-=(i.bottom+r.y);

    return r;
  }

  public static final Color getBackground(JComponent c)
  {
    Color r=c.getBackground();

    if(!c.isOpaque())
    {
      Container p=c.getParent();
      if(p instanceof JComponent)
	r=getBackground((JComponent)p);
      else
      if(p!=null)
	r=p.getBackground();
    }

    return r;
  }

  public static final Color getForeground(JComponent c)
  {
    Color r=c.getForeground();
    return r;
  }

  // AbstractButton

   public static final void setRollover(AbstractButton b)
    { setRollover(b,SlafManager.getBoolean("Theme.rollover",true)); }

  public static final void setRollover(AbstractButton b, boolean d)
  {
    String ui=b.getUIClassID();
    if(ui.endsWith("UI")) ui=ui.substring(0,ui.length()-2);
    b.setRolloverEnabled(SlafManager.getBoolean(ui+".rollover",d));
  }

  public static final Insets getDefaultMargin(AbstractButton b)
  {
    return new Insets(2,2,2,2);
  }

  public static final Rectangle getViewRect(AbstractButton b)
  {
    Insets    i=b.getInsets();
    Rectangle r=getFullRect(b);

    if(b.getBorder()==null)
      i=getDefaultMargin(b);

    r.x     +=i.left;
    r.y     +=i.top;
    r.width -=(i.right +r.x);
    r.height-=(i.bottom+r.y);

    return r;
  }

  public static final int getStatus(AbstractButton b)
  {
    ButtonModel m=b.getModel();
    int         r=ENABLED;

    if(b.isEnabled())
    {
      if(!(b instanceof MenuElement)) // isFocusPainted()
      {
	String ui=b.getUIClassID();
	if(ui.endsWith("UI")) ui=ui.substring(0,ui.length()-2);

	if(m.isArmed()&&m.isPressed())
	  r=PRESSED;
	else
	if(m.isRollover()&&SlafManager.getBoolean(ui+".rollover",true))
	{
	  if(  m.isSelected()
	     &&!(b instanceof JRadioButton)
	     &&!(b instanceof JCheckBox))
	    r=ROLLOVER_SELECTED;
	  else
	    r=ROLLOVER;
	}
	else
	if(  m.isSelected()
	   &&!(b instanceof JRadioButton)
	   &&!(b instanceof JCheckBox))
	  r=SELECTED;
      }
      else
      {
	if(m.isArmed()||(b instanceof JMenu && m.isSelected()))
	  r=SELECTED;//PRESSED;//ROLLOVER;

	/*
	else
	if(b.isSelected())
	{
	  // SlafUtils.trace(" selected="+m.isSelected());
	  // SlafUtils.trace(" armed   ="+m.isArmed());
	  // SlafUtils.trace(" pressed ="+m.isPressed());
	  // SlafUtils.trace(" rolled  ="+m.isRollover());

	  r=SlafUtils.DARK;
	}
	*/
      }
    }
    else
    if(  m.isSelected()
       &&!(b instanceof MenuElement )
       &&!(b instanceof JRadioButton)
       &&!(b instanceof JCheckBox   )) r=SELECTED;

    return r;
  }

  public static final Color getAdaptedBackground(AbstractButton b)
  {
    return intensity(getBackground(b),HUE[getStatus(b)]);
  }

  public static final Color getAdaptedForeground(AbstractButton b)
  {
    return intensity(getForeground(b),HUE[getStatus(b)]);
  }

  public static final Color getAdaptedBackground(AbstractButton b, String ui)
  {
    int   s=getStatus    (b);
    Color c=getBackground(b);
    Color r=null;
    if(c instanceof ColorUIResource)
    {
           if(s==ROLLOVER) r=SlafManager.getColor(ui+".rolloverBackground");
      else if(s==SELECTED) r=SlafManager.getColor(ui+".selectionBackground");
      else if(s==PRESSED ) r=SlafManager.getColor(ui+".pressedBackground");
      else if(s==ROLLOVER_SELECTED)
        r=SlafManager.getColor
	  (ui+".rolloverSelectedBackground",
	   SlafManager.getColor(ui+".selectionBackground"));
    }
    if(r==null) r=intensity(c,HUE[getStatus(b)]);
    return r;
  }

  public static final Color getAdaptedForeground(AbstractButton b, String ui)
  {
    int   s=getStatus(b);
    Color c=getForeground(b);
    Color r=null;
    if(c instanceof ColorUIResource)
    {
           if(s==ROLLOVER) r=SlafManager.getColor(ui+".rolloverForeground");
      else if(s==SELECTED) r=SlafManager.getColor(ui+".selectionForeground");
      else if(s==PRESSED ) r=SlafManager.getColor(ui+".pressedForeground");
      else if(s==ROLLOVER_SELECTED)
	r=SlafManager.getColor
	  (ui+".rolloverSelectedForeground",
	   SlafManager.getColor(ui+".selectionForeground"));
    }
    if(r==null) r=intensity(c,HUE[getStatus(b)]);
    return r;
  }

  // Focus

  public static final void installFocusBorder(JComponent _c,String _ui,
					      boolean _resource)
  {
    if(SlafManager.getBorder(_ui+".normalBorder")!=null)
    {
      if(!_resource||(_c.getBorder() instanceof BorderUIResource))
      {
	Border b=null;

	if(SlafManager.getBoolean(_ui+".extraBorderInside",false))
	  b=new SlafCompoundBorder
	    (_c.getBorder(),new SlafExtraBorder(_ui),true);
	else
	  b=new SlafCompoundBorder
	    (new SlafExtraBorder(_ui),_c.getBorder(),true);
	if(_resource) b=new BorderUIResource(b);
	_c.setBorder(b);
      }
      if(_c instanceof AbstractButton)
	  ((AbstractButton)_c).setFocusPainted
	    (SlafManager.getBoolean(_ui+".focusPainted",false));
    }
  }

  // Transparent

  protected static PropertyChangeListener PC_OPAQUE=
    new PropertyChangeListener()
    {
      public void propertyChange(PropertyChangeEvent _evt)
      {
	if("opaque".equals(_evt.getPropertyName()))
	{
	  JComponent c=(JComponent)_evt.getSource();
	  if(Boolean.FALSE.equals(_evt.getNewValue()))
	      c.setOpaque(true);
	}
      }
    };

  protected static PropertyChangeListener PC_TRANSPARENT=
    new PropertyChangeListener()
    {
      public void propertyChange(PropertyChangeEvent _evt)
      {
	if("opaque".equals(_evt.getPropertyName()))
	{
	  JComponent c=(JComponent)_evt.getSource();
	  if(Boolean.TRUE.equals(_evt.getNewValue()))
	      c.setOpaque(false);
	}
      }
    };

  public static void forceOpaque(JComponent _c, boolean _opaque)
  {
    if(_opaque) forceOpaque(_c);
    else        forceTransparent(_c);
  }

  public static void forceOpaque(JComponent _c)
  {
    _c.removePropertyChangeListener(PC_OPAQUE);
    _c.removePropertyChangeListener(PC_TRANSPARENT);
    _c.setOpaque(true);
    _c.addPropertyChangeListener(PC_OPAQUE);
  }

  public static void forceTransparent(JComponent _c)
  {
    _c.removePropertyChangeListener(PC_OPAQUE);
    _c.removePropertyChangeListener(PC_TRANSPARENT);
    _c.setOpaque(false);
    _c.addPropertyChangeListener(PC_TRANSPARENT);
  }

  public static void clearTransparent(JComponent _c)
  {
    _c.removePropertyChangeListener(PC_OPAQUE);
    _c.removePropertyChangeListener(PC_TRANSPARENT);
    _c.setOpaque(true);
  }

  private static void getAllSubComponents(Vector _v, Container _parent)
  {
    Component[] c=_parent.getComponents();

    for(int i=0;i<c.length;i++)
    {
      if(!_v.contains(c[i]))
	_v.addElement(c[i]);
      if(c[i] instanceof Container)
	getAllSubComponents(_v,(Container)c[i]);
    }
  }

  /**
   * Returns all subcomponents of a container.
   */
  public static Vector getAllSubComponents(Container _parent)
  {
    Vector r=new Vector();
    r.addElement(_parent);
    getAllSubComponents(r,_parent);
    return r;
  }

  /**
   *
   */
  public static void printAncestors(Component _c)
  {
    int n=0;
    while(_c!=null)
    {
      String s=n+": "+_c;
      s=s.substring(0,Math.min(78-n,s.length()));
      for(int i=0;i<n;i++) s=" "+s;
      System.err.println(s);
      _c=_c.getParent();
      n++;
    }
  }
}
